package com.ruoyi.web.controller.app;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtArticle;
import com.ruoyi.system.domain.RlFunction;
import com.ruoyi.system.domain.agent.RlAgent;
import com.ruoyi.system.domain.city.RlCity;
import com.ruoyi.system.domain.city.RlCityQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.domain.store.StoreQuery;
import com.ruoyi.system.domain.vo.PriceVO;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * app接口（不需要登录的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/app")
public class AppController extends BaseController
{

    @Resource
    private IRlCityService rlCityService;

    @Autowired
    private ISysDictDataService dictDataService;

    @Resource
    private IRlAgentService rlAgentService;

    @Autowired
    private IRlModelService modelService;

    @Autowired
    private RlStoreService storeService;

    @Autowired
    private IRlFeeRuleService rlFeeRuleService;

    @Autowired
    private IRlDeviceService eDeviceService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlArticleService asArticleService;

    @Autowired
    private IRlArticleClassifyService asArticleClassifyService;

    @Resource
    private IRlUserService userService;

    /**
     * 查询城市列表
     */
    @GetMapping("/city/list")
    public AjaxResult cityList(RlCityQuery city)
    {
        logger.info("查询城市列表：【city="+city+"】");
        List<RlCity> list = rlCityService.selectRlCityList(city);
        return success(list);
    }

    /**
     * 获取字典类型列表
     */
    @GetMapping("/getDictData")
    public AjaxResult cityList(String dictType)
    {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType(dictType);
        List<SysDictData> list = dictDataService.selectDictDataList(sysDictData);
        return success(list);
    }

    /**
     * 根据城市id获取代理商
     */
    @GetMapping("/getAgentByCityId")
    public AjaxResult getAgentByCityId(Long cityId)
    {
        logger.info("根据城市id获取代理商：【cityId="+cityId+"】");
        if(cityId == null){
            return error("城市id不能为空");
        }
        return success(rlAgentService.selectRlAgentByCityId(cityId));
    }

    /**
     * 根据代理商id获取代理商
     */
    @GetMapping("/getAgentByAgentId")
    public AjaxResult getAgentByAgentId(Long agentId)
    {
        logger.info("根据代理商id获取代理商：【agentId="+agentId+"】");
        if(agentId == null){
            return error("代理商id不能为空");
        }
        return success(rlAgentService.selectRlAgentByAgentId(agentId));
    }


    /**
     * 根据城市id获取车型列表
     */
    @GetMapping("/getModelListByCityId")
    public AjaxResult getModelListByCityId(Long cityId)
    {
        logger.info("根据城市id获取车型列表：【cityId="+cityId+"】");
        RlAgent rlAgent = rlAgentService.selectRlAgentByCityId(cityId);
        if(rlAgent!=null){
            return success(modelService.selectEModelListByAgentId(rlAgent.getAgentId()));
        }
        return error("代理商不存在");
    }

    /**
     * 根据代理商id获取车型列表
     */
    @GetMapping("/getModelListByAgentId")
    public AjaxResult getModelListByAgentId(Long agentId)
    {
        logger.info("根据代理商id获取车型列表：【agentId="+agentId+"】");
        if(agentId==null){
            return error("代理商id不能为空");
        }
        return success(modelService.selectEModelListByAgentId(agentId));
    }

    /**
     * 根据定位获取哪个城市
     */
    @GetMapping("/getCity")
    public AjaxResult getCity(String lon,String lat)
    {
        logger.info("根据定位获取哪个城市：【lon="+lon+",lat="+lat+"】");
        AjaxResult ajax = AjaxResult.success();
        RlCity city = rlCityService.getCityByLocation(lon, lat);
        ajax.put(AjaxResult.DATA_TAG, city);
        return ajax;
    }

    /**
     * 根据定位获取详细地址
     */
    @GetMapping("/getAddress")
    public AjaxResult getAddress(String lon,String lat)
    {
        logger.info("根据定位获取哪个城市：【lon="+lon+",lat="+lat+"】");
        AjaxResult ajax = AjaxResult.success();
        String city = rlCityService.getAddressByLocation(lon,lat);
        ajax.put(AjaxResult.DATA_TAG, city);
        return ajax;
    }


    /**
     * 根据店铺id获取店铺详情
     */
    @GetMapping("/getStore")
    public AjaxResult getStore(Long storeId)
    {
        logger.info("根据城市id获取车型列表：【storeId="+storeId+"】");
        return success(storeService.selectSmStoreById(storeId));
    }

    /**
     * 根据代理商id获取店铺列表
     */
    @GetMapping("/getStoreListByAgentId")
    public AjaxResult getStoreListByAgentId(Long agentId,String lon,String lat)
    {
        logger.info("根据代理商id获取店铺列表：【agentId="+agentId+",lon="+lon+",lat="+lat+"】");
        StoreQuery storeQuery = new StoreQuery();
        storeQuery.setAgentId(agentId);
        storeQuery.setPhoneLon(lon);
        storeQuery.setPhoneLat(lat);
        return success(storeService.selectSmStoreList(storeQuery));
    }

    /**
     * 根据车型id获取详情
     */
    @GetMapping("/getModelById")
    public AjaxResult getModelById(Long modelId)
    {
        logger.info("根据车型id获取详情：【modelId="+modelId+"】");
        if(modelId==null){
            return error("车型id[modelId]未传");
        }
        return success(modelService.selectEModelByModelId(modelId));
    }

    /**
     * 根据定位获取附近店铺列表
     */
    @PostMapping("/getStoreListByLocation")
    public AjaxResult getStoreListByLocation(@RequestBody StoreQuery query)
    {
        logger.info("根据定位获取附近店铺列表：【StoreQuery="+ JSON.toJSONString(query)+"】");
        if(StrUtil.isBlank(query.getPhoneLon())){
            return error("经度[phoneLon]未传");
        }
        if(StrUtil.isBlank(query.getPhoneLat())){
            return error("纬度[phoneLat]未传");
        }
        return success(storeService.getStoreListByLocation(query));
    }

    /**
     * 根据车型查询收费方式
     * 配件表
     * 配件
     */
    @GetMapping("/getRuleByModelId")
    public AjaxResult getRuleByModelId(Long modelId)
    {
        logger.info("根据车型查询收费方式：【modelId="+modelId+"】");
        if(modelId==null){
            return error("车型id[modelId]未传");
        }
        List<RlFeeRule> rlFeeRules = rlFeeRuleService.selectRlFeeRuleListByModelId(modelId);
        return success(rlFeeRules);
    }

    /**
     * 根据sn号查询车辆实时信息
     */
    @GetMapping("/getDeviceBySn")
    public AjaxResult getDeviceBySn(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("【sn查询车辆】没有sn号参数：【sn={}】",sn);
            return error("【sn查询车辆】sn不能为空");
        }
        RlDeviceVO device = eDeviceService.getLatestLocationBySn(sn);
        return success(device);
    }


    /**
     * 响铃寻车
     */
    @PostMapping("/device/ring")
    public AjaxResult ring(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        Boolean i =eDeviceService.ring(sn);
        return success(i);
    }

    /**
     * 计算距离
     */
    @GetMapping("/computeDistance")
    public AjaxResult computeDistance(String lon,String lat,Long storeId)
    {
        logger.info("计算距离：【lon="+lon+",lat="+lat+",storeId="+storeId+"】");
        //判空
        if(StrUtil.isBlank(lon)||StrUtil.isBlank(lat)||storeId==null){
            return error("参数[lon,lat,storeId]不能为空");
        }
        BigDecimal distance = storeService.computeDistance(lon,lat,storeId);
        return success(distance);
    }


    /**
     * 根据车型id获取功能
     */
    @GetMapping("/getFunctionListByModelId")
    public AjaxResult getFunctionListByModelId(Long modelId)
    {
        logger.info("根据车型id获取功能：【modelId="+modelId+"】");
        //判空
        if(modelId == null){
            return error("参数[modelId]不能为空");
        }
        List<RlFunction> functionList = modelService.getFunctionListByModelId(modelId);
        return success(functionList);
    }

    /**
     * 计算价格
     */
    @PostMapping("/calculatePrice")
    public AjaxResult calculatePrice(@RequestBody RlOrderQuery order)
    {
        logger.info("【计算价格】请求参数：order={}", JSON.toJSONString(order));
        if(order.getModelId() == null){
            return error("车型id不能为空");
        }
        if(order.getRuleId() == null){
            return error("计费规则id不能为空");
        }
        if(order.getAgentId() == null){
            return error("代理商id不能为空");
        }
        if(order.getDeliveryMethod() == null){
            return error("配送方式不能为空");
        }
        if(StrUtil.isBlank(order.getType())){
            return error("类型不能为空");
        }
        PriceVO priceVO = orderService.calculatePrice(order);
        return AjaxResult.success(priceVO);
    }

    /**
     * 查询文章列表
     */
    @GetMapping("/article/list")
    public TableDataInfo articleList(EtArticle etArticle)
    {
        startPage();
        List<EtArticle> list = asArticleService.selectAsArticleListByApp(etArticle);
        return getDataTable(list);
    }

    /**
     * 获取文章详细信息
     */
    @GetMapping(value = "/article/{articleId}")
    public AjaxResult getArticleInfo(@PathVariable("articleId") Long articleId)
    {
        return success(asArticleService.selectAsArticleByArticleId(articleId));
    }

    /**
     * 根据查询文章分类列表
     */
    @GetMapping("/classify/list")
    public AjaxResult classifyList(AsArticleClassify asArticleClassify)
    {
        logger.info("查询分类列表：【asArticleClassify="+asArticleClassify+"】");
        List<AsArticleClassify> asArticleClassifies = asArticleClassifyService.selectClassifyList(asArticleClassify);
        return success(asArticleClassifies);
    }

    /**
     * 修改用户类型
     */
    @PostMapping("/changeUserType")
    public AjaxResult changeUserType(Long userId, String userType)
    {
        RlUser user = new RlUser();
        user.setUserId(userId);
        user.setUserType(userType);
        return toAjax(userService.updateUser(user));
    }

}
