package com.ruoyi.web.controller.app;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.verify.vo.IDResponse;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.domain.channel.ChannelVO;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.modelAccessory.RlModelAccessory;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.query.AuthenticationQuery;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * app接口（需要登录校验的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/appVerify")
public class AppVerifyController extends BaseController
{

    @Resource
    private IRlUserService rlUserService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlDeviceService deviceService;

    @Autowired
    private IRlDeviceAccessoryService deviceAccessoryService;

    @Autowired
    private IRlWithdrawService rlWithdrawService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private RlChannelService channelService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private RlStoreService storeService;

    @Autowired
    private IRlModelAccessoryService modelAccessoryService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRlOrderOperService orderOperService;

    /**
     * 实名认证
     */
    @Log(title = "实名认证", businessType = BusinessType.AUTHENTICATION)
    @GetMapping("/user/authentication")
    public AjaxResult authentication(AuthenticationQuery authenticationQuery)
    {
        logger.info("【实名认证】请求参数：authenticationQuery={}", JSON.toJSON(authenticationQuery));
        Object authentication = rlUserService.authentication(authenticationQuery);
        //判断o是否是字符串
        if(authentication instanceof String){
            return AjaxResult.error((String) authentication);
        }else{
            IDResponse.Result result = (IDResponse.Result)authentication;
            if(result.getIsok()){
                //保存身份信息
                // 更新用户并更新缓存
                LoginUser loginUser = SecurityUtils.getLoginUser();
                RlUser asUser = loginUser.getUser();
                asUser.setUserId(authenticationQuery.getUserId());
                asUser.setRealName(authenticationQuery.getRealName());
                asUser.setIdCard(authenticationQuery.getIdCard());
                asUser.setIsAuthentication(true);
                asUser.setPhonenumber(authenticationQuery.getPhone());
                int updateUser = rlUserService.updateUser(asUser);
                if (updateUser>0)
                {
                    logger.info("【实名认证】保存身份信息成功");
                    // 更新缓存用户信息
                    tokenService.setLoginUser(loginUser);
                }
            }
            return AjaxResult.success(authentication);
        }
    }

    /**
     * 下单
     */
    @Log(title = "下单", businessType = BusinessType.CREATEORDER)
    @PostMapping("/createOrder")
    public AjaxResult createOrder(@RequestBody RlOrderQuery order)
    {
        logger.info("【下单】请求参数：order={}", JSON.toJSON(order));
        // 下单校验
        orderCheck(order);
        PrepayResponseVO orderServiceOrder = orderService.createOrder(order,getUserId());
        return success(orderServiceOrder);
    }

    private void orderCheck(RlOrderQuery order) {
        // 车型id不能为空
        if(order.getModelId() == null){
            throw new ServiceException("车型id不能为空");
        }
        // 配送方式不能为空
        if(order.getDeliveryMethod() == null){
            throw new ServiceException("配送方式不能为空");
        }
        // 取车时间不能为空
        if(ObjectUtil.isNull(order.getPickupTime())){
            throw new ServiceException("取车时间不能为空");
        }
        // 取车经纬度不能为空
        if(StringUtils.isEmpty(order.getPickupLon()) && StringUtils.isEmpty(order.getPickupLat()) && !order.getDeliveryMethod().equals(ServiceConstants.DELIVERY_METHOD_SELF_PICKUP)){
            throw new ServiceException("取车经度不能为空");
        }
        // ruleId不能为空
        if(order.getRuleId() == null){
            throw new ServiceException("计费规则id不能为空");
        }
        // agentId不能为空
        if(order.getAgentId() == null){
            throw new ServiceException("代理商id不能为空");
        }
        // storeId不能为空
        if(order.getStoreId() == null){
            throw new ServiceException("门店id不能为空");
        }
        // 数量num不能为空
        if(order.getNum() == null){
            throw new ServiceException("数量不能为空");
        }
        // 订单类型type不能为空
        if(order.getType() == null){
            throw new ServiceException("订单类型不能为空");
        }
    }

    /**
     * 确认支付
     */
    @Log(title = "确认支付", businessType = BusinessType.CONFIRMPAY)
    @PostMapping("/confirmPay")
    public AjaxResult confirmPay(String orderNo)
    {
        logger.info("【确认支付】请求参数：orderNo={}",orderNo);
        PrepayResponseVO orderServiceOrder = orderService.confirmPay(orderNo);
        return success(orderServiceOrder);
    }

    /**
     * 订单列表
     */
    @GetMapping("/orderList")
    public AjaxResult orderList(RlOrderQuery order)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSONString(order)+"】");
        List<RlOrderVO> rlOrderVOS = orderService.selectRlOrderList(order);
        return success(rlOrderVOS);
    }

    /**
     * 我的订单列表
     */
    @GetMapping("/myOrderList")
    public AjaxResult myOrderList(String[] statusList)
    {
        logger.info("订单列表请求：statusList=【"+JSON.toJSONString(statusList)+"】");
        RlOrderQuery order = new RlOrderQuery();
        order.setUserId(getUserId());
        if(statusList.length == 0){
            statusList = null;
        }
        order.setStatusList(statusList);
        List<RlOrderVO> rlOrderVOS = orderService.selectRlOrderList(order);
        return success(rlOrderVOS);
    }

    /**
     * 订单信息
     */
    @GetMapping("/orderInfo")
    public AjaxResult orderInfo(String orderNo)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSON(orderNo)+"】");
        RlOrderVO rlOrder = orderService.getOrderInfoByOrderNo(orderNo);
        return success(rlOrder);
    }

    /**
     * 扫码绑定车辆
     */
    @Log(title = "扫码绑定车辆", businessType = BusinessType.BANDDEVICE)
    @PostMapping("/bandDevice")
    public AjaxResult bandDevice(String orderNo, String sn)
    {
        logger.info("扫码绑定车辆：orderNo=【"+orderNo+"】,sn=【"+sn+"】");
        orderService.bandDevice(orderNo,sn);
        return success();
    }

    /**
     * 临时锁车
     */
    @Log(title = "临时锁车", businessType = BusinessType.LOCK)
    @PostMapping("/device/lock")
    public AjaxResult deviceLock(String sn,String orderNo,String isBluetooth,String lon,String lat)
    {
        logger.info("【临时锁车请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
        Boolean i =deviceService.lock(sn,orderNo,isBluetooth,lon,lat);
        return success(i);
    }

    /**
     * 临时解锁
     */
    @Log(title = "临时解锁", businessType = BusinessType.UNLOCK)
    @PostMapping("/device/unlock")
    public AjaxResult deviceUnlock(String sn,String orderNo,String isBluetooth,String lon,String lat)
    {
        logger.info("【临时解锁请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
        if(StrUtil.isBlank(orderNo)){
            throw new ServiceException("订单号为空");
        }
        Boolean i =deviceService.unlock(sn,orderNo,isBluetooth,lon,lat);
        return success(i);
    }

    /**
     * 更新车牌号
     */
    @Log(title = "更新车牌号", businessType = BusinessType.UPDATE)
    @PutMapping("/device/edit")
    public AjaxResult deviceEdit(String sn, String vehicleNum, String status)
    {
        logger.info("【更新车牌号】请求参数：sn={},vehicleNum={}", sn,vehicleNum);
        if(StrUtil.isBlank(sn)){
            return error("【更新车牌号】sn不能为空");
        }
        RlDevice rlDevice = deviceService.selectDeviceBySn(sn);

        if(rlDevice == null){
            return error("【更新车牌号】设备不存在【"+sn+"】");
        }
        if(StrUtil.isBlank(vehicleNum)){
            return error("【更新车牌号】车牌号不能为空");
        }
        RlDevice device = new RlDevice();
        device.setSn(sn);
        device.setVehicleNum(vehicleNum);
        device.setStatus(status);
        return toAjax(deviceService.updateDeviceBySn(device));
    }


    /**
     * 续租
     */
    @Log(title = "续租", businessType = BusinessType.RELET)
    @PostMapping("/relet")
    public AjaxResult relet(@RequestBody RlOrderQuery orderQuery)
    {
        logger.info("【续租】请求参数：order={}", JSON.toJSONString(orderQuery));
        orderService.validate(orderQuery);
        PrepayResponseVO responseVO =orderService.relet(orderQuery);
        return success(responseVO);
    }

    /**
     * 还车
     */
    @Log(title = "还车", businessType = BusinessType.RETURN_VEHICLE)
    @PostMapping("/returnVehicle")
    public AjaxResult returnVehicle(@RequestBody RlOrderQuery order)
    {
        logger.info("【还车】请求参数：order={}", JSON.toJSONString(order));
        Boolean aBoolean = deviceService.returnVehicle(order);
        return success(aBoolean);
    }

    /**
     * 订单取消
     */
    @Log(title = "订单取消", businessType = BusinessType.ORDERCANCEL)
    @PostMapping("/orderCancel")
    public AjaxResult orderCancel(String orderNo)
    {
        logger.info("【订单取消】请求参数：orderNo={}", orderNo);
        Boolean aBoolean = orderService.orderCancel(orderNo);
        return success(aBoolean);
    }

    /**
     * 管理员开锁
     */
    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
    @PostMapping("/admin/unlocking")
    public AjaxResult unlocking(String sn)
    {
        logger.info("【管理员开锁请求】：{}",sn);
        RlUser rlUser = getLoginUser().getUser();
        Boolean aBoolean = deviceService.unlocking(sn,rlUser.getUserName());
        return success(aBoolean);
    }

    /**
     * 管理员关锁
     */
    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
    @PostMapping("/admin/lock")
    public AjaxResult lock(String sn)
    {
        logger.info("【管理员关锁请求】：{}",sn);
        RlUser rlUser = getLoginUser().getUser();
        Boolean aBoolean = deviceService.adminLock(sn,rlUser.getUserName());
        return success(aBoolean);
    }

//    /**
//     * 选择配件  配送工单  Distribution   deliveryOrder   配送员   deliveryOrder
//     */
//    @Log(title = "选择配件", businessType = BusinessType.LOCK)
//    @PostMapping("/select/accessory")
//    public AjaxResult lock(Long deviceId,Long[] accessoryIds)
//    {
//        if(deviceId==null){
//            return error("【选择配件】车辆id不能为空");
//        }
//        if(accessoryIds==null||accessoryIds.length==0){
//            return error("【选择配件】配件id不能为空");
//        }
//        logger.info("【选择配件】请求参数：deviceId={},accessoryIds={}", deviceId,accessoryIds);
//        int i = deviceAccessoryService.deleteRlDeviceAccessoryByDeviceId(deviceId);
//        if(i>0){
//            logger.info("【删除成功】");
//            for (Long accessoryId:accessoryIds) {
//                RlDeviceAccessory rlDeviceAccessory = new RlDeviceAccessory();
//                rlDeviceAccessory.setAccessoryId(accessoryId);
//                rlDeviceAccessory.setDeviceId(deviceId);
//                deviceAccessoryService.insertRlDeviceAccessory(rlDeviceAccessory);
//            }
//            return success();
//        }
//        return error();
//    }

    /**
     * 选择配件
     */
    @Log(title = "选择配件", businessType = BusinessType.LOCK)
    @PostMapping("/select/accessory")
    public AjaxResult lock(Long modelId,Long[] accessoryIds)
    {
        if(modelId==null){
            return error("【选择配件】车型id不能为空");
        }
        if(accessoryIds==null||accessoryIds.length==0){
            return error("【选择配件】配件id不能为空");
        }
        logger.info("【选择配件】请求参数：modelId={},accessoryIds={}", modelId,accessoryIds);
        int i = modelAccessoryService.deleteRlModelAccessoryByModelId(modelId);
        if(i>0){
            logger.info("【删除成功】");
            for (Long accessoryId:accessoryIds) {
                RlModelAccessory modelAccessory = new RlModelAccessory();
                modelAccessory.setAccessoryId(accessoryId);
                modelAccessory.setModelId(modelId);
                modelAccessoryService.insertRlModelAccessory(modelAccessory);
            }
            return success();
        }
        return error();
    }

    /**
     * 车损扣款
     */
    @Log(title = "车损扣款", businessType = BusinessType.DEDUCTMONEY)
    @PostMapping("/deductMoney")
    public AjaxResult deductMoney(BigDecimal money,String orderNo)
    {
        logger.info("【车损扣款】请求参数：money={},orderNo={}", money,orderNo);

        if(money==null){
            return error("扣款金额不能为空");
        }
        if(money.compareTo(BigDecimal.ZERO)<=0){
            return error("扣款金额不能为0");
        }
        if(StrUtil.isEmpty(orderNo)){
            return error("订单号不能为空");
        }
        RlOrderVO rlOrderVO = orderService.selectRlOrderByOrderNo(orderNo);
        if(!rlOrderVO.getStatus().equals(ServiceConstants.ORDER_STATUS_TO_BE_AUDITED)){
            return error("非审核状态不能扣款");
        }
        Boolean aBoolean = orderService.deductMoney(money,orderNo);
        if(aBoolean){
            return success();
        }
        return error();
    }

    /**
     * 押金退款
     */
    @Log(title = "押金退款", businessType = BusinessType.DEPOSITREFUND)
    @PostMapping("/depositRefund")
    public AjaxResult depositRefund(String orderNo)
    {
        logger.info("【押金退款】请求参数：orderNo={}", orderNo);
        if(StrUtil.isEmpty(orderNo)){
            return error("订单号不能为空");
        }
        Boolean aBoolean = orderService.depositRefund(orderNo);
        if(aBoolean){
            return success();
        }
        return error();
    }


    /**
     * 获取提现记录详细信息
     */
    @GetMapping(value = "/withdraw/{id}")
    public AjaxResult withdrawInfo(@PathVariable("id") Long id)
    {
        return success(rlWithdrawService.selectRlWithdrawById(id));
    }

    /**
     * 账变记录  after balance  变动前  原因  rl_capital_record  change  服务费 打款方式
     */
//    @GetMapping("/myAccountInfo")
//    public AjaxResult myAccountInfo()
//    {
//
//        return AjaxResult.success();
//    }
//    /**
//     * 提现
//     * 1. 根据id获取用户
//     * 2. 判断是否是代理商，不是代理商抛异常
//     * 3. 根据用户查询代理商对象
//     * 4. 根据支付渠道查询支付渠道对象
//     * 5.
//     */
//    @PostMapping("/withdraw")
//    public AjaxResult withdraw()
//    {
//        logger.info("【提现请求】-----------------");
//        LoginUser loginUser = getLoginUser();
//        RlUser user = loginUser.getUser();
////        Integer withdraw = depositWithdraw(etOrder1);
//        return toAjax(1);
//    }

    /**
     * 保存还车方式
     */
    @Log(title = "保存还车方式", businessType = BusinessType.EDITPRICE)
    @PutMapping("/returnAddress")
    public AjaxResult returnAddress(@RequestBody RlOrderQuery order)
    {
        logger.info("【保存还车方式】：{}", JSON.toJSON(order));
        if(StrUtil.isBlank(order.getReturnMethod())){
            throw new ServiceException("还车方式不能为空");
        }
        if(order.getReturnMethod().equals(ServiceConstants.RETURN_METHOD_PICKUP) &&
                (StrUtil.isEmpty(order.getReturnLon()) || StrUtil.isEmpty(order.getReturnLat()) || StrUtil.isEmpty(order.getReturnAddress()))){
            throw new ServiceException("还车方式为上门取车，请传取车经纬度");
        }
        int i = orderService.updateRlOrderByOrderNo(order);
        return toAjax(i);
    }

    /**
     * 根据订单号查询可还车的店铺
     */
    @GetMapping("/getReturnStoreList")
    public AjaxResult getReturnStoreList(String orderNo,String lon,String lat)
    {
        logger.info("【根据订单号查询可还车的店铺】：{},lon=【{}】,lat=【{}】", JSON.toJSON(orderNo),lon,lat);
        if(StrUtil.isBlank(orderNo)){
            throw new ServiceException("订单号不能为空");
        }
        if(StrUtil.isBlank(lon) || StrUtil.isBlank(lat)){
            throw new ServiceException("经纬度不能为空");
        }
        List<StoreVo> storeVos = orderService.getReturnStoreList(orderNo,lon,lat);
        return success(storeVos);
    }


    /**
     * 改价
     */
    @Log(title = "订单改价", businessType = BusinessType.EDITPRICE)
    @PutMapping("/order/changePrice")
    public AjaxResult orderEdit(@RequestBody RlOrderQuery order)
    {
        logger.info("【订单改价信息】：{}", JSON.toJSON(order));
        //根据订单号查询订单信息
        RlOrder etOrder1 = orderService.selectRlOrderByOrderNo(order.getOrderNo());
        if(ObjectUtil.isNull(etOrder1)){
            throw new ServiceException("订单不存在");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(etOrder1.getPaid())){
            throw new ServiceException("订单已支付，不能改价");
        }
        if(!ServiceConstants.ORDER_STATUS_TO_BE_PAID.equals(etOrder1.getStatus())){
            throw new ServiceException("非待支付订单无法改价");
        }
        String outTradeNo = etOrder1.getOutTradeNo();
        ChannelVO channelVO = channelService.selectSmChannelByChannelId(etOrder1.getPayChannel());
        Boolean execute = transactionTemplate.execute(e -> {
            if(StrUtil.isNotBlank(outTradeNo)){
                /** 关闭未支付订单 */
                if(wxPayService.closeOrder(outTradeNo,channelVO)){
                    throw new ServiceException("关闭未支付订单失败");
                }
                logger.info("【订单改价】订单【{}】,有outTradeNo = 【{}】,查询订单未支付，关闭订单：{}", etOrder1.getOrderNo(),outTradeNo);
                BigDecimal payFee = BigDecimal.ZERO;
                if(ObjectUtil.isNotNull(order.getPayFee())){
                    payFee = payFee.add(order.getPayFee());
                }
                order.setPayFee(payFee);
                /** 更新订单信息 */
                int i = orderService.updateRlOrderByOrderNo(order);
                if(i == 0){
                    throw new ServiceException("【改价】更新订单信息失败");
                }
                /** 记录订单履历 */
                if(!orderOperService.recordOrderHistory(order.getOrderNo(),ServiceConstants.ORDER_OPERATION_CHANGE_PRICE,
                        etOrder1.getStatus(),order.getStatus(),etOrder1.getPayFee(),order.getPayFee(),etOrder1.getUserId(),etOrder1.getPhone(),
                        "订单改价,修改前价格："+etOrder1.getPayFee()+",修改后价格："+order.getPayFee())){
                    throw new ServiceException("【改价】更新订单信息失败");
                }
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【改价】失败");
        return toAjax(1);
    }


    /**
     * 根据sn查询正在进行中的订单
     */
    @GetMapping("/getInProgressOrder")
    public AjaxResult getInProgressOrder(String sn)
    {
        logger.info("【根据sn查询正在进行中的订单】：sn=【{}】", sn);
        if(StrUtil.isBlank(sn)){
            throw new ServiceException("【根据sn查询正在进行中的订单】sn不能为空");
        }
        RlOrderVO order = orderService.getInProgressOrder(sn);
        return success(order);
    }

}
