package com.ruoyi.web.controller.appAgent;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.system.domain.accessory.RlAccessory;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.model.RlModel;
import com.ruoyi.system.domain.model.RlModelQuery;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import com.ruoyi.system.service.store.StoreAssembler;
import com.ruoyi.system.service.store.StoreValidator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * app接口（需要登录校验的）
 *  校验
 * @author ruoyi
 */
@Slf4j
@RestController
@RequestMapping("/appAgent")
public class AppAgentController extends BaseController
{

    @Resource
    private IRlUserService rlUserService;

    @Autowired
    private SysLoginService loginService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlDeviceService deviceService;

    @Autowired
    private IRlUserExtService rlUserExtService;

    @Autowired
    private IRlModelService modelService;

    @Autowired
    private RlStoreService storeService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private StoreAssembler storeAssembler;

    @Autowired
    private IRlFeeRuleService feeRuleService;

    @Autowired
    private IRlDeliveryOrderService deliveryOrderService;

    @Resource
    private IRlAgentService rlAgentService;

    @Autowired
    private IRlAccessoryService rlAccessoryService;

    /**
     * 车辆列表
     */
    @GetMapping(value = "/vehicleList")
    public AjaxResult allVehicleInfo(RlDeviceQuery query)
    {
        logger.info("【车辆列表】请求参数：query={}", JSON.toJSONString(query));
        authorityVerify();
        List<RlDeviceVO> devices = deviceService.selectDeviceList(query);
        return success(devices);
    }

    /**
     * 订单列表
     */
    @GetMapping("/orderList")
    public AjaxResult orderList(RlOrderQuery order)
    {
        RlAgentVO rlAgentVO = rlAgentService.selectRlAgentByUserId(getUserId());
        if(ObjectUtil.isNull(rlAgentVO)){
            throw new ServiceException("根据userid【"+getUserId()+"】代理商【"+rlAgentVO+"】不存在");
        }
        order.setAgentId(rlAgentVO.getAgentId());
        logger.info("订单列表请求：order=【"+JSON.toJSONString(order)+"】");
        authorityVerify();
        List<RlOrderVO> rlOrderVOS = orderService.selectRlOrderList(order);
        return success(rlOrderVOS);
    }


    /**
     * 订单信息
     */
    @GetMapping("/orderInfo")
    public AjaxResult orderInfo(String orderNo)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSON(orderNo)+"】");
        RlOrderVO rlOrder = orderService.getOrderInfoByOrderNo(orderNo);
        return success(rlOrder);
    }

    /**
     * 新增车辆型号
     */
    @Log(title = "车辆型号", businessType = BusinessType.INSERT)
    @PostMapping(value = "/model/save")
    public AjaxResult add(@RequestBody RlModelQuery model)
    {
        authorityVerify();
        addVerify(model);
        if(model.getModelId() == null){
            return toAjax(modelService.insertEModel(model));
        }else{
            return toAjax(modelService.updateEModel(model));
        }
    }

    private void addVerify(RlModelQuery model) {
        if(model.getAgentId() == null){
            RlUserExt rlUserExt = rlUserExtService.selectRlUserExtByUserId(getUserId());
            if(rlUserExt.getAgentId() == null){
                throw new ServiceException("用户【"+getUserId()+"】未绑定代理商");
            }
            model.setAgentId(rlUserExt.getAgentId());
        }
        if(StrUtil.isBlank(model.getModel())){
            throw new ServiceException("车型名称不能为空");
        }
        if(model.getDeposit() == null){
            throw new ServiceException("押金不能为空");
        }
    }

    /**
     * 修改车辆型号
     */
    @Log(title = "车辆型号", businessType = BusinessType.UPDATE)
    @PutMapping(value = "/model/edit")
    public AjaxResult edit(@RequestBody RlModelQuery model)
    {
        authorityVerify();
        return toAjax(modelService.updateEModel(model));
    }

    /**
     * 删除车辆型号
     */
    @Log(title = "车辆型号", businessType = BusinessType.DELETE)
    @DeleteMapping("/model/{modelIds}")
    public AjaxResult remove(@PathVariable Long[] modelIds)
    {
        authorityVerify();
        for (Long modelId:modelIds) {
            RlModelQuery model = new RlModelQuery();
            model.setModelId(modelId);
            model.setDelFlag("2");
            modelService.updateEModel(model);
        }
        return toAjax(1);
    }

    /**
     * 新增收费模板
     */
    @Log(title = "收费模板", businessType = BusinessType.INSERT)
    @PostMapping("/rule/save")
    public AjaxResult addModel(@RequestBody RlFeeRule rlFeeRule)
    {
        authorityVerify();
        return toAjax(feeRuleService.insertRlFeeRule(rlFeeRule));
    }

    /**
     * 修改收费模板
     */
    @Log(title = "收费模板", businessType = BusinessType.UPDATE)
    @PutMapping("/rule/edit")
    public AjaxResult editModel(@RequestBody RlFeeRule rlFeeRule)
    {
        authorityVerify();
        return toAjax(feeRuleService.updateRlFeeRule(rlFeeRule));
    }

    /**
     * 删除收费模板
     */
    @Log(title = "收费模板", businessType = BusinessType.DELETE)
    @DeleteMapping("/rule/{ruleIds}")
    public AjaxResult removeModel(@PathVariable Long[] ruleIds)
    {
        authorityVerify();
        for (Long ruleId:ruleIds) {
            RlFeeRule rule = new RlFeeRule();
            rule.setRuleId(ruleId);
            rule.setDelFlag("2");
            feeRuleService.updateRlFeeRule(rule);
        }
        return toAjax(1);
    }

    private void authorityVerify() {
        RlUser rlUser = rlUserService.selectUserById(getUserId());
        if(!rlUser.getUserType().equals(ServiceConstants.USER_TYPE_AGENT)){
            throw new ServiceException("您没有数据的权限，请联系管理员添加权限");
        }
    }

    /**
     * 根据userId获取车型列表（代理商）
     */
    @GetMapping("/getModelListByUserId")
    public AjaxResult getModelListByUserId()
    {
        authorityVerify();
        logger.info("根据userId获取车型列表（代理商）：【userId="+getUserId()+"】");
        RlUserExt rlUserExt = rlUserExtService.selectRlUserExtByUserId(getUserId());
        return success(modelService.selectEModelListByAgentId(rlUserExt.getAgentId()));
    }

    /**
     * 根据token获取店铺列表（代理商）
     */
    @GetMapping("/getStoreListByToken")
    public AjaxResult getStoreListByToken()
    {
        authorityVerify();
        logger.info("根据token获取店铺列表（代理商）：【userId="+getUserId()+"】");
        RlUserExt rlUserExt = rlUserExtService.selectRlUserExtByUserId(getUserId());
        return success(storeService.selectSmStoreListByAgentId(rlUserExt.getAgentId()));
    }

    /**
     * 查询配件列表（代理商）
     */
    @GetMapping("/getAccessorylist")
    public AjaxResult getAccessorylist()
    {
        RlAccessory rlAccessory = new RlAccessory();
        List<RlAccessory> list = rlAccessoryService.selectRlAccessoryList(rlAccessory);
        return success(list);
    }

}
