package com.ruoyi.web.controller.appadmin;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.domain.entity.RlUserVO;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.changeBalance.RlChangeBalance;
import com.ruoyi.system.domain.changeBalance.RlChangeBalanceQuery;
import com.ruoyi.system.domain.channelWithdraw.ChannelWithdrawQuery;
import com.ruoyi.system.domain.channelWithdraw.ChannelWithdrawVO;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrderQuery;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrderVO;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.store.StoreBO;
import com.ruoyi.system.domain.store.StoreQuery;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.system.domain.withdraw.RlWithdrawQuery;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import com.ruoyi.system.service.store.StoreAssembler;
import com.ruoyi.system.service.store.StoreValidator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

/**
 * app接口（需要登录校验的）
 *  校验
 * @author ruoyi
 */
@Slf4j
@RestController
@RequestMapping("/appAdmin")
public class AppAdminController extends BaseController
{

    @Resource
    private IRlUserService rlUserService;

    @Autowired
    private SysLoginService loginService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlDeviceService deviceService;

    @Autowired
    private IRlUserExtService rlUserExtService;

    @Autowired
    private IRlModelService modelService;

    @Autowired
    private RlStoreService storeService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private StoreAssembler storeAssembler;

    @Autowired
    private IRlFeeRuleService feeRuleService;

    @Autowired
    private IRlDeliveryOrderService deliveryOrderService;

    @Resource
    private IRlAgentService rlAgentService;

    @Autowired
    private RlChannelWithdrawService etChannelWithdrawService;

    /**
     * 首页统计
     */
    @GetMapping(value = {  "/index" })
    public AjaxResult index()
    {
        AjaxResult ajax = AjaxResult.success();
        IndexAdminVo incomeVos = orderService.indexStatistics(getUserId());
        ajax.put(AjaxResult.DATA_TAG, incomeVos);
        return ajax;
    }

    /**
     * 验证码登录
     */
    @PostMapping("/appCodeLogin")
    public AjaxResult appCodeLogin(@RequestBody LoginBody loginBody) {
        AjaxResult ajax = AjaxResult.success();
        /**通过手机号找到用户名*/
        String phone = loginBody.getPhone();
        // 生成令牌
        String token = loginService.appCodeLogin(phone, loginBody.getPhoneCode(),loginBody.getPassword(), loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 微信登录
     */
    @PostMapping("/wxlogin")
    public AjaxResult wxlogin(String mobileCode,String jsCode) {
        log.info("【微信登录/wxlogin】areaId参数：【mobileCode={}】", mobileCode);
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.wxloing(mobileCode,jsCode,ServiceConstants.APPLET_TYPE_MERCHANT);
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 我的账户
     */
    @GetMapping(value = {  "/myAccountInfo" })
    public AjaxResult myAccountInfo()
    {
        AjaxResult ajax = AjaxResult.success();
        RlUserVO rlUser = rlUserService.myAccountInfoByUserId(getUserId());
        ajax.put(AjaxResult.DATA_TAG, rlUser);
        return ajax;
    }

    /**
     * 账变记录
     */
    @GetMapping(value = {  "/myChangeBalanceList" })
    public AjaxResult myChangeBalanceList(RlChangeBalanceQuery query)
    {
        log.info("【账变记录/myChangeBalanceList】参数：【query={}】", query);
        AjaxResult ajax = AjaxResult.success();
        if(ObjectUtil.isNotNull(query.getType()) && query.getType().equals("0")){
            query.setType(null);
        }
        List<RlChangeBalance> changeBalances  = rlUserService.myChangeBalanceList(query,getUserId());
        ajax.put(AjaxResult.DATA_TAG, changeBalances);
        return ajax;
    }

    /**
     * 车辆列表
     */
    @GetMapping(value = "/vehicleList")
    public AjaxResult allVehicleInfo(RlDeviceQuery query)
    {
        authorityVerify();
        logger.info("【车辆列表】请求参数：query={}", JSON.toJSONString(query));
        List<RlDeviceVO> devices = deviceService.selectDeviceList(query);
        return success(devices);
    }

    /**
     * 上传收款码
     * type:1-微信收款码，2-支付宝收款码
     */
    @Log(title = "上传收款码", businessType = BusinessType.UPDATE)
    @PostMapping("/uploadPaymentCode")
    public AjaxResult uploadPaymentCode(String collectionCode,String type)
    {
        log.info("【上传收款码】请求参数：collectionCode={}", collectionCode);
        if(collectionCode == null || collectionCode.equals("")){
            return AjaxResult.warn("收款码不能为空");
        }
        if(type == null || type.equals("")){
            return AjaxResult.warn("收款码类型不能为空");
        }
        RlUserExt userExt = new RlUserExt();
        userExt.setUserId(getUserId());
        if(type.equals("1")){
            userExt.setWxCollectionCode(collectionCode);
        }else{
            userExt.setAliCollectionCode(collectionCode);
        }
        int result = rlUserExtService.updateRlUserExt(userExt);
        return toAjax(result);
    }


    /**
     * 车辆上线
     */
    @Log(title = "车辆上线", businessType = BusinessType.ONLINE)
    @PostMapping("/device/online")
    public AjaxResult online(String sn)
    {
        logger.info("【车辆上线请求】：{}",sn);
        Boolean aBoolean = deviceService.online(sn);
        return success(aBoolean);
    }

    /**
     * 车辆下线
     */
    @Log(title = "车辆下线", businessType = BusinessType.OFFLINE)
    @PostMapping("/device/offline")
    public AjaxResult offline(String sn)
    {
        logger.info("【车辆下线请求】：{}",sn);
        Boolean aBoolean = deviceService.offline(sn, ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
        return success(aBoolean);
    }

    /**
     * 根据商户id[merchantId]获取车型列表
     */
    @GetMapping("/getModelListByMerchantId")
    public AjaxResult getModelListByMerchantId()
    {
        authorityVerify();
        logger.info("根据userid获取车型列表：【userId="+getUserId()+"】");
        return success(modelService.getModelListByMerchantId(getUserId()));
    }

    /**
     * 订单列表
     */
    @GetMapping("/orderList")
    public AjaxResult orderList(RlOrderQuery order)
    {
        authorityVerify();
        order.setMerchantId(getUserId());
        logger.info("订单列表请求：order=【"+JSON.toJSONString(order)+"】");
        List<RlOrderVO> rlOrderVOS = orderService.selectRlOrderList(order);
        return success(rlOrderVOS);
    }

    /**
     * 订单信息
     */
    @GetMapping("/orderInfo")
    public AjaxResult orderInfo(String orderNo)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSON(orderNo)+"】");
        RlOrderVO rlOrder = orderService.getOrderInfoByOrderNo(orderNo);
        return success(rlOrder);
    }

    /**
     * 退款
     */
    @Log(title = "订单退款", businessType = BusinessType.REFUND)
    @PutMapping("/order/refund")
    public AjaxResult orderRefund(@RequestBody RlOrderQuery order)
    {
        logger.info("【订单退款请求】：{}", JSON.toJSON(order));
        return toAjax(orderService.refund(order));
    }

    /**
     * 管理员提现
     */
    @Log(title = "管理员提现", businessType = BusinessType.ADMINWITHDRAW)
    @PostMapping("/admin/withdraw")
    public AjaxResult add(@RequestBody RlWithdrawQuery withdraw)
    {
        logger.info("管理员提现请求：【{}】", JSON.toJSON(withdraw));
        int i = rlUserService.adminWithdraw(withdraw);
        return toAjax(i);
    }

    /**
     * 查询提现渠道列表
     */
    @GetMapping("/withdrawList")
    public AjaxResult withdrawList(ChannelWithdrawQuery query)
    {
        List<ChannelWithdrawVO> list = etChannelWithdrawService.selectChannelWithdrawList(query);
        return success(list);
    }

    /**
     * 获取商户详细信息
     */
    @GetMapping(value = "/store/{storeId}")
    public AjaxResult getInfo(@PathVariable("storeId") Long storeId)
    {
        StoreVo store = storeService.selectSmStoreById(storeId);
        List<StoreVo> list = Collections.singletonList(store);
        storeAssembler.assembleDeviceCount(list);   // 设备数量
        storeAssembler.assembleRevenue(list);       // 经营数据
        return success(store);
    }

    /**
     * 保存店铺
     */
    @Log(title = "保存店铺", businessType = BusinessType.SAVESTORE)
    @PostMapping(value = "/store/save")
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) StoreBO store)
    {
        int result;
        Long userId = getUserId();
        store.setMerchantId(userId);
        RlUserExt rlUserExt = rlUserExtService.selectRlUserExtByUserId(userId);
        if(ObjectUtil.isNotNull(rlUserExt)){
            RlAgentVO rlAgentVO = rlAgentService.selectRlAgentByAgentId(rlUserExt.getAgentId());
            if(ObjectUtil.isNull(rlAgentVO)) throw new ServiceException("代理商【"+rlUserExt.getAgentId()+"】不存在");
            store.setAgentId(rlAgentVO.getAgentId());
            store.setServerPhone(rlAgentVO.getServicePhone());
        }else{
            throw new ServiceException("用户【"+userId+"】没有扩展信息");
        }
        if(ObjectUtil.isNull(store.getStoreId())){
            store = store.filterCreate();
            ServiceUtil.assertion(storeValidator.preCreate(store));
            result = storeService.insertSmStore(store);
        }else{
            store = store.filterUpdate();
            ServiceUtil.assertion(storeValidator.preUpdate(store));
            result = storeService.updateSmStore(store);
        }
        return toAjax(result);
    }

    /**
     * 根据token获取店铺列表
     */
    @GetMapping("/getStoreListByToken")
    public AjaxResult getStoreListByToken()
    {
        logger.info("【根据token获取店铺列表】，userid="+getUserId());
        StoreQuery storeQuery = new StoreQuery();
        storeQuery.setMerchantId(getUserId());
        return success(storeService.selectSmStoreList(storeQuery));
    }

    /**
     * 查询配送工单列表  送达时间
     */
    @GetMapping("/deliveryOrderList")
    public AjaxResult list(RlDeliveryOrderQuery rlDeliveryOrder)
    {
        if(ObjectUtil.isNull(rlDeliveryOrder.getStatusList()) || rlDeliveryOrder.getStatusList().length == 0){
            rlDeliveryOrder.setStatusList(null);
        }
        log.info("【查询配送工单列表】请求参数：rlDeliveryOrder="+JSON.toJSONString(rlDeliveryOrder));
        List<RlDeliveryOrderVO> list = deliveryOrderService.selectRlDeliveryOrderList(rlDeliveryOrder);
        return success(list);
    }

    /**
     * 获取配送工单详细信息
     */
    @GetMapping(value = "/deliveryOrderinfo/{deliveryId}")
    public AjaxResult deliveryOrderinfo(@PathVariable("deliveryId") Long deliveryId)
    {
        return success(deliveryOrderService.selectRlDeliveryOrderByDeliveryId(deliveryId));
    }

    /**
     * 立即接单
     */
    @PostMapping("/orderTaking")
    public AjaxResult orderTaking(String orderNo)
    {
        Long userId = getUserId();
        logger.info("【立即接单】，orderNo={}，userid={}",orderNo,userId);
        if(StrUtil.isBlank(orderNo)){
            throw new ServiceException("订单号不能为空");
        }
        return success(deliveryOrderService.orderTaking(orderNo,userId));
    }

    /**
     * 立即配送
     */
    @PostMapping("/inDelivery")
    public AjaxResult inDelivery(String orderNo)
    {
        Long userId = getUserId();
        logger.info("【立即配送】，orderNo={}，userid={}",orderNo,userId);
        if(StrUtil.isBlank(orderNo)){
            throw new ServiceException("订单号不能为空");
        }
        return success(deliveryOrderService.inDelivery(orderNo,userId));
    }

    /**
     * 结束配送
     */
    @PostMapping("/endDelivery")
    public AjaxResult endDelivery(String orderNo)
    {
        Long userId = getUserId();
        logger.info("【结束配送】，orderNo={}，userid={}",orderNo,userId);
        if(StrUtil.isBlank(orderNo)){
            throw new ServiceException("订单号不能为空");
        }
        return success(deliveryOrderService.endDelivery(orderNo,userId));
    }

    /**
     * 取消配送
     */
    @PostMapping("/cancelDelivery")
    public AjaxResult cancelDelivery(String orderNo)
    {
        Long userId = getUserId();
        logger.info("【取消配送】，orderNo={}，userid={}",orderNo,userId);
        if(StrUtil.isBlank(orderNo)){
            throw new ServiceException("订单号不能为空");
        }
        return success(deliveryOrderService.cancelDelivery(orderNo,userId));
    }

    private void authorityVerify() {
        RlUser rlUser = rlUserService.selectUserById(getUserId());
        if(!rlUser.getUserType().equals(ServiceConstants.USER_TYPE_MERCHANT)){
            throw new ServiceException("您没有数据的权限，请联系管理员添加权限");
        }
    }

}
