package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.accessory.RlAccessory;
import com.ruoyi.system.service.IRlAccessoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 配件Controller
 *
 * @author qzz
 * @date 2024-09-19
 */
@RestController
@RequestMapping("/system/accessory")
public class RlAccessoryController extends BaseController
{
    @Autowired
    private IRlAccessoryService rlAccessoryService;

    /**
     * 查询配件列表
     */
    @PreAuthorize("@ss.hasPermi('system:accessory:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlAccessory rlAccessory)
    {
        startPage();
        List<RlAccessory> list = rlAccessoryService.selectRlAccessoryList(rlAccessory);
        return getDataTable(list);
    }

    /**
     * 导出配件列表
     */
    @PreAuthorize("@ss.hasPermi('system:accessory:export')")
    @Log(title = "配件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlAccessory rlAccessory)
    {
        List<RlAccessory> list = rlAccessoryService.selectRlAccessoryList(rlAccessory);
        ExcelUtil<RlAccessory> util = new ExcelUtil<RlAccessory>(RlAccessory.class);
        util.exportExcel(response, list, "配件数据");
    }

    /**
     * 获取配件详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:accessory:query')")
    @GetMapping(value = "/{accessoryId}")
    public AjaxResult getInfo(@PathVariable("accessoryId") Long accessoryId)
    {
        return success(rlAccessoryService.selectRlAccessoryByAccessoryId(accessoryId));
    }

    /**
     * 新增配件
     */
    @PreAuthorize("@ss.hasPermi('system:accessory:add')")
    @Log(title = "配件", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlAccessory rlAccessory)
    {
        return toAjax(rlAccessoryService.insertRlAccessory(rlAccessory));
    }

    /**
     * 修改配件
     */
    @PreAuthorize("@ss.hasPermi('system:accessory:edit')")
    @Log(title = "配件", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlAccessory rlAccessory)
    {
        return toAjax(rlAccessoryService.updateRlAccessory(rlAccessory));
    }

    /**
     * 删除配件
     */
    @PreAuthorize("@ss.hasPermi('system:accessory:remove')")
    @Log(title = "配件", businessType = BusinessType.DELETE)
	@DeleteMapping("/{accessoryIds}")
    public AjaxResult remove(@PathVariable Long[] accessoryIds)
    {
        return toAjax(rlAccessoryService.deleteRlAccessoryByAccessoryIds(accessoryIds));
    }

    /**
     * 状态修改
     */
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody RlAccessory accessory)
    {
        accessory.setUpdateBy(getUsername());
        accessory.setUpdateTime(DateUtils.getNowDate());
        return toAjax(rlAccessoryService.updateRlAccessory(accessory));
    }
}
