package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.agent.RlAgent;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.service.IRlAgentService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 代理商Controller
 *
 * @author qzz
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/system/agent")
public class RlAgentController extends BaseController
{
    @Resource
    private IRlAgentService rlAgentService;

    /**
     * 查询代理商列表
     */
    @PreAuthorize("@ss.hasPermi('system:agent:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlAgent rlAgent)
    {
        startPage();
        List<RlAgentVO> list = rlAgentService.selectRlAgentList(rlAgent);
        return getDataTable(list);
    }

    /**
     * 导出代理商列表
     */
    @PreAuthorize("@ss.hasPermi('system:agent:export')")
    @Log(title = "代理商", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlAgent rlAgent)
    {
        List<RlAgentVO> list = rlAgentService.selectRlAgentList(rlAgent);
        ExcelUtil<RlAgentVO> util = new ExcelUtil<RlAgentVO>(RlAgentVO.class);
        util.exportExcel(response, list, "代理商数据");
    }

    /**
     * 获取代理商详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:agent:query')")
    @GetMapping(value = "/{agentId}")
    public AjaxResult getInfo(@PathVariable("agentId") Long agentId)
    {
        return success(rlAgentService.selectRlAgentByAgentId(agentId));
    }

    /**
     * 新增代理商
     */
    @PreAuthorize("@ss.hasPermi('system:agent:add')")
    @Log(title = "代理商", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlAgent rlAgent)
    {
        return toAjax(rlAgentService.insertRlAgent(rlAgent));
    }

    /**
     * 修改代理商
     */
    @PreAuthorize("@ss.hasPermi('system:agent:edit')")
    @Log(title = "代理商", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlAgent rlAgent)
    {
        return toAjax(rlAgentService.updateRlAgent(rlAgent));
    }

    /**
     * 删除代理商
     */
    @PreAuthorize("@ss.hasPermi('system:agent:remove')")
    @Log(title = "代理商", businessType = BusinessType.DELETE)
	@DeleteMapping("/{agentIds}")
    public AjaxResult remove(@PathVariable Long[] agentIds)
    {
        return toAjax(rlAgentService.deleteRlAgentByAgentIds(agentIds));
    }
}
