package com.ruoyi.web.controller.rl;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.system.domain.cancelRule.RlCancelRule;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.service.IRlCancelRuleService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 取消规则Controller
 *
 * @author qzz
 * @date 2024-09-25
 */
@RestController
@RequestMapping("/system/cancelRule")
public class RlCancelRuleController extends BaseController
{
    @Autowired
    private IRlCancelRuleService rlCancelRuleService;

    /**
     * 查询取消规则列表
     */
    @PreAuthorize("@ss.hasPermi('system:cancelRule:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlCancelRule rlCancelRule)
    {
        startPage();
        List<RlCancelRule> list = rlCancelRuleService.selectRlCancelRuleList(rlCancelRule);
        return getDataTable(list);
    }

    /**
     * 导出取消规则列表
     */
    @PreAuthorize("@ss.hasPermi('system:cancelRule:export')")
    @Log(title = "取消规则", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlCancelRule rlCancelRule)
    {
        List<RlCancelRule> list = rlCancelRuleService.selectRlCancelRuleList(rlCancelRule);
        ExcelUtil<RlCancelRule> util = new ExcelUtil<RlCancelRule>(RlCancelRule.class);
        util.exportExcel(response, list, "取消规则数据");
    }

    /**
     * 获取取消规则详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:cancelRule:query')")
    @GetMapping(value = "/{cancalId}")
    public AjaxResult getInfo(@PathVariable("cancalId") Long cancalId)
    {
        return success(rlCancelRuleService.selectRlCancelRuleByCancalId(cancalId));
    }

    /**
     * 新增取消规则
     */
    @PreAuthorize("@ss.hasPermi('system:cancelRule:add')")
    @Log(title = "取消规则", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlCancelRule rlCancelRule)
    {
        return toAjax(rlCancelRuleService.insertRlCancelRule(rlCancelRule));
    }

    /**
     * 修改取消规则
     */
    @PreAuthorize("@ss.hasPermi('system:cancelRule:edit')")
    @Log(title = "取消规则", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlCancelRule rlCancelRule)
    {
        return toAjax(rlCancelRuleService.updateRlCancelRule(rlCancelRule));
    }

    /**
     * 删除取消规则
     */
    @PreAuthorize("@ss.hasPermi('system:cancelRule:remove')")
    @Log(title = "取消规则", businessType = BusinessType.DELETE)
	@DeleteMapping("/{cancalIds}")
    public AjaxResult remove(@PathVariable Long[] cancalIds)
    {
        return toAjax(rlCancelRuleService.deleteRlCancelRuleByCancalIds(cancalIds));
    }
}
