package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.citycode.RlCityCode;
import com.ruoyi.system.service.IRlCityCodeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 城市区号Controller
 *
 * @author qzz
 * @date 2024-09-19
 */
@RestController
@RequestMapping("/system/code")
public class RlCityCodeController extends BaseController
{
    @Autowired
    private IRlCityCodeService rlCityCodeService;

    /**
     * 查询城市区号列表
     */
    @PreAuthorize("@ss.hasPermi('system:code:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlCityCode rlCityCode)
    {
        startPage();
        List<RlCityCode> list = rlCityCodeService.selectRlCityCodeList(rlCityCode);
        return getDataTable(list);
    }

    /**
     * 导出城市区号列表
     */
    @PreAuthorize("@ss.hasPermi('system:code:export')")
    @Log(title = "城市区号", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlCityCode rlCityCode)
    {
        List<RlCityCode> list = rlCityCodeService.selectRlCityCodeList(rlCityCode);
        ExcelUtil<RlCityCode> util = new ExcelUtil<RlCityCode>(RlCityCode.class);
        util.exportExcel(response, list, "城市区号数据");
    }

    /**
     * 获取城市区号详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:code:query')")
    @GetMapping(value = "/{code}")
    public AjaxResult getInfo(@PathVariable("code") String code)
    {
        return success(rlCityCodeService.selectRlCityCodeByCode(code));
    }

    /**
     * 新增城市区号
     */
    @PreAuthorize("@ss.hasPermi('system:code:add')")
    @Log(title = "城市区号", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlCityCode rlCityCode)
    {
        return toAjax(rlCityCodeService.insertRlCityCode(rlCityCode));
    }

    /**
     * 修改城市区号
     */
    @PreAuthorize("@ss.hasPermi('system:code:edit')")
    @Log(title = "城市区号", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlCityCode rlCityCode)
    {
        return toAjax(rlCityCodeService.updateRlCityCode(rlCityCode));
    }

    /**
     * 删除城市区号
     */
    @PreAuthorize("@ss.hasPermi('system:code:remove')")
    @Log(title = "城市区号", businessType = BusinessType.DELETE)
	@DeleteMapping("/{codes}")
    public AjaxResult remove(@PathVariable Long[] codes)
    {
        return toAjax(rlCityCodeService.deleteRlCityCodeByCodeIds(codes));
    }
}
