package com.ruoyi.web.controller.rl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrder;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrderQuery;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrderVO;
import com.ruoyi.system.service.IRlDeliveryOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 配送工单Controller
 *
 * @author qzz
 * @date 2024-10-10
 */
@RestController
@RequestMapping("/system/deliveryOrder")
public class RlDeliveryOrderController extends BaseController
{
    @Autowired
    private IRlDeliveryOrderService rlDeliveryOrderService;

    /**
     * 查询配送工单列表
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrder:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlDeliveryOrderQuery rlDeliveryOrder)
    {
        if(ObjectUtil.isNull(rlDeliveryOrder.getStatusList()) || rlDeliveryOrder.getStatusList().length == 0){
            rlDeliveryOrder.setStatusList(null);
        }
        startPage();
        List<RlDeliveryOrderVO> list = rlDeliveryOrderService.selectRlDeliveryOrderList(rlDeliveryOrder);
        return getDataTable(list);
    }

    /**
     * 导出配送工单列表
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrder:export')")
    @Log(title = "配送工单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlDeliveryOrder rlDeliveryOrder)
    {
        List<RlDeliveryOrderVO> list = rlDeliveryOrderService.selectRlDeliveryOrderList(rlDeliveryOrder);
        ExcelUtil<RlDeliveryOrderVO> util = new ExcelUtil<RlDeliveryOrderVO>(RlDeliveryOrderVO.class);
        util.exportExcel(response, list, "配送工单数据");
    }

    /**
     * 获取配送工单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrder:query')")
    @GetMapping(value = "/{deliveryId}")
    public AjaxResult getInfo(@PathVariable("deliveryId") Long deliveryId)
    {
        return success(rlDeliveryOrderService.selectRlDeliveryOrderByDeliveryId(deliveryId));
    }

    /**
     * 新增配送工单
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrder:add')")
    @Log(title = "配送工单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlDeliveryOrder rlDeliveryOrder)
    {
        return toAjax(rlDeliveryOrderService.insertRlDeliveryOrder(rlDeliveryOrder));
    }

    /**
     * 修改配送工单
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrder:edit')")
    @Log(title = "配送工单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlDeliveryOrder rlDeliveryOrder)
    {
        return toAjax(rlDeliveryOrderService.updateRlDeliveryOrder(rlDeliveryOrder));
    }

    /**
     * 删除配送工单
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrder:remove')")
    @Log(title = "配送工单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{deliveryIds}")
    public AjaxResult remove(@PathVariable Long[] deliveryIds)
    {
        return toAjax(rlDeliveryOrderService.deleteRlDeliveryOrderByDeliveryIds(deliveryIds));
    }
}
