package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.deliveryOrderOper.RlDeliveryOrderOper;
import com.ruoyi.system.service.IRlDeliveryOrderOperService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 配送单履历Controller
 *
 * @author qzz
 * @date 2024-10-10
 */
@RestController
@RequestMapping("/system/deliveryOrderOper")
public class RlDeliveryOrderOperController extends BaseController
{
    @Autowired
    private IRlDeliveryOrderOperService rlDeliveryOrderOperService;

    /**
     * 查询配送单履历列表
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrderOper:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        startPage();
        List<RlDeliveryOrderOper> list = rlDeliveryOrderOperService.selectRlDeliveryOrderOperList(rlDeliveryOrderOper);
        return getDataTable(list);
    }

    /**
     * 导出配送单履历列表
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrderOper:export')")
    @Log(title = "配送单履历", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        List<RlDeliveryOrderOper> list = rlDeliveryOrderOperService.selectRlDeliveryOrderOperList(rlDeliveryOrderOper);
        ExcelUtil<RlDeliveryOrderOper> util = new ExcelUtil<RlDeliveryOrderOper>(RlDeliveryOrderOper.class);
        util.exportExcel(response, list, "配送单履历数据");
    }

    /**
     * 获取配送单履历详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrderOper:query')")
    @GetMapping(value = "/{operId}")
    public AjaxResult getInfo(@PathVariable("operId") Long operId)
    {
        return success(rlDeliveryOrderOperService.selectRlDeliveryOrderOperByOperId(operId));
    }

    /**
     * 新增配送单履历
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrderOper:add')")
    @Log(title = "配送单履历", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        return toAjax(rlDeliveryOrderOperService.insertRlDeliveryOrderOper(rlDeliveryOrderOper));
    }

    /**
     * 修改配送单履历
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrderOper:edit')")
    @Log(title = "配送单履历", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        return toAjax(rlDeliveryOrderOperService.updateRlDeliveryOrderOper(rlDeliveryOrderOper));
    }

    /**
     * 删除配送单履历
     */
    @PreAuthorize("@ss.hasPermi('system:deliveryOrderOper:remove')")
    @Log(title = "配送单履历", businessType = BusinessType.DELETE)
	@DeleteMapping("/{operIds}")
    public AjaxResult remove(@PathVariable Long[] operIds)
    {
        return toAjax(rlDeliveryOrderOperService.deleteRlDeliveryOrderOperByOperIds(operIds));
    }
}
