package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.service.IRlDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 设备Controller
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
@RestController
@RequestMapping("/system/device")
public class RlDeviceController extends BaseController
{
    @Autowired
    private IRlDeviceService eDeviceService;

    /**
     * 查询设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlDevice rlDevice)
    {
        startPage();
        List<RlDevice> list = eDeviceService.selectDeviceListWithIsolate(rlDevice);
        return getDataTable(list);
    }

    /**
     * 导出设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:export')")
    @Log(title = "设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlDeviceQuery rlDevice)
    {
        List<RlDeviceVO> list = eDeviceService.selectDeviceList(rlDevice);
        ExcelUtil<RlDeviceVO> util = new ExcelUtil<RlDeviceVO>(RlDeviceVO.class);
        util.exportExcel(response, list, "设备数据");
    }

    /**
     * 获取设备详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:device:query')")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId)
    {
        return success(eDeviceService.selectDeviceByDeviceId(deviceId));
    }

    /**
     * 新增设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:add')")
    @Log(title = "设备", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlDevice RlDevice)
    {
        if(!eDeviceService.checkSNUnique(RlDevice)){
            return error("新增车辆'" + RlDevice.getDeviceName() + "'失败，SN已存在");
        }
        if(!eDeviceService.checkMACUnique(RlDevice)){
            return error("新增车辆'" + RlDevice.getDeviceName() + "'失败，MAC已存在");
        }
        return toAjax(eDeviceService.insertDevice(RlDevice));
    }

    /**
     * 修改设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:edit')")
    @Log(title = "设备", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlDevice RlDevice)
    {
        if(!eDeviceService.checkSNUnique(RlDevice)){
            return error("修改车辆'" + RlDevice.getDeviceName() + "'失败，SN已存在");
        }
        if(!eDeviceService.checkMACUnique(RlDevice)){
            return error("修改车辆'" + RlDevice.getDeviceName() + "'失败，MAC已存在");
        }
        return toAjax(eDeviceService.updateDevice2(RlDevice));
    }

    /**
     * 删除设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:remove')")
    @Log(title = "设备", businessType = BusinessType.DELETE)
	@DeleteMapping("/{deviceIds}")
    public AjaxResult remove(@PathVariable Long[] deviceIds)
    {
        return toAjax(eDeviceService.deleteDeviceByDeviceIds(deviceIds));
    }

//    /**
//     * 一键上线
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:oneClickOnline')")
//    @Log(title = "设备", businessType = BusinessType.ONLINE)
//    @PostMapping("/oneClickOnline/{deviceIds}")
//    public AjaxResult oneClickOnline(@PathVariable Long[] deviceIds)
//    {
//        return toAjax(eDeviceService.oneClickOnline(deviceIds));
//    }
//
//    /**
//     * 一键下线
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:oneClickOffline')")
//    @Log(title = "设备", businessType = BusinessType.OFFLINE)
//    @PostMapping("/oneClickOffline/{deviceIds}")
//    public AjaxResult oneClickOffline(@PathVariable Long[] deviceIds)
//    {
//        return toAjax(eDeviceService.oneClickOffline(deviceIds));
//    }
//
//
//    /**
//     * 一键出仓
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:oneClickOnline')")
//    @Log(title = "设备", businessType = BusinessType.ONLINE)
//    @PostMapping("/oneClickListing/{deviceIds}")
//    public AjaxResult oneClickListing(@PathVariable Long[] deviceIds)
//    {
//        return toAjax(eDeviceService.oneClickListing(deviceIds));
//    }
//
//
//    /**
//     * 一键入仓
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:oneClickOnline')")
//    @Log(title = "设备", businessType = BusinessType.ONLINE)
//    @PostMapping("/oneClickWarehousing/{deviceIds}")
//    public AjaxResult oneClickWarehousing(@PathVariable Long[] deviceIds)
//    {
//        return toAjax(eDeviceService.oneClickWarehousing(deviceIds));
//    }
//
//    /**
//     * 更新设备
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:refresh')")
//    @Log(title = "更新设备", businessType = BusinessType.REFRESH)
//    @PostMapping("/refreshDevice/{deviceIds}")
//    public AjaxResult refreshDevice(@PathVariable Long[] deviceIds)
//    {
//        return toAjax(eDeviceService.refreshDevice(deviceIds));
//    }
//
//    /**
//     * 响铃寻车
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:ring')")
//    @PostMapping("/device/ring")
//    public AjaxResult ring(String sn)
//    {
//        if(StrUtil.isBlank(sn)){
//            logger.info("没有sn号参数：【sn={}】",sn);
//            return error("请传sn号参数"+"【sn="+sn+"】");
//        }
//        Boolean i =eDeviceService.ring(sn);
//        return success(i);
//    }
//
//    /**
//     * 重启 reboot
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:reboot')")
//    @PostMapping("/device/reboot")
//    public AjaxResult reboot(String mac)
//    {
//        if(StrUtil.isBlank(mac)){
//            logger.info("没有sn号参数：【mac={}】",mac);
//            return error("请传sn号参数"+"【mac="+mac+"】");
//        }
//        Boolean i =eDeviceService.rebootByMac(mac);
//        return success(i);
//    }
//
//    /**
//     * 管理员开锁
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
//    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
//    @PostMapping("/admin/unlocking")
//    public AjaxResult unlocking(String sn)
//    {
//        logger.info("【管理员开锁请求】：{}",sn);
//        Boolean aBoolean =eDeviceService.unlocking(sn,getUsername());
//        return success(aBoolean);
//    }
//
//
//    /**
//     * 管理员开锁
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
//    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
//    @PostMapping("/admin/unlockingByMac")
//    public AjaxResult unlockingByMac(String mac)
//    {
//        logger.info("【管理员开锁请求】：{}",mac);
//        Boolean aBoolean =eDeviceService.unlockingByMac(mac,getUsername());
//        return success(aBoolean);
//    }
//
//    /**
//     * 管理员关锁
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
//    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
//    @PostMapping("/admin/lock")
//    public AjaxResult lock(String sn)
//    {
//        logger.info("【管理员关锁请求】：{}",sn);
//        Boolean aBoolean =eDeviceService.adminLock(sn,getUsername());
//        return success(aBoolean);
//    }
//
//    /**
//     * 管理员关锁
//     */
//    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
//    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
//    @PostMapping("/admin/lockByMac")
//    public AjaxResult lockByMac(String mac)
//    {
//        logger.info("【管理员关锁请求】：{}",mac);
//        Boolean aBoolean =eDeviceService.adminLockByMac(mac,getUsername());
//        return success(aBoolean);
//    }


}
