package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.service.IRlFeeRuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 收费模板Controller
 *
 * @author qzz
 * @date 2024-09-19
 */
@RestController
@RequestMapping("/system/rule")
public class RlFeeRuleController extends BaseController
{
    @Autowired
    private IRlFeeRuleService rlFeeRuleService;

    /**
     * 查询收费模板列表
     */
    @PreAuthorize("@ss.hasPermi('system:rule:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlFeeRule rlFeeRule)
    {
        startPage();
        List<RlFeeRule> list = rlFeeRuleService.selectRlFeeRuleList(rlFeeRule);
        return getDataTable(list);
    }

    /**
     * 导出收费模板列表
     */
    @PreAuthorize("@ss.hasPermi('system:rule:export')")
    @Log(title = "收费模板", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlFeeRule rlFeeRule)
    {
        List<RlFeeRule> list = rlFeeRuleService.selectRlFeeRuleList(rlFeeRule);
        ExcelUtil<RlFeeRule> util = new ExcelUtil<RlFeeRule>(RlFeeRule.class);
        util.exportExcel(response, list, "收费模板数据");
    }

    /**
     * 获取收费模板详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:rule:query')")
    @GetMapping(value = "/{ruleId}")
    public AjaxResult getInfo(@PathVariable("ruleId") Long ruleId)
    {
        return success(rlFeeRuleService.selectRlFeeRuleByRuleId(ruleId));
    }

    /**
     * 新增收费模板
     */
    @PreAuthorize("@ss.hasPermi('system:rule:add')")
    @Log(title = "收费模板", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlFeeRule rlFeeRule)
    {
        return toAjax(rlFeeRuleService.insertRlFeeRule(rlFeeRule));
    }

    /**
     * 修改收费模板
     */
    @PreAuthorize("@ss.hasPermi('system:rule:edit')")
    @Log(title = "收费模板", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlFeeRule rlFeeRule)
    {
        return toAjax(rlFeeRuleService.updateRlFeeRule(rlFeeRule));
    }

    /**
     * 删除收费模板
     */
    @PreAuthorize("@ss.hasPermi('system:rule:remove')")
    @Log(title = "收费模板", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ruleIds}")
    public AjaxResult remove(@PathVariable Long[] ruleIds)
    {
        return toAjax(rlFeeRuleService.deleteRlFeeRuleByRuleIds(ruleIds));
    }
}
