package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.model.RlModel;
import com.ruoyi.system.domain.model.RlModelQuery;
import com.ruoyi.system.service.IRlModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 车辆型号Controller
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@RestController
@RequestMapping("/system/model")
public class RlModelController extends BaseController
{
    @Autowired
    private IRlModelService eModelService;

    /**
     * 查询车辆型号列表
     */
    @PreAuthorize("@ss.hasPermi('system:model:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlModel rlModel)
    {
        startPage();
        List<RlModel> list = eModelService.selectEModelList(rlModel);
        return getDataTable(list);
    }

    /**
     * 导出车辆型号列表
     */
    @PreAuthorize("@ss.hasPermi('system:model:export')")
    @Log(title = "车辆型号", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlModel rlModel)
    {
        List<RlModel> list = eModelService.selectEModelList(rlModel);
        ExcelUtil<RlModel> util = new ExcelUtil<RlModel>(RlModel.class);
        util.exportExcel(response, list, "车辆型号数据");
    }

    /**
     * 获取车辆型号详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:model:query')")
    @GetMapping(value = "/{modelId}")
    public AjaxResult getInfo(@PathVariable("modelId") Long modelId)
    {
        return success(eModelService.selectEModelByModelId(modelId));
    }

    /**
     * 新增车辆型号
     */
    @PreAuthorize("@ss.hasPermi('system:model:add')")
    @Log(title = "车辆型号", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlModelQuery rlModel)
    {
        return toAjax(eModelService.insertEModel(rlModel));
    }

    /**
     * 修改车辆型号
     */
    @PreAuthorize("@ss.hasPermi('system:model:edit')")
    @Log(title = "车辆型号", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlModelQuery etModel)
    {
        return toAjax(eModelService.updateEModel(etModel));
    }

    /**
     * 删除车辆型号
     */
    @PreAuthorize("@ss.hasPermi('system:model:remove')")
    @Log(title = "车辆型号", businessType = BusinessType.DELETE)
	@DeleteMapping("/{modelIds}")
    public AjaxResult remove(@PathVariable Long[] modelIds)
    {
        return toAjax(eModelService.deleteEModelByModelIds(modelIds));
    }
}
