package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.RlMsgLog;
import com.ruoyi.system.service.IRlMsgLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 短信日志Controller
 *
 * @author qiuzhenzhen
 * @date 2024-07-24
 */
@RestController
@RequestMapping("/system/msgLog")
public class RlMsgLogController extends BaseController
{
    @Autowired
    private IRlMsgLogService etMsgLogService;

    /**
     * 查询短信日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlMsgLog rlMsgLog)
    {
        startPage();
        List<RlMsgLog> list = etMsgLogService.selectEtMsgLogList(rlMsgLog);
        return getDataTable(list);
    }

    /**
     * 导出短信日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:export')")
    @Log(title = "短信日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlMsgLog rlMsgLog)
    {
        List<RlMsgLog> list = etMsgLogService.selectEtMsgLogList(rlMsgLog);
        ExcelUtil<RlMsgLog> util = new ExcelUtil<RlMsgLog>(RlMsgLog.class);
        util.exportExcel(response, list, "短信日志数据");
    }

    /**
     * 获取短信日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etMsgLogService.selectEtMsgLogById(id));
    }

    /**
     * 新增短信日志
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:add')")
    @Log(title = "短信日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlMsgLog rlMsgLog)
    {
        return toAjax(etMsgLogService.insertEtMsgLog(rlMsgLog));
    }

    /**
     * 修改短信日志
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:edit')")
    @Log(title = "短信日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlMsgLog rlMsgLog)
    {
        return toAjax(etMsgLogService.updateEtMsgLog(rlMsgLog));
    }

    /**
     * 删除短信日志
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:remove')")
    @Log(title = "短信日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etMsgLogService.deleteEtMsgLogByIds(ids));
    }
}
