package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.RlOnlineLog;
import com.ruoyi.system.service.IRlOnlineLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 在线/离线日志Controller
 *
 * @author 邱贞招
 * @date 2024-07-13
 */
@RestController
@RequestMapping("/system/onlineLog")
public class RlOnlineLogController extends BaseController
{
    @Autowired
    private IRlOnlineLogService etOnlineLogService;

    /**
     * 查询在线/离线日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlOnlineLog rlOnlineLog)
    {
        startPage();
        List<RlOnlineLog> list = etOnlineLogService.selectEtOnlineLogList(rlOnlineLog);
        return getDataTable(list);
    }

    /**
     * 导出在线/离线日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:export')")
    @Log(title = "在线/离线日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlOnlineLog rlOnlineLog)
    {
        List<RlOnlineLog> list = etOnlineLogService.selectEtOnlineLogList(rlOnlineLog);
        ExcelUtil<RlOnlineLog> util = new ExcelUtil<RlOnlineLog>(RlOnlineLog.class);
        util.exportExcel(response, list, "在线/离线日志数据");
    }

    /**
     * 获取在线/离线日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etOnlineLogService.selectEtOnlineLogById(id));
    }

    /**
     * 新增在线/离线日志
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:add')")
    @Log(title = "在线/离线日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlOnlineLog rlOnlineLog)
    {
        return toAjax(etOnlineLogService.insertEtOnlineLog(rlOnlineLog));
    }

    /**
     * 修改在线/离线日志
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:edit')")
    @Log(title = "在线/离线日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlOnlineLog rlOnlineLog)
    {
        return toAjax(etOnlineLogService.updateEtOnlineLog(rlOnlineLog));
    }

    /**
     * 删除在线/离线日志
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:remove')")
    @Log(title = "在线/离线日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etOnlineLogService.deleteEtOnlineLogByIds(ids));
    }
}
