package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.domain.entity.RlUserVO;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.domain.vo.RlUserQuery;
import com.ruoyi.system.service.IRlAgentService;
import com.ruoyi.system.service.IRlUserExtService;
import com.ruoyi.system.service.IRlUserService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户信息
 *
 * @author ruoyi 代理商
 */
@RestController
@RequestMapping("/user/user")
public class RlUserController extends BaseController
{
    @Resource
    private IRlUserService userService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRlUserExtService userExtService;

    @Autowired
    private IRlAgentService agentService;

    @Autowired
    private ISysRoleService roleService;

    /**
     * 获取用户列表
     */
    @GetMapping("/list")
    public TableDataInfo list(RlUserQuery rlUser)
    {
        startPage();
        List<RlUserVO> list = userService.selectUserList(rlUser);
        return getDataTable(list);
    }

//    /**
//     * 根据手机号快速搜索用户列表
//     */
//    @GetMapping("/fast/search")
//    public AjaxResult fastSearch(String phonenumber)
//    {
//        List<LabelVo> list = eUserService.fastSearch(phonenumber);
//        return success(list);
//    }


//    @Log(title = "用户管理", businessType = BusinessType.IMPORT)
//    @PostMapping("/importData")
//    public AjaxResult importData(MultipartFile file, boolean updateSupport) throws Exception
//    {
//        ExcelUtil<EUser> util = new ExcelUtil<EUser>(EUser.class);
//        List<EUser> userList = util.importExcel(file.getInputStream());
//        String operName = getUsername();
//        String message = eUserService.importUser(userList, updateSupport, operName);
//        return success(message);
//    }

    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<RlUser> util = new ExcelUtil<>(RlUser.class);
        util.importTemplateExcel(response, "用户数据");
    }

    /**
     * 根据用户编号获取详细信息
     */
    @GetMapping(value = { "/", "/{userId}" })
    public AjaxResult getInfo(@PathVariable(value = "userId", required = false) Long userId)
    {
        AjaxResult ajax = AjaxResult.success();
        List<SysRole> roles = roleService.selectRoleAll();
        ajax.put("roles", RlUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        if (StringUtils.isNotNull(userId))
        {
            RlUser rlUser = userService.selectUserById(userId);
            ajax.put(AjaxResult.DATA_TAG, rlUser);
            ajax.put("roleIds", rlUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
        }
        return ajax;
    }

    /**
     * 新增商户
     */
    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody RlUserQuery user)
    {
        if (!userService.checkUserNameUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !userService.checkPhoneUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        else if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setCreateBy(getUsername());
        user.setUserType(ServiceConstants.USER_TYPE_MERCHANT);
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        Boolean execute = transactionTemplate.execute(e -> {
            int i = userService.insertUser(user);
            if(i == 0){
               throw new ServiceException("【新增用户】失败");
            }
            /** 新增用户扩展表 */
            saveUserExt(user);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【新增商户】失败");
        return toAjax(1);
    }

    private void saveUserExt(RlUserQuery user) {
        RlUserExt userExt = new RlUserExt();
        userExt.setUserId(user.getUserId());
        userExt.setDividendProportion(user.getDividendProportion().divide(BigDecimal.valueOf(100)));
        userExt.setCooperationTime(user.getCooperationTime());
        userExt.setDividendStatus(user.getDividendStatus());
        userExt.setHandlingChargeType(user.getHandlingChargeType());
        userExt.setWithdrawHandlingCharge(user.getWithdrawHandlingCharge());
        RlAgentVO agentVO = agentService.selectRlAgentByCityId(user.getCityId());
        userExt.setAgentId(agentVO.getAgentId());
        int i1 = userExtService.insertRlUserExt(userExt);
        if(i1 == 0){
            throw new ServiceException("【新增用户扩展表】失败");
        }
    }

    /**
     * 修改用户
     */
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody RlUserQuery user)
    {
        if (!userService.checkUserNameUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setUpdateBy(getUsername());
        user.setUserType(ServiceConstants.USER_TYPE_MERCHANT);
        Boolean execute = transactionTemplate.execute(e -> {
            int i = userService.updateUser(user);
            if(i == 0){
                throw new ServiceException("【修改商户】失败");
            }
            int i3 = userExtService.deleteRlUserExtByUserId(user.getUserId());
            if(i3 == 0){
                throw new ServiceException("【删除用户扩展表】失败");
            }else{
                /** 修改用户扩展表 */
                saveUserExt(user);
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【修改商户】失败");
        return toAjax(1);
    }


//    /**
//     * 绑定系统用户
//     */
//    @Log(title = "绑定系统用户", businessType = BusinessType.UPDATE)
//    @PutMapping("/bandSystemUser")
//    public AjaxResult bandSystemUser(@RequestBody EUser user)
//    {
//        return toAjax(eUserService.bandSystemUser(user));
//    }

//    /**
//     * 删除用户
//     */
//    @Log(title = "用户管理", businessType = BusinessType.DELETE)
//    @DeleteMapping("/{userIds}")
//    public AjaxResult remove(@PathVariable Long[] userIds)
//    {
//        if (ArrayUtils.contains(userIds, getUserId()))
//        {
//            return error("当前用户不能删除");
//        }
//        return toAjax(eUserService.deleteUserByIds(userIds));
//    }

//    /**
//     * 重置密码
//     */
//    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
//    @PutMapping("/resetPwd")
//    public AjaxResult resetPwd(@RequestBody EUser user)
//    {
//        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
//        user.setUpdateBy(getUsername());
//        return toAjax(eUserService.resetPwd(user));
//    }

    /**
     * 状态修改
     */
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody RlUser user)
    {
        user.setUpdateBy(getUsername());
        return toAjax(userService.updateUserStatus(user));
    }


}
