package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.service.IRlUserExtService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 用户扩展Controller
 *
 * @author qzz
 * @date 2024-10-07
 */
@RestController
@RequestMapping("/system/ext")
public class RlUserExtController extends BaseController
{
    @Autowired
    private IRlUserExtService rlUserExtService;

    /**
     * 查询用户扩展列表
     */
    @PreAuthorize("@ss.hasPermi('system:ext:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlUserExt rlUserExt)
    {
        startPage();
        List<RlUserExt> list = rlUserExtService.selectRlUserExtList(rlUserExt);
        return getDataTable(list);
    }

    /**
     * 导出用户扩展列表
     */
    @PreAuthorize("@ss.hasPermi('system:ext:export')")
    @Log(title = "用户扩展", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlUserExt rlUserExt)
    {
        List<RlUserExt> list = rlUserExtService.selectRlUserExtList(rlUserExt);
        ExcelUtil<RlUserExt> util = new ExcelUtil<RlUserExt>(RlUserExt.class);
        util.exportExcel(response, list, "用户扩展数据");
    }

    /**
     * 获取用户扩展详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:ext:query')")
    @GetMapping(value = "/{extId}")
    public AjaxResult getInfo(@PathVariable("extId") Long extId)
    {
        return success(rlUserExtService.selectRlUserExtByExtId(extId));
    }

    /**
     * 新增用户扩展
     */
    @PreAuthorize("@ss.hasPermi('system:ext:add')")
    @Log(title = "用户扩展", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlUserExt rlUserExt)
    {
        return toAjax(rlUserExtService.insertRlUserExt(rlUserExt));
    }

    /**
     * 修改用户扩展
     */
    @PreAuthorize("@ss.hasPermi('system:ext:edit')")
    @Log(title = "用户扩展", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlUserExt rlUserExt)
    {
        return toAjax(rlUserExtService.updateRlUserExt(rlUserExt));
    }

    /**
     * 删除用户扩展
     */
    @PreAuthorize("@ss.hasPermi('system:ext:remove')")
    @Log(title = "用户扩展", businessType = BusinessType.DELETE)
	@DeleteMapping("/{extIds}")
    public AjaxResult remove(@PathVariable Long[] extIds)
    {
        return toAjax(rlUserExtService.deleteRlUserExtByExtIds(extIds));
    }
}
