package com.ruoyi.web.controller.rl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.withdraw.RlWithdraw;
import com.ruoyi.system.service.IRlWithdrawService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 提现记录Controller
 *
 * @author qzz
 * @date 2024-09-28
 */
@RestController
@RequestMapping("/system/withdraw")
public class RlWithdrawController extends BaseController
{
    @Autowired
    private IRlWithdrawService rlWithdrawService;

    /**
     * 查询提现记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:withdraw:list')")
    @GetMapping("/list")
    public TableDataInfo list(RlWithdraw rlWithdraw)
    {
        startPage();
        List<RlWithdraw> list = rlWithdrawService.selectRlWithdrawList(rlWithdraw);
        return getDataTable(list);
    }

    /**
     * 导出提现记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:withdraw:export')")
    @Log(title = "提现记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RlWithdraw rlWithdraw)
    {
        List<RlWithdraw> list = rlWithdrawService.selectRlWithdrawList(rlWithdraw);
        ExcelUtil<RlWithdraw> util = new ExcelUtil<RlWithdraw>(RlWithdraw.class);
        util.exportExcel(response, list, "提现记录数据");
    }

    /**
     * 获取提现记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:withdraw:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(rlWithdrawService.selectRlWithdrawById(id));
    }

    /**
     * 新增提现记录
     */
    @PreAuthorize("@ss.hasPermi('system:withdraw:add')")
    @Log(title = "提现记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RlWithdraw rlWithdraw)
    {
        return toAjax(rlWithdrawService.insertRlWithdraw(rlWithdraw));
    }

    /**
     * 修改提现记录
     */
    @PreAuthorize("@ss.hasPermi('system:withdraw:edit')")
    @Log(title = "提现记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RlWithdraw rlWithdraw)
    {
        return toAjax(rlWithdrawService.updateRlWithdraw(rlWithdraw));
    }

    /**
     * 删除提现记录
     */
    @PreAuthorize("@ss.hasPermi('system:withdraw:remove')")
    @Log(title = "提现记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(rlWithdrawService.deleteRlWithdrawByIds(ids));
    }
}
