package com.ruoyi.system.domain.order;

import java.math.BigDecimal;
import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;

/**
 * 订单对象 rl_order
 *
 * @author qzz
 * @date 2024-09-19
 */
@Data
public class RlOrder extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 订单id */
    private Long orderId;

    /** 订单号 */
    @Excel(name = "订单号")
    private String orderNo;

    /** 微信商户系统内部的订单号,32个字符内、可包含字母, 其他说明见商户订单号 */
    @Excel(name = "微信商户系统内部的订单号,32个字符内、可包含字母, 其他说明见商户订单号")
    private String outTradeNo;

    /** 用户ID */
    @Excel(name = "用户ID")
    private Long userId;

    /** 用户名 */
    @Excel(name = "用户名")
    private String userName;

    /** 真实姓名 */
    @Excel(name = "真实姓名")
    private String realName;

    /** 手机号 */
    @Excel(name = "手机号")
    private String phone;

    /** 计费规则id */
    @Excel(name = "计费规则id")
    private Long ruleId;

    /** 设备mac */
    @Excel(name = "设备mac")
    private String deviceMac;

    /** sn码 */
    @Excel(name = "sn码")
    private String sn;

    @Excel(name = "车型id")
    private Long modelId;

    /** 车型 */
    @Excel(name = "车型")
    private String model;

    /** 支付时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "支付时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date payTime;

    /** 支付方式 */
    @Excel(name = "支付方式")
    private String payType;

    /** 支付状态 */
    @Excel(name = "支付状态")
    private String paid;

    /** 类型 */
    @Excel(name = "类型")
    private String type;

    /** 订单总金额（元） */
    @Excel(name = "订单总金额", readConverterExp = "元")
    private BigDecimal totalFee;

    /** 实际支付金额 */
    @Excel(name = "实际支付金额")
    private BigDecimal payFee;

    /** 逾期费 */
    @Excel(name = "逾期费")
    private BigDecimal overdueFee;

    /** 调度费 */
    @Excel(name = "调度费")
    private BigDecimal dispatchFee;

    /** 配送费 */
    @Excel(name = "配送费")
    private BigDecimal deliveryFee;

    /** 租赁费 */
    @Excel(name = "租赁费")
    private BigDecimal leaseFee;

    /** 押金 */
    @Excel(name = "押金")
    private BigDecimal deposit;

    /** 备注 */
    @Excel(name = "备注")
    private String mark;

    /** 订单时长 */
    @Excel(name = "订单时长")
    private Integer duration;

    /** 状态 */
    @Excel(name = "状态")
    private String status;

    /** 是否逾期 */
    @Excel(name = "是否逾期")
    private Boolean isOverdue;

    /** 还车时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "还车时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date returnTime;

    /** 是否押金抵扣 */
    @Excel(name = "是否押金抵扣")
    private String depositDeduction;

    /** 押金订单号，押金抵扣时，记录是哪个押金订单 */
    private String depositOrderNo;

    /** 扣除金额 */
    @Excel(name = "扣除金额")
    private BigDecimal deductionAmount;

    /** 使用过的sn */
    private String usedSn;

    /** 换车原因 */
    private String changeReason;

    /** 还车结算___小时后自动退押金 */
    @Excel(name = "还车结算___小时后自动退押金")
    private Integer autoRefundDeposit;

    /** 租赁单位 */
    @Excel(name = "租赁单位")
    private String rentalUnit;

    /** 手续费 */
    @Excel(name = "手续费")
    private BigDecimal handlingCharge;

    /** 服务费 */
    @Excel(name = "服务费")
    private BigDecimal platformServiceFee;

    /** 运营商分账 */
    @Excel(name = "运营商分账")
    private BigDecimal operatorDividend;

    /** 支付渠道id */
    @Excel(name = "支付渠道id")
    private Long payChannel;

    /** 配送方式 */
    @Excel(name = "配送方式")
    private Integer deliveryMethod;

    /** 取车时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "取车时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date pickupTime;

    /** 代理商id */
    @Excel(name = "代理商id")
    private Long agentId;

    /** 店铺id */
    @Excel(name = "店铺id")
    private Long storeId;

    /** 店铺名称 */
    @Excel(name = "店铺名称")
    private String storeName;

    /** 商户id */
    @Excel(name = "商户id")
    private Long merchantId;

    /** 取车城市 */
    @Excel(name = "取车城市")
    private String pickupCity;

    /** 取车地点 */
    @Excel(name = "取车地点")
    private String pickupLoc;

    /** 取车经度 */
    @Excel(name = "取车经度")
    private String pickupLon;

    /** 取车纬度 */
    @Excel(name = "取车纬度")
    private String pickupLat;

    /** 到期时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "到期时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date expiryTime;

    /** 原订单号  订单类型为续租时需要 */
    private String originalOrderNo;

    /** 数量 */
    @Excel(name = "数量")
    private Integer num;

    /** 售价 */
    @Excel(name = "售价")
    private BigDecimal price;

    /** 说明 */
    @Excel(name = "说明")
    private String explain;

    /** 骑行价格说明 */
    @Excel(name = "骑行价格说明")
    private String instructions;

    /** 超出单位 */
    @Excel(name = "超出单位")
    private String outUnit;

    /** 超出价格 */
    @Excel(name = "超出价格")
    private BigDecimal outPrice;

    /** 还车类型:type:1-正常还车；2-辅助还车 */
    private String returnType;

    /** 还车方式：1-自行前往门店；2-上门取车 */
    private String returnMethod;

    /** 还车地址*/
    private String returnAddress;

    /** 还车经度*/
    private String returnLon;

    /** 还车纬度*/
    private String returnLat;

    /** 还车店铺id*/
    private Long returnStoreId;

    /** 还车店铺名称*/
    private String returnStoreName;

    /** 系统自动取消时间*/
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "支付时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    private Date autoCancelTime;

    /** 成本 */
    private BigDecimal cost;
}
