package com.ruoyi.system.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 设备Mapper接口
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
public interface RlDeviceMapper extends BaseMapper<RlDevice>
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public RlDevice selectDeviceByDeviceId(Long deviceId);

    /**
     * 根据mac号查询设备信息
     *
     * @param mac 设备列表主键
     * @return 设备列表
     */
    public RlDevice selectDeviceByMac(String mac);

    /**
     * 根据SN查询设备信息
     *
     * @param sn 设备SN
     * @return 设备列表
     */
    public RlDeviceVO selectDeviceBySn(String sn);


    /**
     * 查询设备列表
     *
     * @param RlDevice 设备
     * @return 设备集合
     */
    public List<RlDeviceVO> selectDeviceList(RlDeviceQuery RlDevice);

    /**
     * 查询设备列表（带数据隔离）
     *
     * @param RlDevice 设备
     * @return 设备集合
     */
    public List<RlDevice> selectDeviceListWithIsolate(RlDevice RlDevice);

    /**
     * 新增设备
     *
     * @param RlDevice 设备
     * @return 结果
     */
    public int insertDevice(RlDevice RlDevice);

    /**
     * 修改设备
     *
     * @param RlDevice 设备
     * @return 结果
     */
    public int updateDevice(RlDevice RlDevice);

    /**
     * 修改设备(车型和硬件版本可为空)
     *
     * @param RlDevice 设备
     * @return 结果
     */
    public int updateDevice2(RlDevice RlDevice);

    /**
     * 根据SN修改设备
     *
     * @param RlDevice 设备
     * @return 结果
     */
    public int updateDeviceBySn(RlDevice RlDevice);

    /**
     * 删除设备
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    public int deleteDeviceByDeviceId(Long deviceId);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteDeviceByDeviceIds(Long[] deviceIds);

//    /**
//     * 一键上线
//     *
//     * @param deviceIds 需要一键上线的设备主键集合
//     * @return 结果
//     */
//    public int oneClickOnline(Long[] deviceIds);
//
//    /**
//     * 一键下线
//     *
//     * @param deviceIds 需要一键下线的设备主键集合
//     * @return 结果
//     */
//    public int oneClickOffline(Long[] deviceIds);
//
//    /**
//     * 一键入仓
//     *
//     * @param deviceIds 需要一键入仓的设备主键集合
//     * @return 结果
//     */
//    public int oneClickWarehousing(Long[] deviceIds);
//
//
//    /**
//     * 有订单车辆
//     */
//    String getInOrderDevice(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);
//
//    /**
//     * 无订单车辆
//     */
//    String getNoOrderDevice(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);
//
//    public int batchDisable(String[] sns);
//

    RlDevice checkSNUnique(String sn);

    RlDevice checkMACUnique(String mac);

    /**
     * 根据店铺id查询可租车辆数
     */
    Integer selectRentalDeviceCountByStoreId(@Param("storeId")Long storeId, @Param("modelId")Long modelId);

    /**
     * 根据店铺id查询所有车辆数
     */
    Integer selectAllDeviceCountByStoreId(Long storeId);
}
