package com.ruoyi.system.mapper;

import com.ruoyi.system.domain.model.RlModel;
import com.ruoyi.system.domain.model.RlModelVO;

import java.util.List;

/**
 * 车辆型号Mapper接口
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
public interface RlModelMapper
{
    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    public RlModelVO selectEModelByModelId(Long modelId);

    /**
     * 查询车辆型号列表
     *
     * @param etModel 车辆型号
     * @return 车辆型号集合
     */
    public List<RlModel> selectEModelList(RlModel etModel);

    /**
     * 查询车辆型号列表
     *
     * @param agentId 代理商id
     * @return 车辆型号集合
     */
    public List<RlModel> selectEModelListByAgentId(Long agentId);

    /**
     * 根据店铺id查询车辆型号列表
     *
     * @param storeId 店铺id
     * @return 车辆型号集合
     */
    public List<RlModelVO> selectEModelListByStoreId(Long storeId);

    /**
     * 新增车辆型号
     *
     * @param rlModel 车辆型号
     * @return 结果
     */
    public int insertEModel(RlModel rlModel);

    /**
     * 修改车辆型号
     *
     * @param rlModel 车辆型号
     * @return 结果
     */
    public int updateEModel(RlModel rlModel);

    /**
     * 删除车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    public int deleteEModelByModelId(Long modelId);

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEModelByModelIds(Long[] modelIds);


    int selectAllCount();

    List<RlModelVO> getModelListByMerchantId(Long merchantId);

    /**
     * 根据店铺id查询车辆型号列表ids
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    List<Long> selectEModelLongListByStoreId(Long storeId);
}
