package com.ruoyi.system.mapper;

import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 订单Mapper接口
 *
 * @author qzz
 * @date 2024-09-19
 */
public interface RlOrderMapper
{
    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOrderId(Long orderId);

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOrderNo(String orderNo);

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单集合
     */
    public List<RlOrderVO> selectRlOrderList(RlOrderQuery rlOrder);

    /**
     * 查询所有订单列表
     *
     * @param rlOrder 订单
     * @return 订单集合
     */
    public List<RlOrderVO> selectAllRlOrderList(RlOrderQuery rlOrder);

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int insertRlOrder(RlOrderQuery rlOrder);

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int updateRlOrder(RlOrder rlOrder);

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int updateRlOrderByOrderNo(RlOrder rlOrder);

    /**
     * 删除订单
     *
     * @param orderId 订单主键
     * @return 结果
     */
    public int deleteRlOrderByOrderId(Long orderId);

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteRlOrderByOrderIds(Long[] orderIds);


    /**
     * 第三方订单号
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    RlOrderVO selectRlOrderByOutTradeNo(String outTradeNo);

    /**
     * 根据sn查询正在进行中的订单
     */
    List<RlOrderVO> getInProgressOrder(String sn);

//    /**
//     * 扫码绑定车辆
//     */
//    int bandDevice(String orderNo, String sn);
}
