package com.ruoyi.system.mapper;

import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.domain.entity.RlUserVO;
import com.ruoyi.common.core.domain.vo.LabelVo;
import com.ruoyi.system.domain.vo.RlUserQuery;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 用户表 数据层
 *
 * @author ruoyi
 */
public interface RlUserMapper
{
    /**
     * 根据条件分页查询用户列表
     *
     * @param RlUser 用户信息
     * @return 用户信息集合信息
     */
    public List<RlUserVO> selectUserList(RlUserQuery RlUser);

    /**
     * 根据条件分页查询已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<RlUser> selectAllocatedList(RlUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<RlUser> selectUnallocatedList(RlUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public RlUser selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public RlUserVO selectUserById(Long userId);

    /**
     * 通过用户ID查询账户
     *
     * @param RlUserId 用户ID
     * @return 用户对象信息
     */
    public RlUserVO myAccountInfoByUserId(Long RlUserId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(RlUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(RlUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    public int updateUserAvatar(@Param("userName") String userName, @Param("avatar") String avatar);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(@Param("userName") String userName, @Param("password") String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    public int deleteUserByIds(Long[] userIds);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    public RlUser checkUserNameUnique(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public RlUser checkPhoneUnique(String phonenumber);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public RlUser checkEmailUnique(String email);

    /**
     * 通过手机号查询用户
     *
     * @param phone 手机号
     * @return 用户对象信息
     */
    public RlUser selectUserByPhone(String phone);

    /**
     * 通过用户手机号和appid查询用户
     *
     * @param phone 手机号
     * @param appid appid
     * @return 用户对象信息
     */
    public RlUser selectUserByPhoneAndAppId(@Param("phone") String phone, @Param("appid") String appid);

    RlUser selectUserByWxopenid(String openid);

    /**
     * 新增用户
     */
    String getNewUser(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 总用户
     */
    String getTotalUser(@Param("areaId") String areaId);

    /**
     * 租赁用户
     */
    String getLeaseUser(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 快速查询
     */
    List<LabelVo> fastSearch(String phonenumber);

    /**
     * 获取用户总数
     */
    Integer selectAllCount(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd);

    /**
     * 更新用户密码
     */
    int updateUserPwd(@Param("userId") Long userId, @Param("password") String password);

    /**
     * 获取用户余额
     *
     * @param userId 用户id
     * @return 结果
     */
    BigDecimal selectUserBalanceById(Long userId);

    /**
     * 更新用户余额
     */
    int changeBalance(@Param("changeAmount")BigDecimal changeAmount,@Param("userId")Long userId);
}
