package com.ruoyi.system.service;

import com.ruoyi.system.domain.changeBalance.RlChangeBalance;
import com.ruoyi.system.domain.changeBalance.RlChangeBalanceQuery;
import com.ruoyi.system.domain.order.RlOrder;

import java.math.BigDecimal;
import java.util.List;

/**
 * 余额变动Service接口
 *
 * @author qzz
 * @date 2024-09-28
 */
public interface IRlChangeBalanceService
{
    /**
     * 查询余额变动
     *
     * @param changeId 余额变动主键
     * @return 余额变动
     */
    public RlChangeBalance selectRlChangeBalanceByChangeId(Long changeId);

    /**
     * 查询余额变动列表
     *
     * @param rlChangeBalance 余额变动
     * @return 余额变动集合
     */
    public List<RlChangeBalance> selectRlChangeBalanceList(RlChangeBalance rlChangeBalance);

    /**
     * 查询余额变动列表
     *
     * @param userId 余额变动
     * @return 余额变动集合
     */
    public List<RlChangeBalance> selectRlChangeBalanceListByUserId(RlChangeBalanceQuery query, Long userId);


    /**
     * 新增余额变动
     *
     * @param rlChangeBalance 余额变动
     * @return 结果
     */
    public int insertRlChangeBalance(RlChangeBalance rlChangeBalance);

    /**
     * 修改余额变动
     *
     * @param rlChangeBalance 余额变动
     * @return 结果
     */
    public int updateRlChangeBalance(RlChangeBalance rlChangeBalance);

    /**
     * 批量删除余额变动
     *
     * @param changeIds 需要删除的余额变动主键集合
     * @return 结果
     */
    public int deleteRlChangeBalanceByChangeIds(Long[] changeIds);

    /**
     * 删除余额变动信息
     *
     * @param changeId 余额变动主键
     * @return 结果
     */
    public int deleteRlChangeBalanceByChangeId(Long changeId);

    /**
     * 根据订单信息生成账变记录
     *
     * @param orderNo 订单号
     * @param outTradeNo 第三方流水号
     * @param type 收支类型：1-收入；2-支出
     * @param busType 业务类型
     * @param payFee 支付金额
     * @param userId 用户id
     * @param userName 用户名称
     * @param phone 用户电话
     * @return 结果
     */
    public int generateChanggeBalance(String orderNo, String outTradeNo, String type, String busType, BigDecimal payFee, Long userId, String userName, String phone);
}
