package com.ruoyi.system.service;

import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrder;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrderVO;

import java.util.List;

/**
 * 配送工单Service接口
 *
 * @author qzz
 * @date 2024-10-10
 */
public interface IRlDeliveryOrderService
{
    /**
     * 查询配送工单
     *
     * @param deliveryId 配送工单主键
     * @return 配送工单
     */
    public RlDeliveryOrder selectRlDeliveryOrderByDeliveryId(Long deliveryId);

    /**
     * 查询配送工单
     *
     * @param orderNo 订单号
     * @return 配送工单
     */
    public RlDeliveryOrder selectRlDeliveryOrderByOrderNo(String orderNo);

    /**
     * 查询配送工单列表
     *
     * @param rlDeliveryOrder 配送工单
     * @return 配送工单集合
     */
    public List<RlDeliveryOrderVO> selectRlDeliveryOrderList(RlDeliveryOrder rlDeliveryOrder);

    /**
     * 新增配送工单
     *
     * @param rlDeliveryOrder 配送工单
     * @return 结果
     */
    public int insertRlDeliveryOrder(RlDeliveryOrder rlDeliveryOrder);

    /**
     * 修改配送工单
     *
     * @param rlDeliveryOrder 配送工单
     * @return 结果
     */
    public int updateRlDeliveryOrder(RlDeliveryOrder rlDeliveryOrder);

    /**
     * 批量删除配送工单
     *
     * @param deliveryIds 需要删除的配送工单主键集合
     * @return 结果
     */
    public int deleteRlDeliveryOrderByDeliveryIds(Long[] deliveryIds);

    /**
     * 删除配送工单信息
     *
     * @param deliveryId 配送工单主键
     * @return 结果
     */
    public int deleteRlDeliveryOrderByDeliveryId(Long deliveryId);

    /**
     * 立即接单
     */
    int orderTaking(String orderNo, Long userId);

    /**
     * 立即配送
     */
    int inDelivery(String orderNo, Long userId);

    /**
     * 结束配送
     */
    int endDelivery(String orderNo, Long userId);

    /**
     * 取消配送
     */
    int cancelDelivery(String orderNo, Long userId);
}
