package com.ruoyi.system.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.common.utils.onenet.DataPointRes;
import com.ruoyi.common.utils.onenet.ResponseVo;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;

import java.math.BigDecimal;
import java.util.List;

/**
 * 设备Service接口
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
public interface IRlDeviceService extends IService<RlDevice>
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public RlDevice selectDeviceByDeviceId(Long deviceId);

    /**
     * 查询设备
     *
     * @param mac 设备主键
     * @return 设备
     */
    public RlDevice selectDeviceByMac(String mac);

    /**
     * 根据SN查询设备信息
     *
     * @param sn 设备主键
     * @return 设备
     */
    public RlDeviceVO getLatestLocationBySn(String sn);


    /**
     * 根据SN查询设备信息
     *
     * @param sn 设备主键
     * @return 设备
     */
    public RlDevice selectDeviceBySn(String sn);

    /**
     * 查询设备列表
     *
     * @param device 设备
     * @return 设备集合
     */
    public List<RlDeviceVO> selectDeviceList(RlDeviceQuery device);

    /**
     * 查询设备列表
     *
     * @param device 设备
     * @return 设备集合
     */
    public List<RlDevice> selectDeviceListWithIsolate(RlDevice device);

    /**
     * 新增设备
     *
     * @param device 设备
     * @return 结果
     */
    public int insertDevice(RlDevice device);

    /**
     * 修改设备
     *
     * @param device 设备
     * @return 结果
     */
    public int updateDevice(RlDevice device);

    /**
     * 修改设备
     *
     * @param device 设备
     * @return 结果
     */
    public int updateDevice2(RlDevice device);

    /**
     * 根据SN修改设备
     *
     * @param device 设备
     * @return 结果
     */
    public int updateDeviceBySn(RlDevice device);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键集合
     * @return 结果
     */
    public int deleteDeviceByDeviceIds(Long[] deviceIds);

//    /**
//     * 一键上线
//     *
//     * @param deviceIds 需要一键上线的设备主键集合
//     * @return 结果
//     */
//    public int oneClickOnline(Long[] deviceIds);
//
//    /**
//     * 一键下线
//     *
//     * @param deviceIds 需要一键下线的设备主键集合
//     * @return 结果
//     */
//    public int oneClickOffline(Long[] deviceIds);
//
//    /**
//     * 一键出仓
//     *
//     * @param deviceIds 需要一键出仓的设备主键集合
//     * @return 结果
//     */
//    int oneClickListing(Long[] deviceIds);
//
//    /**
//     * 一键入仓
//     *
//     * @param deviceIds 需要一键入仓的设备主键集合
//     * @return 结果
//     */
//    int oneClickWarehousing(Long[] deviceIds);
//
//    /**
//     * 更新设备
//     *
//     * @param deviceIds 需要更新设备的设备主键集合
//     * @return 结果
//     */
//    int refreshDevice(Long[] deviceIds);
//
//    /**
//     * 删除设备信息
//     *
//     * @param deviceId 设备主键
//     * @return 结果
//     */
//    public int deleteDeviceByDeviceId(Long deviceId);
//
//    /**
//     * 根据定位获取运营区信息，并返回所有车辆定位
//     *
//     * @param longitude 经度
//     * @param latitude 纬度
//     * @return 结果
//     */
//    List<Device> vehicleLocalization(String longitude, String latitude);

    /**
     * 所有车辆定位
     */
//    List<RlDevice> allVehicleInfo(String powerStart, String powerEnd, String status,String sort,String areaId,String onlineStatus);

    /**
     * 查询车辆数量
     */
//    DeviceNumVo allVehicleNum(String powerStart, String powerEnd,String areaId);

    /**
     * 扫码/编号开锁骑行
     */
//    OrderResponse snSwitch(EtOrderVo orderVo);

    /**
     * 发送命令
     */
    public void sendCommand(String mac, String token,String command,String type,String orderNo,String userName);

    /**
     * 发送命令(带响应)
     */
    public ResponseVo sendCommandWithResp(String mac, String token, String command, String type, String orderNo);

    /**
     * 查询数据点
     */
    public DataPointRes historyDatapoints(String mac, String token);

    /**
     * 响铃寻车
     */
    Boolean ring(String sn);

    /**
     * 开坐垫锁
     */
    Boolean seatCushionLock(String sn);


    /**
     * 重启设备
     */
    Boolean rebootByMac(String mac);
    /**
     * 重启设备
     */
    Boolean reboot(String sn);

    /**
     * 响铃寻车
     */
    Boolean ringByMac(String mac);

    /**
     * 坐垫锁用mac
     */
    Boolean seatCushionLockByMac(String mac);

    /**
     * 临时锁车
     */
    Boolean lock(String mac,String orderNo,String isBluetooth,String lon,String lat);

    /**
     * 管理员锁车
     */
    Boolean adminLock(String sn,String userName);

    /**
     * 管理员锁车
     */
    Boolean adminLockByMac(String mac,String userName);

    /**
     * 临时解锁
     */
    Boolean unlock(String sn, String orderNo,String isBluetooth,String lon,String lat);
//
//    /**
//     * 车辆预约
//     */
//    OrderResponse deviceAppointment(EtOrderVo appointmentVo);
//
//    /**
//     * 取消预约
//     */
//    OrderResponse cancelAppointment(EtOrderVo appointmentVo, EDevice EDevice);
//
    /**
     * 还车
     */
    Boolean returnVehicle(RlOrderQuery order);

    /**
     * 管理员开锁
     */
    Boolean unlocking(String sn,String userName);

    /**
     * 管理员开锁
     */
    Boolean unlockingByMac(String mac,String userName);

    /**
     * 车辆上线
     */
    Boolean online(String sn);

    /**
     * 车辆下线
     */
    Boolean offline(String sn,String status);

    /**
     * 判断是否在店铺附近
     */
    Boolean isNearStore(RlOrderQuery orderQuery, RlDevice device);
//
//    /**
//     * 根据条件模糊查询车辆信息
//     * type:1-sn,2-mac,3-车牌号
//     */
//    List<EDevice> deviceSearch(String key,String type,String areaId);
//
//    /**
//     * 运营中车辆
//     */
//    String getInOperationDevice(String timeStart, String timeEnd,String areaId);
//
//    /**
//     * 有订单车辆
//     */
//    String getInOrderDevice(String timeStart, String timeEnd,String areaId);
//
//    /**
//     * 无订单车辆
//     */
//    String getNoOrderDevice(String timeStart, String timeEnd,String areaId);
//
    /**
     * 根据车辆型号查询车辆数量
     */
    Integer selectCountByModelId(Long modelId);

//    /**
//     * 根据运营区查询车辆数量
//     */
//    Integer selectCountByAreaId(Long areaId);
//
//    /**
//     * 判断是否在运营区
//     */
//    public Boolean isAreaZone(String sn, EtOperatingArea area);
//
//    /**
//     * 判断是否在禁行区内
//     */
//    public boolean isNoRidingArea(String sn,Long areaId);
//
//    /**
//     * 判断是否在禁行区内
//     */
//    public boolean isNoRidingAreaWithTolerance(String sn,Long areaId,int tolerance);
//
//    /**
//     * 低电量不得骑行判断
//     */
//    boolean isLowBattery(String sn);
//
//    /**
//     * 判断是否在运营时间内
//     */
//    Boolean isOperatingTime(String sn);
//
//    /**
//     * 根据运营区id获取所有车辆
//     */
//    List<EDevice> allVehicleByArea(Long areaId);


    int updateLocation(RlDevice device);

    /**
     * 判断是否在线
     */
    boolean isOnline(String sn);

    /**
     * 判断是否重复
     */
    boolean checkSNUnique(RlDevice device);

    /**
     * 判断是否重复
     */
    boolean checkMACUnique(RlDevice device);


    /**
     * sn和mac号绑定
     */
    int bandSn(RlDevice device);

    /**
     * 根据mac号判断是否有绑定过
     */
    boolean isBandByMac(String mac);
//
//    /**
//     * 判断是否在停车区内
//     */
//    IsInParkingAreaVo isInParkingArea(String longitude, String latitude,String areaId,String sn);
//
    /**
     * 查询版本并更新
     */
    boolean updateVersion(String sn);

    /**
     * 根据店铺id查询可租车辆数
     */
    Integer selectRentalDeviceCountByStoreId(Long storeId,Long modelId);

    /**
     * 根据店铺id查询所有车辆数
     */
    Integer selectAllDeviceCountByStoreId(Long storeId);


//    /**
//     * 是否靠近运营区边界
//     */
//    boolean isCloseToTheBoundary(String sn, EtOperatingArea area);

    /** 计算逾期费用 */
    public BigDecimal computeOverdueFee(RlOrderVO order, RlOrder updateOrder);
}
