package com.ruoyi.system.service;

import com.ruoyi.system.domain.dividendDetail.RlDividendDetail;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetailVO;
import com.ruoyi.system.domain.order.RlOrder;

import java.math.BigDecimal;
import java.util.List;

/**
 * 分账明细Service接口
 *
 * @author qzz
 * @date 2024-10-10
 */
public interface IRlDividendDetailService
{
    /**
     * 查询分账明细
     *
     * @param id 分账明细主键
     * @return 分账明细
     */
    public RlDividendDetail selectRlDividendDetailById(Long id);

    /**
     * 查询分账明细列表
     *
     * @param rlDividendDetail 分账明细
     * @return 分账明细集合
     */
    public List<RlDividendDetailVO> selectRlDividendDetailList(RlDividendDetail rlDividendDetail);

    /**
     * 新增分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    public int insertRlDividendDetail(RlDividendDetail rlDividendDetail);

    /**
     * 修改分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    public int updateRlDividendDetail(RlDividendDetail rlDividendDetail);

    /**
     * 批量删除分账明细
     *
     * @param ids 需要删除的分账明细主键集合
     * @return 结果
     */
    public int deleteRlDividendDetailByIds(Long[] ids);

    /**
     * 删除分账明细信息
     *
     * @param id 分账明细主键
     * @return 结果
     */
    public int deleteRlDividendDetailById(Long id);

    /**
     * 记录分成比例
     *
     * @param originalOrder 原始订单对象
     * @param userType 用户类型：商户，代理商
     * @return 结果
     */
    public int calculationDividend(RlOrder originalOrder,String userType);

    /**
     * 计算分成金额
     *
     * @param rlDividendDetails 分成明细列表
     * @param totalDividendAmount 分账总金额
     * @return 结果
     */
    public int calculationAmount(List<RlDividendDetailVO> rlDividendDetails, BigDecimal totalDividendAmount);

    /**
     * 更新退款金额
     *
     * @param id 分成明细id
     * @param userRefundAmount 用户退款金额
     * @return 结果
     */
    public int updateRefundAmount(Long id, BigDecimal userRefundAmount);
}
