package com.ruoyi.system.service;

import com.ruoyi.system.domain.RlFunction;
import com.ruoyi.system.domain.model.RlModel;
import com.ruoyi.system.domain.model.RlModelQuery;
import com.ruoyi.system.domain.model.RlModelVO;

import java.util.List;

/**
 * 车辆型号Service接口
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
public interface IRlModelService
{
    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    public RlModelVO selectEModelByModelId(Long modelId);

    /**
     * 查询车辆型号列表
     *
     * @param etModel 车辆型号
     * @return 车辆型号集合
     */
    public List<RlModel> selectEModelList(RlModel etModel);

    /**
     * 根据代理商id查询车辆型号列表
     *
     * @param agentId 代理商id
     * @return 车辆型号集合
     */
    public List<RlModel> selectEModelListByAgentId(Long agentId);

    /**
     * 根据商户id查询车辆型号列表
     *
     * @param merchantId 商户id
     * @return 车辆型号集合
     */
    public List<RlModelVO> getModelListByMerchantId(Long merchantId);


    /**
     * 根据店铺id查询车辆型号列表
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    public List<RlModelVO> selectEModelListByStoreId(Long storeId);

    /**
     * 根据店铺id查询车辆型号列表ids
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    public List<Long> selectEModelLongListByStoreId(Long storeId);

    /**
     * 新增车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    public int insertEModel(RlModelQuery etModel);

    /**
     * 修改车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    public int updateEModel(RlModelQuery etModel);

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的车辆型号主键集合
     * @return 结果
     */
    public int deleteEModelByModelIds(Long[] modelIds);

    /**
     * 删除车辆型号信息
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    public int deleteEModelByModelId(Long modelId);

    /**
     * 查询所有车辆型号数
     * @return
     */
    public int selectAllCount();

    /**
     * 根据车型id获取功能
     */
    List<RlFunction> getFunctionListByModelId(Long modelId);
}
