package com.ruoyi.system.service;

import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.refund.RlRefund;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.system.domain.vo.PriceVO;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 订单Service接口
 *
 * @author qzz
 * @date 2024-09-19
 */
public interface IRlOrderService
{
    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOrderId(Long orderId);

    /**
     * 查询订单
     *
     * @param orderNo 订单主键
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOrderNo(String orderNo);

    /**
     * 查询订单
     *
     * @param outTradeNo 第三方订单号
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOutTradeNo(String outTradeNo);

    /**
     * 查询订单详情，详细信息
     *
     * @param orderNo 订单主键
     * @return 订单
     */
    public RlOrderVO getOrderInfoByOrderNo(String orderNo);

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单集合
     */
    public List<RlOrderVO> selectRlOrderList(RlOrderQuery rlOrder);

    /**
     * 查询所有订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    public List<RlOrderVO> selectAllRlOrderList(RlOrderQuery rlOrder);

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int insertRlOrder(RlOrderQuery rlOrder);

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int updateRlOrder(RlOrder rlOrder);

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int updateRlOrderByOrderNo(RlOrder rlOrder);

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键集合
     * @return 结果
     */
    public int deleteRlOrderByOrderIds(Long[] orderIds);

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    public int deleteRlOrderByOrderId(Long orderId);

    /**
     * 下单
     */
    PrepayResponseVO createOrder(RlOrderQuery order,Long userId);

    /**
     * 确认支付
     */
    PrepayResponseVO confirmPay(String orderNo);

    /**
     * 扫码绑定车辆
     */
    int bandDevice(String orderNo, String sn);

    /**
     * 续租
     */
    PrepayResponseVO relet(RlOrderQuery order);

    /**
     * 订单取消
     */
    Boolean orderCancel(String orderNo);

    /**
     * 车损扣款
     */
    Boolean deductMoney(BigDecimal money, String orderNo);

    /**
     * 押金退款
     */
    Boolean depositRefund(String orderNo);

    /**
     * 订单退款
     */
    int refund(RlOrderQuery order);

    /**
     * 创建退款
     *
     */
    public RlRefund createRefund(String reason, Long userId,String orderNo,
                                 BigDecimal refundAmount,
                                 BigDecimal dispatchFee,
                                 BigDecimal deliveryFee,
                                 BigDecimal leaseFee, String outRefundNo, String type);



    /**
     * 计算价格
     */
    PriceVO calculatePrice(RlOrderQuery order);

    /**
     * 根据订单号查询可还车的店铺
     */
    List<StoreVo> getReturnStoreList(String orderNo,String lon,String lat);

    /**
     * 根据sn查询正在进行中的订单
     */
    RlOrderVO getInProgressOrder(String sn);


    void validate(RlOrderQuery orderQuery);

    /**
     * app首页统计
     */
    IndexAdminVo indexStatistics(Long userId);
}
