package com.ruoyi.system.service;

import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.refund.RlRefund;
import com.ruoyi.system.domain.refund.RlRefundVO;

import java.util.List;

/**
 * 退款订单Service接口
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
public interface IRlRefundService
{
    /**
     * 查询退款订单
     *
     * @param id 退款订单主键
     * @return 退款订单
     */
    public RlRefund selectEtRefundById(Long id);

    /**
     * 根据订单号查询退款订单对象
     *
     * @param orderNo 订单号
     * @return 退款订单
     */
    public List<RlRefund> selectEtRefundByOrderNo(String orderNo);

    /**
     * 根据退款单号查询退款订单对象
     *
     * @param refundNo 订单号
     * @return 退款订单
     */
    public RlRefund selectEtRefundByRefundNo(String refundNo);

    /**
     * 查询退款订单列表
     *
     * @param rlRefund 退款订单
     * @return 退款订单集合
     */
    public List<RlRefund> selectEtRefundList(RlRefund rlRefund);

    /**
     * 查询退款订单列表（带数据隔离）
     *
     * @param rlRefund 退款订单
     * @return 退款订单集合
     */
    public List<RlRefund> selectEtRefundListWithIsolate(RlRefund rlRefund);

    /**
     * 新增退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    public int insertEtRefund(RlRefund rlRefund);

    /**
     * 修改退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    public int updateEtRefund(RlRefund rlRefund);

    /**
     * 批量删除退款订单
     *
     * @param ids 需要删除的退款订单主键集合
     * @return 结果
     */
    public int deleteEtRefundByIds(Long[] ids);

    /**
     * 删除退款订单信息
     *
     * @param id 退款订单主键
     * @return 结果
     */
    public int deleteEtRefundById(Long id);

    /**
     * 根据退款单号修改退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    public int updateEtRefundByRefundNo(RlRefund rlRefund);

    /**
     * 判断是否用户是否还有未退款的押金
     *
     * @param userId 用户id
     * @return 结果
     */
    List<RlOrder> checkUserDeposit(Long userId);

    /**
     * 统计已退款金额
     *
     * @param orderNo 订单号
     * @return 结果
     */
    RlRefundVO getRefundedAmount(String orderNo);
}
