package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.system.domain.EtCallbackLog;
import com.ruoyi.system.domain.channel.ChannelVO;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrder;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetail;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetailVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.refund.RlRefund;
import com.ruoyi.system.mapper.EtCallbackLogMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;


/**
 * 订单支付回调 CallbackService 实现类
 */
@Service
public class CallbackServiceImpl implements CallbackService {

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Resource
    private IRlOrderService orderService;

    @Resource
    private IRlRefundService etRefundService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Resource
    private EtCallbackLogMapper callbackLogMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private RlChannelService rlChannelService;

    @Autowired
    private IRlUserService userService;

    @Autowired
    private IRlChangeBalanceService changeBalanceService;

    @Autowired
    private IRlDeliveryOrderService deliveryOrderService;

    @Resource
    private IRlDividendDetailService dividendDetailService;

    @Resource
    private IRlUserExtService userExtService;

    @Resource
    private IRlAgentService agentService;

    @Autowired
    private IRlOrderOperService orderOperService;


    /**
     * 微信支付回调
     */
    @Override
    @Transactional
    public void weChat(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信支付回调】接收对象（未验签） : " + JSON.toJSONString(body));
        EtCallbackLog etCallbackLog = new EtCallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("1");
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);

        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);

        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功后的业务处理
                etCallbackLog.setBody(JSON.toJSONString(transaction));
                businessHandle(transaction.getOutTradeNo(), ServiceConstants.PAY_TYPE_WX);//业务处理
            }
        }
    }

    /**
     * 支付成功后的业务处理
     *  1. 更改订单状态，订单到期时间修改，账变记录保存至订单
     *  2. 生成配送工单
     *  3. 生成账变记录
     */
    @Override
    public void businessHandle(String outTradeNo,String payType) {
        // 充值成功后的业务处理  原订单
        RlOrder originalOrder = orderService.selectRlOrderByOutTradeNo(outTradeNo);
        if(originalOrder.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){
            return;
        }
        RlOrder updateOrder = new RlOrder();
        updateOrder.setOrderId(originalOrder.getOrderId());
        logger.info("【微信支付回调】订单信息 : " + JSON.toJSONString(originalOrder));
        Long payChannel = originalOrder.getPayChannel();
        ChannelVO channelVO = rlChannelService.selectSmChannelByChannelId(payChannel);

        logger.info("【微信支付回调】租赁订单支付");

        Boolean execute = transactionTemplate.execute(e -> {
            /** 获取到代理商 生成账变记录  为什么要账变？*/
//            RlAgentVO rlAgentVO = agentService.selectRlAgentByAgentId(originalOrder.getAgentId());
//            RlUser user = userService.selectUserById(rlAgentVO.getUserid());
//            if(ObjectUtil.isNull(user)){
//                logger.info("【微信支付回调】代理商【{}】没有绑定用户",rlAgentVO.getName());
//                throw new ServiceException("【微信支付回调】代理商【{}】没有绑定用户");
//            }
//            changeBalanceService.generateChanggeBalance(originalOrder.getOrderNo(), outTradeNo, ServiceConstants.FLOW_TYPE_INCOME,
//                    ServiceConstants.RIDE_ORDER, originalOrder.getPayFee(), user.getUserId(), user.getUserName(), user.getPhonenumber());
            updateOrder.setPaid("1");
            updateOrder.setPayTime(DateUtils.getNowDate());
            updateOrder.setPayType(payType);
            if(originalOrder.getDeliveryMethod().equals(ServiceConstants.DELIVERY_METHOD_SELF_PICKUP)){
                updateOrder.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_TAKEN);
            }else{
                updateOrder.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_SENT);
            }
            updateOrder.setCost(getCost(channelVO,originalOrder.getPayFee()));

            /** 如果是续租的订单，结束原订单*/
            if(originalOrder.getType().equals(ServiceConstants.ORDER_TYPE_RELET)){
                closeOriginalOrder(originalOrder, updateOrder);
            }else{
                // 如果是免费送取车或收费送取车则创建新的配送工单
                if(originalOrder.getDeliveryMethod().equals(ServiceConstants.DELIVERY_METHOD_FREE_DELIVERY_CAR) || originalOrder.getDeliveryMethod().equals(ServiceConstants.DELIVERY_METHOD_CHARGE_DELIVERY_CAR)){
                    /** 创建配送工单，状态为：待接单 */
                    createDeliveryOrder(originalOrder, updateOrder);
                }
            }
            logger.info("=================【微信支付回调】开始更新订单信息=================={}",JSON.toJSON(updateOrder));
            int updateEtOrder = orderService.updateRlOrder(updateOrder);
            if(updateEtOrder==0){
                logger.error("【微信支付回调】更新订单信息失败");
                throw new ServiceException("【微信支付回调】更新订单信息失败");
            }
            /** 计算分成金额更新分成状态 */
            BigDecimal totalDividendAmount = originalOrder.getLeaseFee().add(originalOrder.getOverdueFee());
            RlDividendDetail rlDividendDetail = new RlDividendDetail();
            rlDividendDetail.setOrderNo(originalOrder.getOrderNo());
            List<RlDividendDetailVO> rlDividendDetailVOS = dividendDetailService.selectRlDividendDetailList(rlDividendDetail);
            dividendDetailService.calculationAmount(rlDividendDetailVOS,totalDividendAmount);

            // 批量生成账变记录
            generateChangeBalance(rlDividendDetailVOS,originalOrder);

            /** 记录订单履历 */
            if(!orderOperService.recordOrderHistory(originalOrder.getOrderNo(),ServiceConstants.ORDER_OPERATION_PAY,
                    originalOrder.getStatus(),updateOrder.getStatus(),BigDecimal.ZERO,originalOrder.getPayFee(),originalOrder.getUserId(),originalOrder.getPhone(),"订单支付：已支付 "+originalOrder.getPayFee()+"元")){
                throw new ServiceException("【订单履历-支付】更新订单信息失败");
            }
            logger.info("=================【微信支付回调】全部结束！！！！！==================");
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("微信支付回调失败");
    }


    private void generateChangeBalance(List<RlDividendDetailVO> rlDividendDetailVOS, RlOrder originalOrder) {
        for(RlDividendDetailVO dividendDetail:rlDividendDetailVOS){
            String busType;
            if(originalOrder.getType().equals(ServiceConstants.ORDER_TYPE_LEASE)){
                busType = ServiceConstants.RIDE_ORDER;
            }else{
                busType = ServiceConstants.RENEW_ORDER;
            }
            // 用户生成账变
            int i = changeBalanceService.generateChanggeBalance(originalOrder.getOrderNo(), originalOrder.getOutTradeNo(), ServiceConstants.FLOW_TYPE_INCOME,
                    busType, dividendDetail.getDividendAmount(), dividendDetail.getPartnerId(), dividendDetail.getPartnerName(), dividendDetail.getPartnerPhone());
            if(i==0){
                throw new ServiceException("【微信支付回调】用户【"+dividendDetail.getPartnerName()+"】生成账变失败");
            }
        }
    }


//    private List<RlUserVO> getPartnerList(Long merchantId) {
//        RlUserQuery rlUserQuery = new RlUserQuery();
//        String[] typeList = {"02","03"};
//        rlUserQuery.setTypeList(typeList);
//        rlUserQuery.setAgentId(agentId);
//        List<RlUserVO> list = eUserService.selectUserList(rlUserQuery);
//        return list;
//    }

    /** 创建配送工单 */
    private void createDeliveryOrder(RlOrder order, RlOrder updateOrder) {
        RlDeliveryOrder deliveryOrder = new RlDeliveryOrder();
        deliveryOrder.setStatus(ServiceConstants.DELIVERY_ORDER_STATUS_PENDING);
        deliveryOrder.setOrderNo(order.getOrderNo());
        deliveryOrder.setAgentId(order.getAgentId());
        deliveryOrder.setCreateTime(DateUtils.getNowDate());
        deliveryOrder.setStoreId(order.getStoreId());
        deliveryOrder.setStoreName(order.getStoreName());
        deliveryOrder.setDeliveryAddress(order.getPickupLoc());
        deliveryOrder.setDeliveryLon(order.getPickupLon());
        deliveryOrder.setDeliveryLat(order.getPickupLat());
        deliveryOrder.setDeliveryTime(order.getPickupTime());
        int i = deliveryOrderService.insertRlDeliveryOrder(deliveryOrder);
        if(i>0){
            logger.info("创建配送工单成功");
        }
    }

    /** 结算原订单 */
    private void closeOriginalOrder(RlOrder order, RlOrder updateOrder) {
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_RELET)){
            String originalOrderNo = order.getOriginalOrderNo();
            RlOrder originalOrder = orderService.selectRlOrderByOrderNo(originalOrderNo);
            RlOrder updateOriginalOrder = new RlOrder();
            updateOriginalOrder.setOrderId(originalOrder.getOrderId());
            updateOriginalOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
            int i = orderService.updateRlOrder(updateOriginalOrder);
            if(i==1){
                logger.error("【微信支付回调】更新原订单状态成功");
            }
            updateOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
        }
    }




    private BigDecimal getCost(ChannelVO channel,BigDecimal payFee) {
        // 获取到渠道成本
        BigDecimal costRate = channel.getCostRate();
        BigDecimal bigDecimal = costRate.divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal cost = bigDecimal.multiply(payFee).setScale(2, BigDecimal.ROUND_HALF_UP);
        logger.info("【保存资金流水记录--订单支付】 成本==============bigDecimal=====================："+bigDecimal);
        logger.info("【保存资金流水记录--订单支付】 成本==============cost=====================："+cost);
        return cost;
    }



    private void asynchronousSaveCallbackLog(EtCallbackLog etCallbackLog) {
        //开异步线程保存回调参数
        scheduledExecutorService.schedule(() -> {
            logger.info("【微信支付回调】异步保存回调参数");
            etCallbackLog.setCreateTime(DateUtils.getNowDate());
            int i = callbackLogMapper.insertEtCallbackLog(etCallbackLog);
            if(i>0){
                logger.info("【微信支付回调】异步保存回调参数成功");
            }
        }, 0, TimeUnit.SECONDS);
    }


    /**
     * 微信退款回调
     */
    @Override
    @Transactional
    public void weChatRefund(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        EtCallbackLog etCallbackLog = new EtCallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("2");
        logger.info("【微信退款回调】接收对象 : " + JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        logger.info("【微信退款回调】转换成notification: " + JSON.toJSONString(notification));
        // 退款成功通知
        if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 RefundNotification
            RefundNotification refundNotification = checkAndParse(request, body, RefundNotification.class);
            logger.info("【微信退款回调】转换成RefundNotification: " + JSON.toJSONString(refundNotification));
            etCallbackLog.setBody(JSON.toJSONString(refundNotification));
            if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                // 退款成功后  改变 etRefundService
                String outRefundNo = refundNotification.getOutRefundNo();
                RlRefund rlRefund = new RlRefund();
                rlRefund.setRefundNo(outRefundNo);
                rlRefund.setRefundResult(Constants.SUCCESS2);
                int i = etRefundService.updateEtRefundByRefundNo(rlRefund);
                if(i==0){
                    logger.error("【微信退款回调】更新退款单失败");
                    throw new ServiceException("【微信退款回调】更新退款单失败");
                }
                logger.info("【微信退款回调】全部结束！！！！！");
            }
        }
        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        String wechatpaySerial = request.getHeader("Wechatpay-Serial");
        NotificationParser notificationParser = getNotificationParser(wechatpaySerial);
        return notificationParser.parse(requestParam, clazz);
    }


    public NotificationParser getNotificationParser(String wechatpaySerial) {
        // 根据证书序列号获取运营商
        ChannelVO channelVO = rlChannelService.selectChannelBySerialNumber(wechatpaySerial);
        if(ObjectUtil.isNull(channelVO)){
            throw new ServiceException("【微信支付回调】根据证书序列号获取支付渠道失败");
        }else{
            NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(channelVO.getMerchantId())
                    .privateKeyFromPath(channelVO.getPrivateKeyPath())
                    .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                    .apiV3Key(channelVO.getApiV3Key())
                    .build();
            return new NotificationParser(config);
        }
    }
}
