package com.ruoyi.system.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.accessory.RlAccessory;
import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.mapper.RlAccessoryMapper;
import com.ruoyi.system.service.IRlAccessoryService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 配件Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class RlAccessoryServiceImpl implements IRlAccessoryService
{
    @Resource
    private RlAccessoryMapper rlAccessoryMapper;

    /**
     * 查询配件
     *
     * @param accessoryId 配件主键
     * @return 配件
     */
    @Override
    public RlAccessory selectRlAccessoryByAccessoryId(Long accessoryId)
    {
        return rlAccessoryMapper.selectRlAccessoryByAccessoryId(accessoryId);
    }

    /**
     * 查询配件列表
     *
     * @param rlAccessory 配件
     * @return 配件
     */
    @Override
    public List<RlAccessory> selectRlAccessoryList(RlAccessory rlAccessory)
    {
        return rlAccessoryMapper.selectRlAccessoryList(rlAccessory);
    }

    /**
     * 根据车型id查询配件列表
     *
     * @param modelId 车型id
     * @return 配件集合
     */
    @Override
    public List<RlAccessoryVO> selectRlAccessoryListByModelId(Long modelId) {
        return rlAccessoryMapper.selectRlAccessoryListByModelId(modelId);
    }

    /**
     * 新增配件
     *
     * @param rlAccessory 配件
     * @return 结果
     */
    @Override
    public int insertRlAccessory(RlAccessory rlAccessory)
    {
        rlAccessory.setCreateTime(DateUtils.getNowDate());
        return rlAccessoryMapper.insertRlAccessory(rlAccessory);
    }

    /**
     * 修改配件
     *
     * @param rlAccessory 配件
     * @return 结果
     */
    @Override
    public int updateRlAccessory(RlAccessory rlAccessory)
    {
        rlAccessory.setUpdateTime(DateUtils.getNowDate());
        return rlAccessoryMapper.updateRlAccessory(rlAccessory);
    }

    /**
     * 批量删除配件
     *
     * @param accessoryIds 需要删除的配件主键
     * @return 结果
     */
    @Override
    public int deleteRlAccessoryByAccessoryIds(Long[] accessoryIds)
    {
        return rlAccessoryMapper.deleteRlAccessoryByAccessoryIds(accessoryIds);
    }

    /**
     * 删除配件信息
     *
     * @param accessoryId 配件主键
     * @return 结果
     */
    @Override
    public int deleteRlAccessoryByAccessoryId(Long accessoryId)
    {
        return rlAccessoryMapper.deleteRlAccessoryByAccessoryId(accessoryId);
    }


    /**
     * 根据车辆id
     *
     * @param modelId 车型id
     * @return 车辆配件
     */
    @Override
    public List<RlAccessoryVO> selectRlDeviceAccessoryByModelId(Long modelId)
    {
        return rlAccessoryMapper.selectRlAccessoryListByModelId(modelId);
    }
}
