package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.system.domain.agent.RlAgent;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.area.RlOperatingArea;
import com.ruoyi.system.domain.area.RlOperatingAreaVO;
import com.ruoyi.system.domain.cancelRule.RlCancelRule;
import com.ruoyi.system.mapper.RlAgentMapper;
import com.ruoyi.system.service.IRlAgentService;
import com.ruoyi.system.service.IRlCancelRuleService;
import com.ruoyi.system.service.IRlOperatingAreaService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 代理商Service业务层处理
 *
 * @author qzz
 * @date 2024-09-18
 */
@Service
public class RlAgentServiceImpl implements IRlAgentService
{
    @Resource
    private RlAgentMapper rlAgentMapper;

    @Resource
    private IRlCancelRuleService cancelRuleService;

    @Resource
    private IRlOperatingAreaService areaService;

    /**
     * 查询代理商
     *
     * @param agentId 代理商主键
     * @return 代理商
     */
    @Override
    public RlAgentVO selectRlAgentByAgentId(Long agentId)
    {
        RlAgentVO rlAgentVO = rlAgentMapper.selectRlAgentByAgentId(agentId);
        if(ObjectUtil.isNull(rlAgentVO)){
            throw new RuntimeException("代理商不存在");
        }
        getAgentInfo(rlAgentVO);
        return rlAgentVO;
    }

    private void getAgentInfo(RlAgentVO rlAgentVO) {
        List<RlCancelRule> rlCancelRules = cancelRuleService.selectRlCancelRuleListByAgentId(rlAgentVO.getAgentId());
        rlAgentVO.setCancelRuleList(rlCancelRules);
        RlOperatingAreaVO rlOperatingArea = areaService.selectRlOperatingAreaByAgentId(rlAgentVO.getAgentId());
        if(ObjectUtil.isNotNull(rlOperatingArea)){
            rlAgentVO.setLongitude(rlOperatingArea.getLongitude());
            rlAgentVO.setLatitude(rlOperatingArea.getLatitude());
        }
    }


    /**
     * 根据城市id查询代理商
     *
     * @param cityId 城市id
     * @return 代理商
     */
    @Override
    public RlAgentVO selectRlAgentByCityId(Long cityId)
    {
        RlAgentVO rlAgentVO = rlAgentMapper.selectRlAgentByCityId(cityId);
        if(ObjectUtil.isNull(rlAgentVO)){
            throw new RuntimeException("代理商不存在");
        }
        getAgentInfo(rlAgentVO);
        return rlAgentVO;
    }

    @Override
    public RlAgentVO selectRlAgentByUserId(Long userId) {
        RlAgentVO rlAgentVO = rlAgentMapper.selectRlAgentByUserId(userId);
        if(ObjectUtil.isNull(rlAgentVO)){
            throw new RuntimeException("代理商不存在");
        }
        return rlAgentVO;
    }


    /**
     * 查询代理商列表
     *
     * @param rlAgent 代理商
     * @return 代理商
     */
    @Override
    public List<RlAgentVO> selectRlAgentList(RlAgent rlAgent)
    {
        return rlAgentMapper.selectRlAgentList(rlAgent);
    }

    /**
     * 新增代理商
     *
     * @param rlAgent 代理商
     * @return 结果
     */
    @Override
    public int insertRlAgent(RlAgent rlAgent)
    {
        return rlAgentMapper.insertRlAgent(rlAgent);
    }

    /**
     * 修改代理商
     *
     * @param rlAgent 代理商
     * @return 结果
     */
    @Override
    public int updateRlAgent(RlAgent rlAgent)
    {
        return rlAgentMapper.updateRlAgent(rlAgent);
    }

    /**
     * 批量删除代理商
     *
     * @param agentIds 需要删除的代理商主键
     * @return 结果
     */
    @Override
    public int deleteRlAgentByAgentIds(Long[] agentIds)
    {
        return rlAgentMapper.deleteRlAgentByAgentIds(agentIds);
    }

    /**
     * 删除代理商信息
     *
     * @param agentId 代理商主键
     * @return 结果
     */
    @Override
    public int deleteRlAgentByAgentId(Long agentId)
    {
        return rlAgentMapper.deleteRlAgentByAgentId(agentId);
    }
}
