package com.ruoyi.system.service.impl;

import com.ruoyi.system.domain.cancelRule.RlCancelRule;
import com.ruoyi.system.mapper.RlCancelRuleMapper;
import com.ruoyi.system.service.IRlCancelRuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 取消规则Service业务层处理
 *
 * @author qzz
 * @date 2024-09-25
 */
@Service
public class RlCancelRuleServiceImpl implements IRlCancelRuleService
{
    @Autowired
    private RlCancelRuleMapper rlCancelRuleMapper;

    /**
     * 查询取消规则
     *
     * @param cancalId 取消规则主键
     * @return 取消规则
     */
    @Override
    public RlCancelRule selectRlCancelRuleByCancalId(Long cancalId)
    {
        return rlCancelRuleMapper.selectRlCancelRuleByCancalId(cancalId);
    }

    /**
     * 查询取消规则列表
     *
     * @param rlCancelRule 取消规则
     * @return 取消规则
     */
    @Override
    public List<RlCancelRule> selectRlCancelRuleList(RlCancelRule rlCancelRule)
    {
        return rlCancelRuleMapper.selectRlCancelRuleList(rlCancelRule);
    }

    /**
     * 根据代理商id查询取消规则列表
     *
     * @param agentId 代理商id
     * @return 取消规则集合
     */
    @Override
    public List<RlCancelRule> selectRlCancelRuleListByAgentId(Long agentId)
    {
        return rlCancelRuleMapper.selectRlCancelRuleListByAgentId(agentId);
    }

    /**
     * 新增取消规则
     *
     * @param rlCancelRule 取消规则
     * @return 结果
     */
    @Override
    public int insertRlCancelRule(RlCancelRule rlCancelRule)
    {
        return rlCancelRuleMapper.insertRlCancelRule(rlCancelRule);
    }

    /**
     * 修改取消规则
     *
     * @param rlCancelRule 取消规则
     * @return 结果
     */
    @Override
    public int updateRlCancelRule(RlCancelRule rlCancelRule)
    {
        return rlCancelRuleMapper.updateRlCancelRule(rlCancelRule);
    }

    /**
     * 批量删除取消规则
     *
     * @param cancalIds 需要删除的取消规则主键
     * @return 结果
     */
    @Override
    public int deleteRlCancelRuleByCancalIds(Long[] cancalIds)
    {
        return rlCancelRuleMapper.deleteRlCancelRuleByCancalIds(cancalIds);
    }

    /**
     * 删除取消规则信息
     *
     * @param cancalId 取消规则主键
     * @return 结果
     */
    @Override
    public int deleteRlCancelRuleByCancalId(Long cancalId)
    {
        return rlCancelRuleMapper.deleteRlCancelRuleByCancalId(cancalId);
    }
}
