package com.ruoyi.system.service.impl;

import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.deliveryOrderOper.RlDeliveryOrderOper;
import com.ruoyi.system.mapper.RlDeliveryOrderOperMapper;
import com.ruoyi.system.service.IRlDeliveryOrderOperService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 配送单履历Service业务层处理
 *
 * @author qzz
 * @date 2024-10-10
 */
@Slf4j
@Service
public class RlDeliveryOrderOperServiceImpl implements IRlDeliveryOrderOperService
{
    @Resource
    private RlDeliveryOrderOperMapper rlDeliveryOrderOperMapper;

    /**
     * 查询配送单履历
     *
     * @param operId 配送单履历主键
     * @return 配送单履历
     */
    @Override
    public RlDeliveryOrderOper selectRlDeliveryOrderOperByOperId(Long operId)
    {
        return rlDeliveryOrderOperMapper.selectRlDeliveryOrderOperByOperId(operId);
    }

    /**
     * 查询配送单履历列表
     *
     * @param rlDeliveryOrderOper 配送单履历
     * @return 配送单履历
     */
    @Override
    public List<RlDeliveryOrderOper> selectRlDeliveryOrderOperList(RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        return rlDeliveryOrderOperMapper.selectRlDeliveryOrderOperList(rlDeliveryOrderOper);
    }

    /**
     * 新增配送单履历
     *
     * @param rlDeliveryOrderOper 配送单履历
     * @return 结果
     */
    @Override
    public int insertRlDeliveryOrderOper(RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        return rlDeliveryOrderOperMapper.insertRlDeliveryOrderOper(rlDeliveryOrderOper);
    }

    /**
     * 修改配送单履历
     *
     * @param rlDeliveryOrderOper 配送单履历
     * @return 结果
     */
    @Override
    public int updateRlDeliveryOrderOper(RlDeliveryOrderOper rlDeliveryOrderOper)
    {
        return rlDeliveryOrderOperMapper.updateRlDeliveryOrderOper(rlDeliveryOrderOper);
    }

    /**
     * 批量删除配送单履历
     *
     * @param operIds 需要删除的配送单履历主键
     * @return 结果
     */
    @Override
    public int deleteRlDeliveryOrderOperByOperIds(Long[] operIds)
    {
        return rlDeliveryOrderOperMapper.deleteRlDeliveryOrderOperByOperIds(operIds);
    }

    /**
     * 删除配送单履历信息
     *
     * @param operId 配送单履历主键
     * @return 结果
     */
    @Override
    public int deleteRlDeliveryOrderOperByOperId(Long operId)
    {
        return rlDeliveryOrderOperMapper.deleteRlDeliveryOrderOperByOperId(operId);
    }

    /**
     * 添加工单履历日志
     *
     * @param userId 用户id
     * @param userName 用户名
     * @param phone 电话
     * @param deliveryId 配送单id
     * @return 结果
     */
    @Override
    public int deliveryOrderOperLog(Long userId, String userName, String phone, Long deliveryId,String type) {
        RlDeliveryOrderOper insertDeliveryOrderOper= new RlDeliveryOrderOper();
        insertDeliveryOrderOper.setDeliveryId(deliveryId);
        insertDeliveryOrderOper.setType(type);
        insertDeliveryOrderOper.setOperUserid(userId);
        insertDeliveryOrderOper.setOperName(userName);
        insertDeliveryOrderOper.setOperPhone(phone);
        insertDeliveryOrderOper.setOperTime(DateUtils.getNowDate());
        int i1 = insertRlDeliveryOrderOper(insertDeliveryOrderOper);
        if(i1>0){
            log.info("保存配送单日志记录成功");
        }
        return i1;
    }
}
