package com.ruoyi.system.service.impl;

import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.domain.deviceAccessory.RlDeviceAccessory;
import com.ruoyi.system.mapper.RlDeviceAccessoryMapper;
import com.ruoyi.system.service.IRlDeviceAccessoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 车辆配件Service业务层处理
 *
 * @author qzz
 * @date 2024-09-23
 */
@Service
public class RlDeviceAccessoryServiceImpl implements IRlDeviceAccessoryService
{
    @Resource
    private RlDeviceAccessoryMapper rlDeviceAccessoryMapper;

    /**
     * 查询车辆配件
     *
     * @param accessoryId 车辆配件主键
     * @return 车辆配件
     */
    @Override
    public RlDeviceAccessory selectRlDeviceAccessoryByAccessoryId(Long accessoryId)
    {
        return rlDeviceAccessoryMapper.selectRlDeviceAccessoryByAccessoryId(accessoryId);
    }

    /**
     * 查询车辆配件列表
     *
     * @param rlDeviceAccessory 车辆配件
     * @return 车辆配件
     */
    @Override
    public List<RlDeviceAccessory> selectRlDeviceAccessoryList(RlDeviceAccessory rlDeviceAccessory)
    {
        return rlDeviceAccessoryMapper.selectRlDeviceAccessoryList(rlDeviceAccessory);
    }

    /**
     * 新增车辆配件
     *
     * @param rlDeviceAccessory 车辆配件
     * @return 结果
     */
    @Override
    public int insertRlDeviceAccessory(RlDeviceAccessory rlDeviceAccessory)
    {
        return rlDeviceAccessoryMapper.insertRlDeviceAccessory(rlDeviceAccessory);
    }

    /**
     * 修改车辆配件
     *
     * @param rlDeviceAccessory 车辆配件
     * @return 结果
     */
    @Override
    public int updateRlDeviceAccessory(RlDeviceAccessory rlDeviceAccessory)
    {
        return rlDeviceAccessoryMapper.updateRlDeviceAccessory(rlDeviceAccessory);
    }

    /**
     * 批量删除车辆配件
     *
     * @param accessoryIds 需要删除的车辆配件主键
     * @return 结果
     */
    @Override
    public int deleteRlDeviceAccessoryByAccessoryIds(Long[] accessoryIds)
    {
        return rlDeviceAccessoryMapper.deleteRlDeviceAccessoryByAccessoryIds(accessoryIds);
    }

    /**
     * 删除车辆配件信息
     *
     * @param accessoryId 车辆配件主键
     * @return 结果
     */
    @Override
    public int deleteRlDeviceAccessoryByAccessoryId(Long accessoryId)
    {
        return rlDeviceAccessoryMapper.deleteRlDeviceAccessoryByAccessoryId(accessoryId);
    }

    /**
     * 根据车辆id删除车辆配件信息
     *
     * @param deviceId 车辆id
     * @return 结果
     */
    @Override
    public int deleteRlDeviceAccessoryByDeviceId(Long deviceId) {
        return rlDeviceAccessoryMapper.deleteRlDeviceAccessoryByDeviceId(deviceId);
    }
}
