package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.*;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.onenet.*;
import com.ruoyi.system.domain.RlCommandLog;
import com.ruoyi.system.domain.RlFunction;
import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.RlOnlineLog;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.model.RlModelVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.mapper.RlDeviceMapper;
import com.ruoyi.system.mapper.RlCommandLogMapper;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
@Slf4j
@Service
public class RlDeviceServiceImpl extends ServiceImpl<RlDeviceMapper, RlDevice> implements IRlDeviceService
{
    @Resource
    private RlDeviceMapper deviceMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private RedisCache redisCache;

    @Resource
    private IRlDeviceAccessoryService deviceAccessoryService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlFeeRuleService feeRuleService;

    @Autowired
    private IRlAccessoryService accessoryService;

    @Autowired
    private RlStoreService storeService;

    @Autowired
    private ISysConfigService sysConfigService;
//
//    @Autowired
//    private IEtFeeRuleService etFeeRuleService;
//
//    @Autowired
//    private IEtParkingAreaService parkingAreaService;
//
//    @Resource
//    private ISysDictDataService sysDictDataService;

    @Resource
    private IRlTripLogService tripLogService;

    @Resource
    private IRlModelService modelService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IWxPayService wxPayService;
//
//    @Autowired
//    private IRlUserService userService;
//
//    @Resource
//    private EtOrderMapper etOrderMapper;
//
    @Resource
    private RlCommandLogMapper rlCommandLogMapper;

    @Autowired
    private IRlOnlineLogService eOnlineLogService;

    @Autowired
    private IRlOrderOperService orderOperService;
//
//    @Autowired
//    private IEtRefundService etRefundService;
//
//    @Resource
//    private IAsUserService asUserService;
//
//    @Autowired
//    private CallbackService callbackService;

    @Value(value = "${iot.iotUrl}")
    private String iotUrl;

    @Value(value = "${iot.productId}")
    private String productId;

    @Value(value = "${iot.timeout}")
    private String timeout;

    @Value(value = "${iot.deviceUrl}")
    private String deviceUrl;

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public RlDevice selectDeviceByDeviceId(Long deviceId)
    {
        RlDevice device = deviceMapper.selectDeviceByDeviceId(deviceId);
//        String status = device.getStatus();
//        if(ObjectUtil.isNotNull(status)){
//            String typeName = sysDictDataService.selectDictLabel("as_device_status", status);
//            device.setStatusStr(typeName);
//            if(status.equals(ServiceConstants.VEHICLE_STATUS_NOT_BAND) && ObjectUtil.isNotNull(device.getAreaId())){
//                device.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_LISTING);
//                int i = eDeviceMapper.updateAsDevice(device);
//            }
//        }
//        Long areaId = device.getAreaId();
//        if (ObjectUtil.isNotNull(areaId) && areaId!=0){
//            EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
//            device.setAreaName(etOperatingArea.getAreaName());
//        }
//        if(ObjectUtil.isNotNull(areaId) && areaId!=0){
//            SysDept sysDept = wxPayService.getDeptObjByAreaId(areaId);
//            //https://dianche.chuantewulian.cn?sn=https://dche.ccttiot.com?sn=3000900
//            device.setQrText(sysDept.getDomain()+"?sn="+device.getSn());
//            device.setDeptName(sysDept.getDeptName());
//        }else{
//            device.setQrText("");
//        }
        return device;
    }

    /**
     * 根据mac查询车辆实时信息
     *
     * @param mac 设备mac
     * @return 设备
     */
    @Override
    public RlDevice selectDeviceByMac(String mac) {
        return deviceMapper.selectDeviceByMac(mac);
    }

    /**
     * 根据sn号查询车辆实时信息  真实金额  实际
     *
     * @param sn 设备sn
     * @return 设备
     */
    @Override
    public RlDeviceVO getLatestLocationBySn(String sn) {
        RlDeviceVO rlDevice = deviceMapper.selectDeviceBySn(sn);
        if (ObjectUtil.isNull(rlDevice)) {
            throw new ServiceException("设备不存在："+ sn);
        }
        String msg = redisCache.getCacheObject(CacheConstants.CACHE_DEVICE_KEY + rlDevice.getSn());
        if(StrUtil.isNotBlank(msg)){
            log.info("【根据sn号查询车辆实时信息】-========redis缓存中的数据：" + msg);
            LogEntry logEntry = JSONObject.parseObject(msg, LogEntry.class);
            log.info("【根据sn号查询车辆实时信息】============logEntry转换后的对象: logEntry---【{}】" , JSON.toJSONString(logEntry));
            LogEntry.LocationValue value = logEntry.getValue();
            // 坐标转换  WGS84 转 GCJ02
            double[] doubles = CommonUtil.coordinateConvert(value.getLon(),value.getLat());
            BigDecimal lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
            BigDecimal lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
            rlDevice.setLongitude(lon.toString());
            rlDevice.setLatitude(lat.toString());
        }
        StoreVo storeVo = storeService.selectSmStoreById(rlDevice.getStoreId());
        if(ObjectUtil.isNotNull(storeVo)){
            rlDevice.setStoreLocation(storeVo.getAddress());
            rlDevice.setStoreName(storeVo.getName());
        }
        RlModelVO rlModelVO = modelService.selectEModelByModelId(rlDevice.getModelId());
        if(ObjectUtil.isNotNull(rlModelVO)){
            if(StrUtil.isNotBlank(rlDevice.getVoltage())){
                rlDevice.setRemainingMileage(CommonUtil.getRemainingMileage(rlDevice.getVoltage(),rlModelVO.getFullVoltage(),rlModelVO.getLowVoltage(),rlModelVO.getFullEndurance()));
            }
            rlDevice.setModel(rlModelVO.getModel());
            List<RlFunction> functionList = modelService.getFunctionListByModelId(rlDevice.getModelId());
            List<RlAccessoryVO> rlAccessoryVOS = accessoryService.selectRlAccessoryListByModelId(rlDevice.getModelId());
            rlDevice.setFunctionList(functionList);
            rlDevice.setAccessorys(rlAccessoryVOS);
        }
        return rlDevice;
    }

    /**
     * 根据sn号查询车辆
     *
     * @param sn 设备sn
     * @return 设备
     */
    @Override
    public RlDevice selectDeviceBySn(String sn) {
        return deviceMapper.selectDeviceBySn(sn);
    }

    /**
     * 查询设备列表
     *
     * @param asDevice 设备
     * @return 设备
     */
    @Override
    public List<RlDeviceVO> selectDeviceList(RlDeviceQuery asDevice)
    {
        List<RlDeviceVO> asDevices = deviceMapper.selectDeviceList(asDevice);
        return asDevices;
    }

    /**
     * 查询设备列表
     *
     * @param asDevice 设备 隔离
     * @return 设备
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<RlDevice> selectDeviceListWithIsolate(RlDevice asDevice)
    {
        if(ServiceConstants.VEHICLE_STATUS_NOT_BAND.equals(asDevice.getStatus())){
            asDevice.setStatus(null);
        }
        List<RlDevice> asDevices = deviceMapper.selectDeviceListWithIsolate(asDevice);
//        for (EDevice asDevice1:asDevices){
//            Long areaId = asDevice1.getAreaId();
//            if (ObjectUtil.isNotNull(areaId) && areaId!=0){
//                EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
//                asDevice1.setAreaName(etOperatingArea.getAreaName());
//            }
//            Long modelId = asDevice1.getModelId();
//            if (ObjectUtil.isNotNull(modelId)){
//                EtModel model  = etModelService.selectEtModelByModelId(modelId);
//                if(ObjectUtil.isNotNull(model)){
//                    asDevice1.setModel(model.getModel());
//                }
//            }
//            String status = asDevice1.getStatus();
//            if(ObjectUtil.isNotNull(status)){
//                String typeName = sysDictDataService.selectDictLabel("as_device_status", status);
//                asDevice1.setStatusStr(typeName);
//                if(status.equals(ServiceConstants.VEHICLE_STATUS_NOT_BAND) && ObjectUtil.isNotNull(asDevice1.getAreaId())){
//                    asDevice1.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_LISTING);
//                    int i = eDeviceMapper.updateAsDevice(asDevice1);
//                }
//            }
//            if(ObjectUtil.isNull(asDevice1.getAreaId())){
//                asDevice1.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_BAND);
//                int i = eDeviceMapper.updateAsDevice(asDevice1);
//            }
//            if(ObjectUtil.isNotNull(areaId) && areaId!=0){
//                SysDept sysDept = wxPayService.getDeptObjByAreaId(areaId);
//                //https://dianche.chuantewulian.cn?sn=https://dche.ccttiot.com?sn=3000900
//                asDevice1.setQrText(sysDept.getDomain()+"?sn="+asDevice1.getSn());
//                asDevice1.setDeptName(sysDept.getDeptName());
//            }else{
//                asDevice1.setQrText("");
//            }
//        }
        return asDevices;
    }

    /**
     * 新增设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int insertDevice(RlDevice asDevice)
    {
        asDevice.setCreateTime(DateUtils.getNowDate());
        return deviceMapper.insertDevice(asDevice);
    }

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateDevice(RlDevice asDevice)
    {
        asDevice.setUpdateTime(DateUtils.getNowDate());
        int i = deviceMapper.updateDevice(asDevice);
        return i;
    }

    /**
     * 修改设备(车型和硬件版本可为空)
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateDevice2(RlDevice asDevice)
    {
        asDevice.setUpdateTime(DateUtils.getNowDate());
//        if(asDevice.getAreaId() == null){
//            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_BAND);
//            asDevice.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_OFFLINE);
//            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
//        }
        return deviceMapper.updateDevice2(asDevice);
    }

    /**
     * 根据SN修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateDeviceBySn(RlDevice asDevice)
    {
        asDevice.setUpdateTime(DateUtils.getNowDate());
        return deviceMapper.updateDeviceBySn(asDevice);
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteDeviceByDeviceIds(Long[] deviceIds)
    {
        return deviceMapper.deleteDeviceByDeviceIds(deviceIds);
    }
//
//    /**
//     * 一键上线
//     *
//     * @param deviceIds 需要一键上线的设备主键集合
//     * @return 结果
//     */
//    @Override
//    public int oneClickOnline(Long[] deviceIds)
//    {
//        for (Long deviceId:deviceIds) {
//            AsDevice device = eDeviceMapper.selectAsDeviceByDeviceId(deviceId);
//            if(ObjectUtil.isNull(device)){
//                throw new ServiceException("车辆【"+deviceId+"】不存在");
//            }
//            if(!device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_OFFLINE)){
//                throw new ServiceException("车辆【"+device.getSn()+"】非下线状态，请重新选择！");
//            }
//        }
//        return eDeviceMapper.oneClickOnline(deviceIds);
//    }
//
//    /**
//     * 一键出仓
//     *
//     * @param deviceIds 需要一键出仓的设备主键集合
//     * @return 结果
//     */
//    @Override
//    public int oneClickListing(Long[] deviceIds)
//    {
//        for (Long deviceId:deviceIds) {
//            AsDevice device = eDeviceMapper.selectAsDeviceByDeviceId(deviceId);
//            if(ObjectUtil.isNull(device)){
//                throw new ServiceException("车辆【"+deviceId+"】不存在");
//            }
//            if(!device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
//                throw new ServiceException("车辆【"+device.getSn()+"】非仓库中状态，请重新选择！");
//            }
//        }
//        return eDeviceMapper.oneClickOnline(deviceIds);
//    }
//
//    /**
//     * 一键下线
//     *
//     * @param deviceIds 需要一键下线的设备主键集合
//     * @return 结果
//     */
//    @Override
//    public int oneClickOffline(Long[] deviceIds)
//    {
//        for (Long deviceId:deviceIds) {
//            AsDevice device = eDeviceMapper.selectAsDeviceByDeviceId(deviceId);
//            if(ObjectUtil.isNull(device)){
//                throw new ServiceException("车辆【"+deviceId+"】不存在");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘预约中’状态不能下线");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_USING)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘使用中’状态不能下线");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘临时停车’状态不能下线");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘未上架’状态不能下线");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_ABANDON)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘废弃’状态不能下线");
//            }
//        }
//        return eDeviceMapper.oneClickOffline(deviceIds);
//    }
//
//    /**
//     * 一键入仓
//     *
//     * @param deviceIds 需要一键入仓的设备主键集合
//     * @return 结果
//     */
//    @Override
//    public int oneClickWarehousing(Long[] deviceIds)
//    {
//        for (Long deviceId:deviceIds) {
//            AsDevice device = eDeviceMapper.selectAsDeviceByDeviceId(deviceId);
//            if(ObjectUtil.isNull(device)){
//                throw new ServiceException("车辆【"+deviceId+"】不存在");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘预约中’状态不能入仓");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_USING)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘使用中’状态不能入仓");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘临时停车’状态不能入仓");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘仓库中’状态不能入仓");
//            }
//            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_ABANDON)){
//                throw new ServiceException("车辆【"+device.getSn()+"】为‘废弃’状态不能入仓");
//            }
//        }
//        return eDeviceMapper.oneClickWarehousing(deviceIds);
//    }
//
//    /**
//     * 更新设备
//     * 1. 发送更新版本
//     * 2. 判断是否在线
//     *
//     * @param deviceIds 需要更新设备的设备主键集合
//     * @return 结果
//     */
//    @Override
//    public int refreshDevice(Long[] deviceIds)
//    {
//        for (Long deviceId:deviceIds) {
//            AsDevice device = eDeviceMapper.selectAsDeviceByDeviceId(deviceId);
//            //设备是否在线
//            if(isOnline(device.getSn())){
//                log.info("设备【"+device.getSn()+"】在线");
//            }else{
//                log.info("设备【"+device.getSn()+"】不在线");
//            }
//            if(updateVersion(device.getSn())){
//                log.info("设备【"+device.getSn()+"】更新版本成功");
//            }else{
//                log.info("设备【"+device.getSn()+"】更新版本失败");
//            }
//        }
//        return 1;
//    }
//
//    /**
//     * 删除设备信息
//     *
//     * @param deviceId 设备主键
//     * @return 结果
//     */
//    @Override
//    public int deleteAsDeviceByDeviceId(Long deviceId)
//    {
//        return eDeviceMapper.deleteAsDeviceByDeviceId(deviceId);
//    }
//
//    /**
//     * 根据定位获取运营区信息，并返回所有车辆定位
//     *   a. 将前端的边界值获取后转成geometry格式
//     *   b. 模拟gps定位根据定位计算在哪个运营区内
//     *   c. 获取到运营区信息，查询出该运营区下所有的车辆信息并获取所有车辆的定位返回前端
//     * @param longitude 经度
//     * @param latitude 纬度
//     * @return 结果
//     */
//    @Override
//    public List<AsDevice> vehicleLocalization(String longitude, String latitude) {
//        List<EtOperatingArea> etOperatingAreas = etOperatingAreaService.selectEtOperatingAreaList(new EtOperatingArea());
//        EtOperatingArea area = null;
//        for(EtOperatingArea etOperatingArea:etOperatingAreas){
//            Geometry geometry = GeoUtils.fromWkt(etOperatingArea.getBoundary());
//            Boolean inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
//            if(inCircle){
//                area = etOperatingArea;
//                break;
//            }
//        }
//        List<AsDevice> asDevices = new ArrayList<>();
//        if(ObjectUtil.isNotNull(area)){
//            /** c. 获取到运营区信息，查询出该运营区下所有的车辆信息并获取所有车辆的定位返回前端*/
//            log.info("在【{}】运营区内",area.getAreaName());
//            QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//            wrapper.eq("area_id", area.getAreaId());
//            wrapper.in("status","1");//0 未上架，1-正常
//            asDevices = eDeviceMapper.selectList(wrapper);
//            return asDevices;
//        }else{
//            log.info("不在任何运营区内");
//            return asDevices;
//        }
//    }
//
//    /**
//     * 所有车辆定位
//     */
//    @Override
//    public List<AsDevice> allVehicleInfo(String powerStart, String powerEnd, String status,String sort,String areaId,String onlineStatus) {
//        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//        if(StrUtil.isNotBlank(areaId)){
//            wrapper.eq("area_id",areaId);
//        }
//        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
//            wrapper.between("remaining_power", Integer.parseInt(powerStart), Integer.parseInt(powerEnd));
//        }
//        if(StrUtil.isNotBlank(status)){
//            if(StrUtil.equals(status,"-0")){
//                wrapper.ne("status","0");
//            }else{
//                wrapper.in("status",status);
//            }
//        }
//        if(StrUtil.isNotBlank(onlineStatus)){
//            wrapper.in("online_status",onlineStatus);
//        }
//        if(StrUtil.isNotBlank(sort)){
//            wrapper.orderBy(true,!StrUtil.equals(sort, "desc"),"remaining_power");
//        }
//        List<AsDevice> asDevices = eDeviceMapper.selectList(wrapper);
//        for (AsDevice asDevice:asDevices){
//            Long modelId = asDevice.getModelId();
//            if (ObjectUtil.isNotNull(modelId)){
//                EtModel model = etModelService.selectEtModelByModelId(modelId);
//                if(ObjectUtil.isNotNull(model)){
//                    asDevice.setModel(model.getModel());
//                }
//            }
//        }
//        return asDevices;
//    }
//
//    /**
//     * 查询车辆数量
//     */
//    @Override
//    public DeviceNumVo allVehicleNum(String powerStart, String powerEnd,String areaId) {
//        DeviceNumVo deviceNumVo = new DeviceNumVo();
//        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//        if(StrUtil.isNotBlank(areaId)){
//            wrapper.eq("area_id",areaId);
//        }
//        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
//            wrapper.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
//        }
//        Integer allNum = eDeviceMapper.selectCount(wrapper);
//        deviceNumVo.setAllNum(allNum);//所有车辆
//
//        Integer ridingNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_IN_USING,areaId);
//        deviceNumVo.setRidingNum(ridingNum);//骑行中
//
//        Integer inOperation = setInOperationNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NOT_LISTING,areaId);
//        deviceNumVo.setInOperation(inOperation);//投放中
//
//        Integer temporarilyLockNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK,areaId);
//        deviceNumVo.setTemporarilyLockNum(temporarilyLockNum);//临时锁车
//
//        Integer disabledNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_IN_OFFLINE,areaId);
//        deviceNumVo.setDisabledNum(disabledNum);//已禁用
//
//        Integer normalNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NORMAL,areaId);
//        deviceNumVo.setNormalNum(normalNum);//正常待租
//
//        Integer inStashNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NOT_LISTING,areaId);
//        deviceNumVo.setInStashNum(inStashNum);
//
//        QueryWrapper<AsDevice> wrapperForOffline = new QueryWrapper<>();
//        wrapperForOffline.eq("online_status","0");//在线状态：0-不在线；1-在线
//        wrapperForOffline.ne("status","0");//
//        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
//            wrapperForOffline.between("remaining_power",Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
//        }
//        if(StrUtil.isNotBlank(areaId)){
//            wrapperForOffline.eq("area_id",areaId);
//        }
//        Integer offlineNum = eDeviceMapper.selectCount(wrapperForOffline);
//        deviceNumVo.setOfflineNum(offlineNum);//已离线
//
//        Integer inAppointmentNum = setNum(powerStart, powerEnd, ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT,areaId);
//        deviceNumVo.setInAppointmentNum(inAppointmentNum);//预约中
//
//        QueryWrapper<AsDevice> wrapperForDispatch = new QueryWrapper<>();
//        wrapperForDispatch.eq("status","6");
//        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
//            wrapperForDispatch.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
//        }
//        if(StrUtil.isNotBlank(areaId)){
//            wrapperForDispatch.eq("area_id",areaId);
//        }
//        Integer dispatchNum = eDeviceMapper.selectCount(wrapperForDispatch);
//        deviceNumVo.setDispatchNum(dispatchNum);//调度设备
//        return deviceNumVo;
//    }
//
//    private Integer setNum(String powerStart, String powerEnd,String status,String areaId) {
//        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//        wrapper.eq("status",status);
//        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
//            wrapper.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
//        }
//        if(StrUtil.isNotBlank(areaId)){
//            wrapper.eq("area_id",areaId);
//        }
//        return eDeviceMapper.selectCount(wrapper);
//    }
//
//    private Integer setInOperationNum(String powerStart, String powerEnd,String status,String areaId) {
//        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//        wrapper.ne("status",status);
//        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
//            wrapper.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
//        }
//        if(StrUtil.isNotBlank(areaId)){
//            wrapper.eq("area_id",areaId);
//        }
//        return eDeviceMapper.selectCount(wrapper);
//    }
//
//
//    /**
//     * 编号开锁
//     * @param orderVo 订单对象
//     *      1.远程开锁
//     *      2.更新车辆状态
//     *      3.创建订单
//     * @return
//     */
//    @SneakyThrows
//    @Override
//    @Transactional
//    public OrderResponse snSwitch(EtOrderVo orderVo) {
//        OrderResponse response = new OrderResponse();
//        String orderNo = "";
//        if(ObjectUtil.isNull(orderVo.getOrderNo())){
//            orderNo = IdUtils.getOrderNo("qx");
//        }else{
//            orderNo = orderVo.getOrderNo();
//        }
//        String sessionId = IdUtils.randomUUIDByDigit(8);
//        AsDevice asDevice = eDeviceMapper.selectAsDeviceBySn(orderVo.getSn());
//        /** 1.获取token*/
//        String token = Token.getToken();
//        String finalOrderNo = orderNo;
//        if(!"true".equals(orderVo.getIsBluetooth())){
//            /** 2.发送命令*/
//            ResponseVo responseVo = sendCommandWithResp(asDevice.getMac(), token, IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_5, "编号开锁", finalOrderNo);
//            if(responseVo.getCode() != 0){
//                throw new ServiceException("【扫码/编号开锁骑行】发送开锁命令失败");
//            }
//        }else{
//            asDevice.setLongitude(orderVo.getLon());
//            asDevice.setLatitude(orderVo.getLat());
//            asDevice.setLastTime(DateUtils.getNowDate());
//        }
//        /** 3.更新车辆状态*/
//        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
//        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);
//        asDevice.setIsAdminUnlocking("0");
//        int device = eDeviceMapper.updateAsDevice(asDevice);
//        if(device==0){
//            log.info("【扫码/编号开锁骑行】更新车辆状态失败");
//            throw new ServiceException("【扫码/编号开锁骑行】更新车辆状态失败");
//        }
//        /** 4.如果有预约订单则更新订单的预约结束时间，如果没有预约，则创建订单*/
//        EtOrder order = etOrderService.selectEtOrderByOrderNo(finalOrderNo);
//        if(ObjectUtil.isNotNull(order)){//有订单号：可能是套餐也可能是有预约
//            if(ObjectUtil.isNotNull(order.getRuleId())){//套餐
//                log.info("【扫码/编号开锁骑行】---预约扫码骑行");
//                order.setSn(asDevice.getSn());
//            }else{
//                throw new ServiceException("ruleId未传");
//            }
//            order.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
//            order.setUnlockTime(DateUtils.getNowDate());
//            order.setAppointmentEndTime(DateUtils.getNowDate());
//            //计算预约费
//            calculateAppointmentFee(order);
//            order.setTotalFee(order.getAppointmentFee());
//            order.setPayFee(order.getAppointmentFee());
//            int update = etOrderService.updateEtOrder(order);
//            if(update==0){
//                log.info("【扫码/编号开锁骑行】更新订单失败");
//            }
//        }else{
//            log.info("【扫码/编号开锁骑行】---无订单号,正常开锁骑行订单");
//            order = etOrderService.createOrder(orderVo, finalOrderNo);
//            int etOrder = etOrderService.insertEtOrder(order);
//            if(etOrder==0){
//                log.info("【扫码/编号开锁骑行】保存订单失败");
//                throw new ServiceException("【扫码/编号开锁骑行】保存订单失败");
//            }
//        }
//        /** 5.记录行程*/
//        int tripLog = tripLogService.tripLog(order.getOrderNo(),order.getSn(),ServiceConstants.TRIP_LOG_TYPE_UNLOCK_RIDE);
//        if(tripLog==0){
//            log.info("【扫码/编号开锁骑行】记录行程失败");
//            throw new ServiceException("【扫码/编号开锁骑行】记录行程失败");
//        }
//        log.info("【扫码/编号开锁骑行】车辆开锁成功");
//        response.setOrderNo(orderNo);
//        response.setSessionId(sessionId);
//        return response;
//    }

    /**
     * 管理员开锁
     */
    @SneakyThrows
    @Override
    public Boolean unlocking(String sn,String userName) {
        RlDevice asDevice = deviceMapper.selectDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            RlDevice updateDevice = new RlDevice();
            updateDevice.setDeviceId(asDevice.getDeviceId());
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_20,"管理员开锁",null,userName);
            updateDevice.setIsAdminUnlocking("1");
            if(!asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_SCHEDULING);
            }
            updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int i = deviceMapper.updateDevice(updateDevice);
            if(i>0){
                log.info("管理员开锁，更新设备状态成功");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("管理员开锁失败");
        return Boolean.TRUE;
    }

    /**
     * 管理员开锁
     */
    @SneakyThrows
    @Override
    public Boolean unlockingByMac(String mac,String userName) {
        RlDevice rlDevice = deviceMapper.selectDeviceByMac(mac);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            RlDevice updateDevice = new RlDevice();
            updateDevice.setDeviceId(rlDevice.getDeviceId());
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_20,"管理员开锁",null,userName);
            updateDevice.setIsAdminUnlocking("1");
            if(!rlDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_SCHEDULING);
            }
            updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int i = deviceMapper.updateDevice(updateDevice);
            if(i>0){
                log.info("管理员开锁，更新设备状态成功");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("管理员开锁失败");
        return Boolean.TRUE;
    }

    /**
     * 车辆上线
     */
    @SneakyThrows
    @Override
    public Boolean online(String sn) {
        RlDevice rlDevice = deviceMapper.selectDeviceBySn(sn);
        RlDevice updateDevice = new RlDevice();
        updateDevice.setDeviceId(rlDevice.getDeviceId());
        updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);
        int device = deviceMapper.updateDevice(updateDevice);
        if(device==0){
            log.info("车辆上线状态失败");
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 车辆下线
     */
    @SneakyThrows
    @Override
    public Boolean offline(String sn,String status) {
        RlDevice rlDevice = deviceMapper.selectDeviceBySn(sn);
        if(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT.equals(rlDevice.getStatus())){
            throw new ServiceException("车辆处于预约中，不能下线");
        }
        if(ServiceConstants.VEHICLE_STATUS_IN_USING.equals(rlDevice.getStatus())){
            throw new ServiceException("车辆使用中，不能下线");
        }
        if(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK.equals(rlDevice.getStatus())){
            throw new ServiceException("车辆临时停车中，不能下线");
        }
        RlDevice updateDevice = new RlDevice();
        updateDevice.setDeviceId(rlDevice.getDeviceId());
        updateDevice.setStatus(status);
        int device = deviceMapper.updateDevice(updateDevice);
        if(device==0){
            log.info("车辆下线状态失败");
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

//    /**
//     * 根据条件模糊查询车辆信息
//     * type:1-sn,2-mac,3-车牌号
//     */
//    @Override
//    public List<AsDevice> deviceSearch(String key, String type, String areaId) {
//        QueryWrapper<AsDevice> queryWrapper = new QueryWrapper<>();
//        if(StrUtil.isNotBlank(areaId)){
//            queryWrapper.eq("area_id",areaId);
//        }
//        switch (type){
//            case "1":
//                queryWrapper.like("sn", key);
//                return eDeviceMapper.selectList(queryWrapper);
//            case "2":
//                queryWrapper.like("mac", key);
//                return eDeviceMapper.selectList(queryWrapper);
//            case "3":
//                queryWrapper.like("vehicle_num", key);
//                return eDeviceMapper.selectList(queryWrapper);
//            default:
//                return null;
//        }
//    }
//
//    /**
//     * 运营中车辆
//     */
//    @Override
//    public String getInOperationDevice(String timeStart, String timeEnd,String areaId) {
//        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
//            QueryWrapper<AsDevice> wrapper = new QueryWrapper<AsDevice>().in("status", "1", "2", "3", "4");
//            wrapper.eq("area_id", areaId);
//            return eDeviceMapper.selectCount(wrapper)+"";
//        }
//        return null;
//    }
//
//    /**
//     * 有订单车辆
//     */
//    @Override
//    public String getInOrderDevice(String timeStart, String timeEnd,String areaId) {
//        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
//            return eDeviceMapper.getInOrderDevice(timeStart,timeEnd,areaId);
//        }
//        return null;
//    }
//
//    /**
//     * 无订单车辆
//     */
//    @Override
//    public String getNoOrderDevice(String timeStart, String timeEnd,String areaId) {
//        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
//            return eDeviceMapper.getNoOrderDevice(timeStart,timeEnd,areaId);
//        }
//        return null;
//    }

    /**
     * 根据车辆型号查询车辆数量
     */
    @Override
    public Integer selectCountByModelId(Long modelId) {
        QueryWrapper<RlDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("model_id", modelId);
        Integer integer = deviceMapper.selectCount(wrapper);
        return integer;
    }

//    /**
//     * 根据运营区查询车辆数量
//     */
//    @Override
//    public Integer selectCountByAreaId(Long areaId) {
//        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//        wrapper.eq("area_id", areaId);
//        Integer integer = eDeviceMapper.selectCount(wrapper);
//        return integer;
//    }

    @Transactional
    @Override
    /** 发送命令*/
    public void sendCommand(String mac, String token,String command,String type,String orderNo,String userName) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        JSONObject paramsObj = JSON.parseObject(result);
        String code = paramsObj.getString("code");

        if ("10500".equals(code)) { // 超时
            log.info("第一次请求超时，进行第二次请求...");
            result = HttpUtils.sendPostWithToken(sendUrl, command, token);
            log.info("【" + type + "】===>IOT第二次请求调用结果:【{}】", result);
            paramsObj = JSON.parseObject(result);
            code = paramsObj.getString("code");
        }

        asynchronousSaveLog(sendUrl,command,mac,result,type,orderNo,userName);
        //记录命令
        if (!HttpStatus.IOT_SUCCESS.equals(code))
        {
            // 异步更新在线状态
            asynchronousUpdateOnlineStatus(mac);
            throw new ServiceException(code+"-----"+ IotUtil.formatMsg(code));
        }
    }

    private void asynchronousUpdateOnlineStatus(String mac) {
        RlDevice device = deviceMapper.selectDeviceByMac(mac);
        //异步更新在线状态
        scheduledExecutorService.schedule(() -> {
            if(device.getOnlineStatus().equals(ServiceConstants.VEHICLE_STATUS_ONLINE)){
                log.info("【接收onenet推送】异步更新在线状态"+JSON.toJSONString(device));
                RlDevice updateDevice = new RlDevice();
                updateDevice.setDeviceId(device.getDeviceId());
                updateDevice.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_OFFLINE);
                int i = deviceMapper.updateDevice(updateDevice);
                if(i>0){
                    log.info("【接收onenet推送】异步保存在线状态成功");
                }
            }
            RlOnlineLog etOnlineLog = new RlOnlineLog();
            etOnlineLog.setMac(device.getMac());
            etOnlineLog.setSn(device.getSn());
            etOnlineLog.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_OFFLINE);
            etOnlineLog.setLongitude(device.getLongitude());
            etOnlineLog.setLatitude(device.getLatitude());
            etOnlineLog.setCreateTime(DateUtils.getNowDate());
            int i1 = eOnlineLogService.insertEtOnlineLog(etOnlineLog);
            if(i1>0){
                log.info("【接收onenet推送】异步保存在线日志成功");
            }
        }, 0, TimeUnit.SECONDS);
    }

    /* 异步保存发送命令日志*/
    private void asynchronousSaveLog(String url,String command,String mac,String result,String type,String orderNo,String userName) {
        //异步保存发送命令日志
        scheduledExecutorService.schedule(() -> {
            RlCommandLog etCommandLog = new RlCommandLog();
            etCommandLog.setUrl(url);
            etCommandLog.setCommand(command);
            etCommandLog.setType(type);
            etCommandLog.setMac(mac);
            RlDevice device = deviceMapper.selectDeviceByMac(mac);
            etCommandLog.setSn(device.getSn());
            etCommandLog.setResult(result);
            etCommandLog.setLongitude(device.getLongitude());
            etCommandLog.setLatitude(device.getLatitude());
            etCommandLog.setCreateTime(DateUtils.getNowDate());
            JSONObject paramsObj = JSON.parseObject(result);
            String code = paramsObj.getString("code");
            etCommandLog.setCallStatus(code);
            etCommandLog.setOrderNo(orderNo);
            etCommandLog.setCreateBy(userName);
            int i = rlCommandLogMapper.insertEtCommandLog(etCommandLog);
            if(i>0){
                log.info("【发送命令】异步保存发送命令日志");
            }
        }, 0, TimeUnit.SECONDS);
    }

    @Override
    /** 发送命令*/
    public ResponseVo sendCommandWithResp(String mac, String token,String command,String type,String orderNo) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        ResponseVo responseVo = JSON.parseObject(result, ResponseVo.class);

        if (responseVo.getCode() == 10500) { // 超时
            log.info("第一次请求超时，进行第二次请求...");
            result = HttpUtils.sendPostWithToken(sendUrl, command, token);
            log.info("【" + type + "】===>IOT第二次请求调用结果:【{}】", result);
            responseVo = JSON.parseObject(result, ResponseVo.class);
        }

        if(responseVo.getCode() != 0){
            asynchronousUpdateOnlineStatus(mac);
        }
        asynchronousSaveLog(sendUrl,command,mac,result,type,orderNo,null);
        return JSON.parseObject(result,ResponseVo.class);
    }

    /** 查询数据点*/
    @Override
    public DataPointRes historyDatapoints(String mac, String token) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_CURRENT_DATAPOINTS + "?"+param;
        String result = HttpUtils.sendGetWithToken(sendUrl, null, token);
        log.info("【查询数据点】===>IOT请求调用结果:【{}】",result);
        return JSON.parseObject(result,DataPointRes.class);
    }

    /**
     * 响铃寻车
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean ring(String sn) {
        RlDevice asDevice = deviceMapper.selectDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY1,"响铃寻车",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("响铃寻车失败");
        return Boolean.TRUE;
    }

    /**
     * 开坐垫锁
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean seatCushionLock(String sn) {
        RlDevice rlDevice = deviceMapper.selectDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(rlDevice.getMac(), token,IotConstants.COMMAND_HPEN,"开坐垫锁",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("开坐垫锁失败");
        return Boolean.TRUE;
    }

    /**
     * 重启设备
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean reboot(String sn) {
        RlDevice asDevice = deviceMapper.selectDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY1,"重启设备",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("重启设备失败");
        return Boolean.TRUE;
    }

    /**
     * 重启设备
     * @param mac
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean rebootByMac(String mac) {
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_REBOOT,"重启设备",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("重启设备失败");
        return Boolean.TRUE;
    }

    /**
     * 响铃寻车
     * @param mac
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean ringByMac(String mac) {
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_PLAY1,"响铃寻车",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("响铃寻车失败");
        return Boolean.TRUE;
    }


    /**
     * 坐垫锁用mac
     * @param mac
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean seatCushionLockByMac(String mac) {
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_HPEN,"开坐垫锁",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("开坐垫锁失败");
        return Boolean.TRUE;
    }

    /**
     * 临时锁车
     * @param orderNo
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean lock(String sn,String orderNo,String isBluetooth,String lon,String lat) {
        RlOrderVO order = orderService.selectRlOrderByOrderNo(orderNo);
        if(StrUtil.isBlank(sn)) {
            sn = order.getSn();
        }
        RlDevice asDevice = deviceMapper.selectDeviceBySn(sn);
        RlDevice updateDevice = new RlDevice();
        updateDevice.setDeviceId(asDevice.getDeviceId());
        /** 1.获取token*/
        String token = Token.getToken();
        if(!"true".equals(isBluetooth)){
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_LLOSE+IotConstants.COMMAND_FREQUENCY_3600,"临时锁车",orderNo,order.getUserName());
        }else{
            updateDevice.setLongitude(lon);
            updateDevice.setLatitude(lat);
            updateDevice.setLastTime(DateUtils.getNowDate());
        }
        updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        if(StrUtil.isNotBlank(orderNo)){//有订单号，则是用户临时锁车
            /**  改变车辆状态：4-临时锁车 */
            updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);//临时锁车
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(orderNo, sn,ServiceConstants.TRIP_LOG_TYPE_TEMPORARILY_LOCK);
            if(tripLog==0){
                log.info("【临时锁车】记录行程失败");
                throw new ServiceException("【临时锁车】记录行程失败");
            }
        }
        int device = deviceMapper.updateDevice(updateDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时锁车】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 管理员锁车
     * @param sn
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean adminLock(String sn,String userName) {
        if(StrUtil.isBlank(sn))throw new ServiceException("sn不能为空");
        RlDevice rlDevice = deviceMapper.selectDeviceBySn(sn);
        /** 2.发送命令*/
        sendCommand(rlDevice.getMac(), Token.getToken(),IotConstants.COMMAND_CLOSE+IotConstants.COMMAND_FREQUENCY_3600,"管理员锁车",null,userName);
        rlDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        int device = deviceMapper.updateDevice(rlDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时锁车】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 管理员锁车
     * @param mac
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean adminLockByMac(String mac,String userName) {
        if(StrUtil.isBlank(mac))throw new ServiceException("mac不能为空");
        RlDevice asDevice = deviceMapper.selectDeviceByMac(mac);
        /** 2.发送命令*/
        sendCommand(asDevice.getMac(), Token.getToken(),IotConstants.COMMAND_CLOSE+IotConstants.COMMAND_FREQUENCY_3600,"管理员锁车",null,userName);
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        int device = deviceMapper.updateDevice(asDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时锁车】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 临时解锁
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean unlock(String sn, String orderNo,String isBluetooth,String lon,String lat) {
        RlOrderVO order = orderService.selectRlOrderByOrderNo(orderNo);
        if(order==null)throw new ServiceException("订单不存在");
        if(StrUtil.isBlank(sn)) {
            sn = order.getSn();
        }
        RlDevice asDevice = deviceMapper.selectDeviceBySn(sn);
        RlDevice updateDevice = new RlDevice();
        updateDevice.setDeviceId(asDevice.getDeviceId());
        /** 1.获取token*/
        String token = Token.getToken();
        String finalSn = sn;
        if(!"true".equals(isBluetooth)){
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_5,"临时解锁",orderNo,order.getUserName());
        }else{
            updateDevice.setLongitude(lon);
            updateDevice.setLatitude(lat);
            updateDevice.setLastTime(DateUtils.getNowDate());
        }
        if(StrUtil.isNotBlank(orderNo)){//有订单号，则是用户骑行中解锁
            /**  改变车辆状态：3-骑行中 */
            updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);//骑行中
            updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(orderNo, finalSn,ServiceConstants.TRIP_LOG_TYPE_TEMPORARILY_UNLOCK);
            if(tripLog==0){
                log.info("【临时解锁】记录行程失败");
                throw new ServiceException("【临时解锁】记录行程失败");
            }
        }else{
            /**  改变车辆锁状态：1-开  */
            updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
        }
        int device = deviceMapper.updateDevice(updateDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时解锁】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 还车
     *
     * 1. 校验
     * 2. 远程关锁
     * 3. 更新订单
     * 4. 更新车辆
     * 5. 记录行程
     * 6. 退押金
     */
    @SneakyThrows
    @Override
    public Boolean returnVehicle(RlOrderQuery orderQuery) {
        String orderNo = orderQuery.getOrderNo();
        if(StrUtil.isNotBlank(orderNo)){
            RlOrderVO order = orderService.selectRlOrderByOrderNo(orderNo);
            if(ObjectUtil.isNull(order)){
                throw new ServiceException("订单不存在");
            }
            if(!ServiceConstants.ORDER_STATUS_IN_USE.equals(order.getStatus())){
                throw new ServiceException("该订单状态非使用中");
            }
            if(StrUtil.isBlank(orderQuery.getReturnLon()) || StrUtil.isBlank(orderQuery.getReturnLat())){
                throw new ServiceException("手机经纬度不能为空");
            }
            RlOrder updateOrder = new RlOrder();
            /** 1. 记录还车时间*/
            updateOrder.setOrderId(order.getOrderId());
            updateOrder.setReturnType(orderQuery.getReturnType());
            updateOrder.setReturnTime(DateUtils.getNowDate());
            updateOrder.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_AUDITED);// 还车后订单状态变成待审核，只有商户确认才算完成
            updateOrder.setReturnTime(DateUtils.getNowDate());
            String token = Token.getToken();

            RlDevice device = deviceMapper.selectDeviceBySn(order.getSn());
            RlDevice updateDevice = new RlDevice();
            updateDevice.setDeviceId(device.getDeviceId());
            if(ServiceConstants.RETURN_TYPE_NORMAL.equals(orderQuery.getReturnType())){
                /** 2. 车辆远程关锁*/
                ResponseVo responseVo = sendCommandWithResp(device.getMac(), token, IotConstants.COMMAND_CLOSE + IotConstants.COMMAND_FREQUENCY_3600, "还车关锁",orderNo);
                if(responseVo.getCode()!=0){
                    log.info("【还车关锁】远程关锁失败");
                    throw new ServiceException("远程关锁失败");
                }

                /** 2.1.判断是否在店铺附近 先根据手机定位判断，再跟进车辆定位判断，如果两个都不在，则提示不在附近*/
                boolean nearStore = isNearStore(orderQuery, device);
                if(!nearStore){
                    throw new ServiceException("不在店铺附近，请前往店铺还车");
                }
            }else{
                ResponseVo responseVo = sendCommandWithResp(device.getMac(), token, IotConstants.COMMAND_CLOSE + IotConstants.COMMAND_FREQUENCY_3600, "辅助还车关锁",orderNo);
                if(responseVo.getCode()!=0){
                    log.info("【还车关锁】远程关锁失败");
                }
            }
            /** 4. 更新车辆状态*/
            updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);
            updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
            int deviceUpdate = deviceMapper.updateDevice(updateDevice);
            if(deviceUpdate==0){
                log.info("【还车关锁】更新车辆状态失败");
                throw new ServiceException("【还车关锁】更新车辆状态失败");
            }
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(order.getOrderNo(),device.getSn(),ServiceConstants.TRIP_LOG_TYPE_RETRUN_LOCK);
            if(tripLog==0){
                log.info("【还车关锁】记录行程失败");
                throw new ServiceException("【还车关锁】记录行程失败");
            }
            /** 6.计算逾期费用 */
            BigDecimal overdueFee= BigDecimal.ZERO;
            if(ObjectUtil.isNull(order.getExpiryTime())){
               throw new ServiceException("订单没有到期时间");
            }
            if (order.getExpiryTime().before(DateUtils.getNowDate())) {
                overdueFee = computeOverdueFee(order, updateOrder);
            }
            updateOrder.setOverdueFee(overdueFee);
            int i = orderService.updateRlOrder(updateOrder);
            if(i==0){
                throw new ServiceException("更新订单状态失败");
            }
            /** 7. 记录订单履历*/
            if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_RETURN_END,
                    order.getStatus(),updateOrder.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"已还车")){
                throw new ServiceException("【记录订单履历失败】");
            }
            log.info("还车成功");
            return Boolean.TRUE;
        }else{
            throw new ServiceException("订单号不能为空");
        }
    }

    /**
     * 判断是否在店铺附近
     */
    @Override
    public Boolean isNearStore(RlOrderQuery orderQuery, RlDevice device) {
        if (device.getStoreId() != null) {
            StoreVo store = storeService.selectSmStoreById(orderQuery.getReturnStoreId());
            if (store != null) {
                BigDecimal lng = store.getLng();
                BigDecimal lat = store.getLat();
                String lon2 = orderQuery.getReturnLon(); // 手机位置
                String lat2 = orderQuery.getReturnLat();
                // 获取可还车的最大距离
                double storeDistance = Double.parseDouble(sysConfigService.selectConfigByKey("store.distance"));

                // 计算手机与店铺的距离
                double distance = GeoUtils.calculateDistance(lat.doubleValue(), lng.doubleValue(), Double.valueOf(lat2), Double.valueOf(lon2));
                log.info("【还车】手机与店铺距离: {} 米", distance);

                if (distance <= storeDistance) {
                    return true;
                }
                // 获取车辆的最新定位
                double[] latestLocation = getLatestLocation(device.getMac());
                if (ObjectUtil.isNotNull(latestLocation)) {
                    double longitude1 = latestLocation[1];
                    double latitude1 = latestLocation[0];

                    log.info("【还车时根据车辆定位判断是否在停车区】，车辆经度: {}，纬度: {}", longitude1, latitude1);

                    // 计算车辆与店铺的距离
                    double distanceToStore = GeoUtils.calculateDistance(lat.doubleValue(), lng.doubleValue(), latitude1, longitude1);
                    log.info("【还车】车辆与店铺距离: {} 米", distanceToStore);

                    return distanceToStore <= storeDistance;
                }
            }
        }
        return false;
    }


    @SneakyThrows
    private double[] getLatestLocation(String mac) {
        RlDevice device = deviceMapper.selectDeviceByMac(mac);
        String token = Token.getToken();
        DataPointRes datapoints = historyDatapoints(mac, token);
        if(datapoints.getCode() == 0){
            Data data = datapoints.getData();
            List<Datastream> datastreams = data.getDevices();
            for (Datastream datastream: datastreams) {
                List<Datapoint> datapointList = datastream.getDatastreams();
                if(ObjectUtil.isNotNull(datapointList)){
                    for (Datapoint datapoint:datapointList) {
                        if(datapoint.getId().equals(IotConstants.ONENET_LOCATION)){
                            String string = JSON.toJSONString(datapoint.getValue());
                            String at = datapoint.getAt();
                            if(StrUtil.isNotBlank(string)){
                                LocationVo locationVo = JSONObject.parseObject(string, LocationVo.class);
                                log.info("【手动更新】： locationVo---【{}】" , JSON.toJSONString(locationVo));
                                double[] doubles = CommonUtil.coordinateConvert(locationVo.getLon(), locationVo.getLat());
                                BigDecimal lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
                                BigDecimal lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
                                log.info("转换后的GCJ02经纬度：" + lon + "---" + lat);
                                if(BigDecimal.ZERO.compareTo(lon) != 0 && BigDecimal.ZERO.compareTo(lat) != 0){
                                    /** 计算电量和里程后更新设备*/
                                    int i = updateDevice(at,locationVo, device, lon, lat);
                                    return doubles;
                                }else{
                                    noLocationUpdateDevice(at,locationVo,device);
                                }
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    /** 计算电量和里程后更新设备*/
    public int updateDevice(String at,LocationVo locationVo, RlDevice device, BigDecimal lon, BigDecimal lat) {
        device.setLatitude(lat.toString());
        device.setLongitude(lon.toString());
        Integer bat = locationVo.getBat();
        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
        log.info("保存电压：" + divide);
        device.setVoltage(divide.toString());//电压
        // 根据电压计算续航里程
        RlModelVO model = modelService.selectEModelByModelId(device.getModelId());
        if(ObjectUtil.isNotNull(model)){
            Integer remainingMileage = 0;
            if(StrUtil.isNotBlank(device.getVoltage())){
                remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
            }
            Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
//            device.setRemainingMileage(remainingMileage);
            device.setRemainingPower(electricQuantity.toString());
        }
        device.setLastTime(DateUtils.parseDate(at));
        device.setLastLocationTime(DateUtils.parseDate(at));
        device.setGps("1");
        // 信号强度
        device.setSignalStrength(locationVo.getCsq());
        device.setSatellites(locationVo.getS());
        device.setQuality(locationVo.getQ());
        int i = updateLocation(device);
        return i;
    }

    /** 无定位更新设备 */
    private void noLocationUpdateDevice(String at,LocationVo locationVo, RlDevice device) {
        Integer bat = locationVo.getBat();
        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
        log.info("保存电压：" + divide);
        device.setVoltage(divide.toString());//电压
        RlModelVO model = modelService.selectEModelByModelId(device.getModelId());
        // 根据电压计算续航里程
        if(ObjectUtil.isNotNull(model)){
            Integer remainingMileage = 0;
            if(StrUtil.isNotBlank(device.getVoltage())){
                remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
            }
            Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
//            device.setRemainingMileage(remainingMileage);
            device.setRemainingPower(electricQuantity.toString());
        }
        device.setLastTime(DateUtils.parseDate(at));
        device.setGps("0");
        device.setSignalStrength(locationVo.getCsq());
        device.setSatellites(0);
        device.setQuality(locationVo.getQ());
        int i = updateLocation(device);
        if(i>0){
            log.info("未获取到定位===============保存电压等数值成功===========>" + device.getSn());
        }
    }

    /** 计算逾期费用 */
    public BigDecimal computeOverdueFee(RlOrderVO order, RlOrder updateOrder) {
        String outUnit = order.getOutUnit();
        BigDecimal outPrice = order.getOutPrice();
        // 计算逾期的时间单位
        Date nowDate = DateUtils.getNowDate();
        int overdueUnits = 0;

        // 根据不同单位计算逾期的具体时间差
        switch (outUnit) {
            case "minutes":
                overdueUnits = DateUtils.timeDifferenceInMinutes(nowDate, order.getExpiryTime()); // 计算分钟差
                break;
            case "hours":
                overdueUnits = DateUtils.timeDifferenceInHours(nowDate, order.getExpiryTime()); // 计算小时差
                break;
            case "day":
                overdueUnits = DateUtils.differentDaysByMillisecond(nowDate, order.getExpiryTime()); // 计算天数差
                break;
            default:
                throw new ServiceException("【还车】未知的逾期单位：" + outUnit);
        }
        // 计算逾期费用 = 逾期的单位数量 * 每单位价格
        BigDecimal overdueFee = outPrice.multiply(new BigDecimal(overdueUnits));
        updateOrder.setOverdueFee(overdueFee);
        return overdueFee;
    }
//
//    /** 押金抵扣 */
//    private void depositDeduction(EtOrder order) {
//        EtFeeRule rule = etFeeRuleService.selectEtFeeRuleByRuleIdIncludeDelete(order.getRuleId());
//        Integer autoRefundDeposit = rule.getAutoRefundDeposit();
//        log.info("【还车关锁】进入押金抵扣环节--------"+autoRefundDeposit+"个小时后退还押金");
//        if(autoRefundDeposit!=null){
//            //创建一个定时器，计算出退还时间后，执行退款操作
//            // 往后推autoRefundDeposit小时执行
//            EtOrder finalOrder = order;
//            scheduledExecutorService.schedule(() -> {
//                /** 2. 获取最后一次押金*/
//                EtOrder depositOrder = etOrderService.getDepositOrder(finalOrder.getUserId());
//                BigDecimal deposit = depositOrder.getTotalFee();
//                BigDecimal leaseFee = finalOrder.getTotalFee();
//                BigDecimal afterDeductionFee;
//                String mark;
//                if(deposit.compareTo(leaseFee) <= 0){// 骑行费大于押金，订单为未支付, 抵扣后的金额 = 骑行费 - 押金
//                    afterDeductionFee = BigDecimal.ZERO;
//                    mark = "押金抵扣成功，骑行费【"+leaseFee+"】大于押金【"+deposit+"】";
//                    finalOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING_END);
//                    finalOrder.setPayFee(deposit);
//                }else{
//                    // 押金大于订单金额  扣除后
//                    afterDeductionFee = deposit.subtract(leaseFee);
//                    mark = "押金抵扣成功，骑行费【"+leaseFee+"】小于押金【"+deposit+"】,扣除后金额【"+afterDeductionFee+"】";
//                    finalOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
//                }
//                /** 更新骑行订单*/
//                finalOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
//                finalOrder.setPayTime(DateUtils.getNowDate());
//                finalOrder.setPayType(ServiceConstants.PAY_TYPE_YJ);
//                finalOrder.setMark(mark);
//                finalOrder.setDepositDeduction(ServiceConstants.IS_DEPOSIT_DEDUCTION);
//                /** 更新押金订单*/
//                depositOrder.setDepositDeduction(ServiceConstants.IS_DEPOSIT_DEDUCTION);
//                int updateEtOrder1 = etOrderMapper.updateEtOrder(depositOrder);
//                if(updateEtOrder1 == 0){
//                    throw new ServiceException("押金抵扣失败，更新押金订单失败");
//                }
//                if(afterDeductionFee.compareTo(BigDecimal.ZERO) > 0){
//                    /** 退款剩余押金*/
//                    Refund refund = wxPayService.refund(depositOrder, "押金抵扣退款",afterDeductionFee,IdUtils.getOrderNo("ref"));
//                    /** 2.记录退款表  创建退款对象*/
//                    depositOrder.setReason("押金抵扣退款");
//                    EtRefund refund1= etOrderService.createRefund(depositOrder, afterDeductionFee, null, null, null, null, refund.getOutRefundNo(),ServiceConstants.REFUND_TYPE_DEPOSIT);
//                    int i = etRefundService.insertEtRefund(refund1);
//                    if(i == 0){
//                        log.info("【押金抵扣】保存退款对象失败");
//                        throw new ServiceException("【押金抵扣】，保存退款对象失败");
//                    }
//                    /** 更新用户余额*/
//                    AsUser asUser = asUserService.selectUserById(depositOrder.getUserId());
//                    if(asUser!=null){
//                        // 更新用户并更新缓存
//                        asUser.setBalance(BigDecimal.ZERO);
//                        if (asUserService.updateUserProfile(asUser) > 0)
//                        {
//                            log.info("【押金抵扣成功】更新用户信息成功:"+ JSON.toJSON(asUser));
//                        }else{
//                            throw new ServiceException("【押金抵扣】，更新用户信息失败");
//                        }
//                    }
//                }
//                /** 押金抵扣后生成资金流水记录 */
//                callbackService.capitalFlowRecords(finalOrder,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_RIDING,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YJ);
//            }, autoRefundDeposit , TimeUnit.HOURS);
//        }
//    }
//    /**
//     * 格式化单位
//     */
//    private String formatUnit(String rentalUnit) {
//        if(ServiceConstants.RENTAL_UNIT_MINUTES.equals(rentalUnit)){
//            return "分钟";
//        }else if(ServiceConstants.RENTAL_UNIT_HOURS.equals(rentalUnit)){
//            return "小时";
//        } else if(ServiceConstants.RENTAL_UNIT_DAY.equals(rentalUnit)){
//            return "天";
//        } else {
//            return "";
//        }
//    }
//
//    /**
//     * 是否在停车区内
//     */
//    private Boolean isParkingZone(String sn,Long areaId) {
//        Boolean inCircle = false;
//        EtParkingArea parkingArea = new EtParkingArea();
//        parkingArea.setAreaId(areaId);
//        parkingArea.setStatus("0");
//        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
//        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
//        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
//            log.info("运营区【{}】没有停车区,",areaId);
//            throw new ServiceException("运营区【{"+area.getAreaName()+"}】没有停车区");
//        }
//        double tolerance = area.getError(); // 误差距离
//        for (EtParkingArea etParkingArea : parkingAreas) {
//            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_PARKFING)){
//                AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//                String latitude = device.getLatitude();
//                String longitude = device.getLongitude();
//                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
//                if(ObjectUtil.isNotNull(etParkingArea.getError())){
//                    tolerance = etParkingArea.getError();
//                }
//                inCircle = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry, tolerance);
//                if(inCircle){
//                    log.info("车辆【{}】在停车区【{}】内",sn,etParkingArea.getParkingName());
//                    inCircle = true;
//                    break;
//                }else{
//                    log.info("车辆【{}】不在停车区【{}】内",sn,etParkingArea.getParkingName());
//                }
//            }
//        }
//        return inCircle;
//    }
//
//    private Boolean isParkingZoneByLocation(String longitude, String latitude,Long areaId) {
//        Boolean inCircle = false;
//        EtParkingArea parkingArea = new EtParkingArea();
//        parkingArea.setAreaId(areaId);
//        parkingArea.setStatus("0");
//        parkingArea.setType(ServiceConstants.PARKING_AREA_TYPE_PARKFING);
//        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
//        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
//        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
//            log.info("运营区【{}】没有停车区,",areaId);
//            throw new ServiceException("运营区【{}】没有停车区"+areaId.toString());
//        }
//        double tolerance = area.getError(); // 误差距离
//        for (EtParkingArea etParkingArea : parkingAreas) {
//            Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
//            if(ObjectUtil.isNotNull(etParkingArea.getError())){
//                tolerance = etParkingArea.getError();
//            }
//            inCircle = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry, tolerance);
//            if(inCircle){
//                log.info("位置【{}，{}】在停车区【{}】内",longitude,latitude,etParkingArea.getParkingName());
//                inCircle = true;
//                break;
//            }else{
//                log.info("位置【{}，{}】在停车区【{}】内",longitude,latitude,etParkingArea.getParkingName());
//            }
//        }
//        return inCircle;
//    }
//
//    /**
//     * 是否在运营区内
//     */
//    @Override
//    public Boolean isAreaZone(String sn,EtOperatingArea area) {
//        Boolean inCircle  = false;
//        AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//        String latitude = device.getLatitude();
//        String longitude = device.getLongitude();
//        Geometry geometry = GeoUtils.fromWkt(area.getBoundary());
//        inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
//        if(inCircle){
//            log.info("车辆【{}】在运营区【{}】内",sn,area.getAreaName());
//            inCircle = true;
//        }else{
//            log.info("车辆【{}】不在运营区【{}】内",sn,area.getAreaName());
//        }
//        return inCircle;
//    }
//
//    /**
//     * 是否禁停区内
//     */
//    public boolean isNoParkingArea(String sn,Long areaId) {
//        Boolean inCircle = false;
//        EtParkingArea parkingArea = new EtParkingArea();
//        parkingArea.setAreaId(areaId);
//        parkingArea.setStatus("0");
//        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
//        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
//            log.info("运营区【{}】没有禁停区,",areaId);
//            return false;
////            throw new ServiceException("运营区【{}】没有禁停区"+areaId.toString());
//        }
//        for (EtParkingArea etParkingArea : parkingAreas) {
//            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_NO_PARKFING)){
//                AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//                String latitude = device.getLatitude();
//                String longitude = device.getLongitude();
//                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
//                inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
//                if(inCircle){
//                    log.info("车辆【{}】在禁停区【{}】内",sn,etParkingArea.getParkingName());
//                    inCircle = true;
//                    break;
//                }else{
//                    log.info("车辆【{}】不在禁停区【{}】内",sn,etParkingArea.getParkingName());
//                }
//            }
//        }
//        return inCircle;
//    }
//
//    /**
//     * 是否禁行区内
//     */
//    @Override
//    public boolean isNoRidingArea(String sn,Long areaId) {
//        Boolean isNoRiding = false;
//        EtParkingArea parkingArea = new EtParkingArea();
//        parkingArea.setAreaId(areaId);
//        parkingArea.setStatus("0");
//        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
//        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
//            log.info("运营区【{}】没有禁行区,",areaId);
//            throw new ServiceException("运营区【{}】没有禁行区"+areaId.toString());
//        }
//        for (EtParkingArea etParkingArea : parkingAreas) {
//            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_BANNED_RIDING)){
//                AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//                String latitude = device.getLatitude();
//                String longitude = device.getLongitude();
//                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
//                isNoRiding = GeoUtils.isInCircle(longitude, latitude, geometry);
//                if(isNoRiding){
//                    log.info("车辆【{}】在禁行区【{}】内",sn,etParkingArea.getParkingName());
//                    isNoRiding = true;
//                    break;
//                }else{
//                    log.info("车辆【{}】不在禁行区【{}】内",sn,etParkingArea.getParkingName());
//                }
//            }
//        }
//        return isNoRiding;
//    }
//
//    /**
//     * 是否禁行区内
//     */
//    @Override
//    public boolean isNoRidingAreaWithTolerance(String sn,Long areaId,int tolerance) {
//        Boolean isNoRiding = false;
//        EtParkingArea parkingArea = new EtParkingArea();
//        parkingArea.setAreaId(areaId);
//        parkingArea.setStatus("0");
//        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
//        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
//            log.info("运营区【{}】没有禁行区,",areaId);
//            throw new ServiceException("运营区【{}】没有禁行区"+areaId.toString());
//        }
//        for (EtParkingArea etParkingArea : parkingAreas) {
//            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_BANNED_RIDING)){
//                AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//                String latitude = device.getLatitude();
//                String longitude = device.getLongitude();
//                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
//                isNoRiding = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry,tolerance);
//                if(isNoRiding){
//                    log.info("车辆【{}】在禁行区（{}米）【{}】内",sn,tolerance,etParkingArea.getParkingName());
//                    isNoRiding = true;
//                    break;
//                }else{
//                    log.info("车辆【{}】不在禁行区（{}米）【{}】内",sn,tolerance,etParkingArea.getParkingName());
//                }
//            }
//        }
//        return isNoRiding;
//    }
//
//    /**
//     * 是否低电量
//     */
//    @Override
//    public boolean isLowBattery(String sn) {
//        AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//        String remainingPower = device.getRemainingPower();
//        log.info("车辆【{}】电量为【{}】",sn,remainingPower);
//        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
//        if(Integer.parseInt(remainingPower) < Integer.parseInt(area.getUndercharge())){
//            log.info("车辆【{}】电量为【{}】低于【{}】",sn,remainingPower,area.getUndercharge());
//            return true;
//        }
//        return false;
//    }
//
//    /**
//     * 判断是否在运营时间内
//     */
//    @Override
//    public Boolean isOperatingTime(String sn) {
//        AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
//        if(area.getAreaTime().equals(ServiceConstants.AREA_TIME_CUSTOM) && !DateUtils.isInTime(area.getAreaTimeStart(),area.getAreaTimeEnd())){
//            log.info("不在运营时间内，运营时间为：【{}】--【{}】",area.getAreaTimeStart(),area.getAreaTimeEnd());
//            return Boolean.FALSE;
//        }
//        return Boolean.TRUE;
//    }
//
//    /**
//     * 获取所有车辆
//     */
//    @Override
//    public List<AsDevice> allVehicleByArea(Long areaId) {
//        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
//        wrapper.eq("area_id",areaId);
//        wrapper.eq("status","1");
//        List<AsDevice> asDevices = eDeviceMapper.selectList(wrapper);
//        // for循环
//        for (AsDevice asDevice : asDevices) {
//            EtModel model = etModelService.selectEtModelByModelId(asDevice.getModelId());
//            Integer remainingMileage = CommonUtil.getRemainingMileage(asDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
//            asDevice.setRemainingMileage(remainingMileage);
//        }
//        return asDevices;
//    }
//
    /**
     * 更新位置信息
     */
    @Override
    public int updateLocation(RlDevice device) {
        UpdateWrapper<RlDevice> wrapper = new UpdateWrapper<>();
        wrapper.eq("sn",device.getSn())
                .set("longitude",device.getLongitude())
                .set("latitude",device.getLatitude())
                .set("remaining_power",device.getRemainingPower())
                .set("last_time",device.getLastTime())
                .set("last_location_time",device.getLastLocationTime())
                .set("voltage",device.getVoltage())
                .set("satellites",device.getSatellites())
                .set("quality",device.getQuality())
                .set("gps",device.getGps())
                .set("signal_strength",device.getSignalStrength());
        return deviceMapper.update(null,wrapper);
    }

    /**
     * 判断是否在线
     */
    @SneakyThrows
    @Override
    public boolean isOnline(String sn) {
        RlDevice device = deviceMapper.selectDeviceBySn(sn);
        ResponseVo responseVo = sendCommandWithResp(device.getMac(), Token.getToken(), "111", "是否在线",null);
        if(responseVo.getCode() == 0){
            return true;
        }
        asynchronousUpdateOnlineStatus(device.getMac());
        return false;
    }

    /**
     * 检查SN是否唯一
     */
    @Override
    public boolean checkSNUnique(RlDevice asDevice) {
        Long deviceId = StringUtils.isNull(asDevice.getDeviceId()) ? -1L : asDevice.getDeviceId();
        RlDevice info = deviceMapper.checkSNUnique(asDevice.getSn());
        if (StringUtils.isNotNull(info) && info.getDeviceId().longValue() != deviceId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 检查SN是否唯一
     */
    @Override
    public boolean checkMACUnique(RlDevice asDevice) {
        Long deviceId = StringUtils.isNull(asDevice.getDeviceId()) ? -1L : asDevice.getDeviceId();
        RlDevice info = deviceMapper.checkMACUnique(asDevice.getMac());
        if (StringUtils.isNotNull(info) && info.getDeviceId().longValue() != deviceId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 根据mac号判断是否有绑定过
     */
    @Override
    public boolean isBandByMac(String mac) {
        RlDevice info = deviceMapper.checkMACUnique(mac);
        if (StringUtils.isNull(info))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

//    /**
//     * 判断是否在停车区申请人
//     */
//    @Override
//    public IsInParkingAreaVo isInParkingArea(String longitude, String latitude,String areaId,String sn){
//        // 线判断车的定位是否在停车区内，如果在停车区内则直接返回，如果不在拿手机的定位再判断是否在停车区内
//        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(Long.parseLong(areaId));
//        IsInParkingAreaVo isInParkingAreaVo = new IsInParkingAreaVo();
//        if(StringUtils.isNotEmpty(sn)){
//            AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//            String longitude1 = device.getLongitude();
//            String latitude1 = device.getLatitude();
//            Boolean parkingZoneByLocation = isParkingZoneByLocation(longitude1, latitude1, Long.parseLong(areaId));
//            if(parkingZoneByLocation){
//                isInParkingAreaVo.setIsInParkingArea(parkingZoneByLocation);
//                isInParkingAreaVo.setParkingReturn("1".equals(area.getParkingReturn()));
//                return isInParkingAreaVo;
//            }
//        }
//        Boolean parkingZoneByLocation = isParkingZoneByLocation(longitude, latitude, Long.parseLong(areaId));
//        isInParkingAreaVo.setIsInParkingArea(parkingZoneByLocation);
//        //停车点还车：0-关闭；1-开启
//        isInParkingAreaVo.setParkingReturn("1".equals(area.getParkingReturn()));
//        return isInParkingAreaVo;
//    }

    /**
     * 查询版本并更新
     */
    @Override
    @SneakyThrows
    public boolean updateVersion(String sn) {
        RlDevice device = deviceMapper.selectDeviceBySn(sn);
        if(device!=null){
            String version = device.getVersion();
            if(StrUtil.isBlank(version)){
                String token = Token.getToken();
                DataPointRes datapoints = historyDatapoints(deviceMapper.selectDeviceBySn(sn).getMac(), token);
                if(datapoints.getCode() == 0){
                    Data data = datapoints.getData();
                    List<Datastream> datastreams = data.getDevices();
                    for (Datastream datastream: datastreams) {
                        List<Datapoint> datapointList = datastream.getDatastreams();
                        if(ObjectUtil.isNotNull(datapointList)){
                            for (Datapoint datapoint:datapointList) {
                                if(datapoint.getId().equals(IotConstants.ONENET_VER)){
                                    Integer value = (Integer)datapoint.getValue();
                                    log.info("【查询数据点】===>获取到版本值：【{}】", value);
                                    if(value != null){
                                        RlDevice device1 = new RlDevice();
                                        device1.setVersion(value.toString());
                                        device1.setSn(sn);
                                        if(deviceMapper.updateDeviceBySn(device1) > 0){
                                            log.info("【查询数据点】===>更新版本成功");
                                            return true;
                                        }else{
                                            log.info("【查询数据点】===>更新版本失败");
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 根据店铺id查询可租车辆数
     */
    @Override
    public Integer selectRentalDeviceCountByStoreId(Long storeId,Long modelId) {
        return deviceMapper.selectRentalDeviceCountByStoreId(storeId,modelId);
    }

    /**
     * 根据店铺id查询所有车辆数
     */
    @Override
    public Integer selectAllDeviceCountByStoreId(Long storeId) {
        return deviceMapper.selectAllDeviceCountByStoreId(storeId);
    }


    /**
     * sn和mac号绑定
     * 注册onenet设备
     */
    @SneakyThrows
    @Override
    public int bandSn(RlDevice rlDevice) {
        RlDevice device = deviceMapper.selectDeviceByMac(rlDevice.getMac());
        if(StringUtils.isNotNull(device)){
            device.setSn(rlDevice.getSn());
            device.setHardwareVersionId(rlDevice.getHardwareVersionId());
            int i = deviceMapper.updateDeviceBySn(device);
            if(i>0){
                log.info("【sn和mac号绑定】===>mac【{}】已经绑定过:更新sn【{}】成功",device.getMac(),device.getSn());
            }
        }else{
            // 调用onenet接口
            CreateDeviceVo createDeviceVo = new CreateDeviceVo();
            createDeviceVo.setDevice_name(rlDevice.getMac());
            createDeviceVo.setProduct_id(productId);
            String result = HttpUtils.sendPostWithToken(deviceUrl,JSON.toJSONString(createDeviceVo), Token.getToken());
            log.info("【sn和mac号绑定】===>IOT请求调用结果:【{}】",result);
            JSONObject paramsObj = JSON.parseObject(result);
            String code = paramsObj.getString("code");
            //记录命令
            if (!HttpStatus.IOT_SUCCESS.equals(code))
            {
                throw new ServiceException(code+"-----"+ paramsObj.getString("msg"));
            }
            try{
                int insert = deviceMapper.insert(rlDevice);
                if(insert==0){
                    throw new ServiceException("该SN已经存在，请勿重复绑定！");
                }
            }catch (Exception e){
                log.error("该SN已经存在",e.getMessage());
                throw new ServiceException("该SN已经存在，请勿重复绑定！");
            }
        }
        return 1;
    }


    /**
     * 判断是否靠近边界
     */
//    @Override
//    public boolean isCloseToTheBoundary(String sn, EtOperatingArea area) {
//        Boolean isClose = false;
//        AsDevice device = eDeviceMapper.selectAsDeviceBySn(sn);
//        String latitude = device.getLatitude();
//        String longitude = device.getLongitude();
//        Geometry geometry = GeoUtils.fromWkt(area.getBoundary());
//
//        double distanceToBoundary = GeoUtils.distanceToBoundary(longitude, latitude, geometry);
//        if(distanceToBoundary <= 3.0) {
//            log.info("车辆【{}】靠近【{}】运营边界",sn,area.getAreaName());
//            isClose = true;
//        }else{
//            log.info("车辆【{}】为靠近靠近【{}】运营边界",sn,area.getAreaName());
//        }
//        return isClose;
//    }


}
