package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetail;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetailVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.mapper.RlDividendDetailMapper;
import com.ruoyi.system.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * 分账明细Service业务层处理
 *
 * @author qzz
 * @date 2024-10-10
 */
@Slf4j
@Service
public class RlDividendDetailServiceImpl implements IRlDividendDetailService
{
    @Resource
    private RlDividendDetailMapper rlDividendDetailMapper;

    @Resource
    private IRlUserExtService userExtService;

    @Autowired
    private IRlUserService userService;

    @Autowired
    private IRlAgentService agentService;

    @Autowired
    private IRlChangeBalanceService changeBalanceService;

    /**
     * 查询分账明细
     *
     * @param id 分账明细主键
     * @return 分账明细
     */
    @Override
    public RlDividendDetail selectRlDividendDetailById(Long id)
    {
        return rlDividendDetailMapper.selectRlDividendDetailById(id);
    }

    /**
     * 查询分账明细列表
     *
     * @param rlDividendDetail 分账明细
     * @return 分账明细
     */
    @Override
    public List<RlDividendDetailVO> selectRlDividendDetailList(RlDividendDetail rlDividendDetail)
    {
        return rlDividendDetailMapper.selectRlDividendDetailList(rlDividendDetail);
    }

    /**
     * 新增分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    @Override
    public int insertRlDividendDetail(RlDividendDetail rlDividendDetail)
    {
        rlDividendDetail.setCreateTime(DateUtils.getNowDate());
        return rlDividendDetailMapper.insertRlDividendDetail(rlDividendDetail);
    }

    /**
     * 修改分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    @Override
    public int updateRlDividendDetail(RlDividendDetail rlDividendDetail)
    {
        return rlDividendDetailMapper.updateRlDividendDetail(rlDividendDetail);
    }

    /**
     * 批量删除分账明细
     *
     * @param ids 需要删除的分账明细主键
     * @return 结果
     */
    @Override
    public int deleteRlDividendDetailByIds(Long[] ids)
    {
        return rlDividendDetailMapper.deleteRlDividendDetailByIds(ids);
    }

    /**
     * 删除分账明细信息
     *
     * @param id 分账明细主键
     * @return 结果
     */
    @Override
    public int deleteRlDividendDetailById(Long id)
    {
        return rlDividendDetailMapper.deleteRlDividendDetailById(id);
    }

    /**
     * 计算分账金额
     * @param originalOrder
     * @return
     */
    @Override
    public int calculationDividend(RlOrder originalOrder,String userType) {
        RlUserExt rlUserExt;
        RlUser rlUser ;
        if(userType.equals(ServiceConstants.USER_TYPE_MERCHANT)){
            rlUserExt = userExtService.selectRlUserExtByUserId(originalOrder.getMerchantId());
            rlUser = userService.selectUserById(rlUserExt.getUserId());
        }else if(userType.equals(ServiceConstants.USER_TYPE_AGENT)){
            RlAgentVO rlAgentVO = agentService.selectRlAgentByAgentId(originalOrder.getAgentId());
            rlUserExt = userExtService.selectRlUserExtByUserId(rlAgentVO.getUserid());
            rlUser = userService.selectUserById(rlUserExt.getUserId());
        }else{
            throw new ServiceException("用户类型有误");
        }
        RlDividendDetail rlDividendDetail = new RlDividendDetail();
        rlDividendDetail.setAgentId(originalOrder.getAgentId());
        rlDividendDetail.setPartnerId(rlUserExt.getUserId());
        rlDividendDetail.setOrderNo(originalOrder.getOrderNo());
        rlDividendDetail.setTotalAmount(originalOrder.getPayFee());
        rlDividendDetail.setDividendProportion(rlUserExt.getDividendProportion());
        rlDividendDetail.setCreateTime(DateUtils.getNowDate());
        rlDividendDetail.setPartnerName(rlUser.getUserName());
        rlDividendDetail.setPartnerPhone(rlUser.getPhonenumber());
        rlDividendDetail.setPartnerType(rlUser.getUserType());
        rlDividendDetail.setStatus(ServiceConstants.DIVIDEND_STATUS_NOT_SETTLED);
        int i = insertRlDividendDetail(rlDividendDetail);
        if(i>0){
            log.info("=================【微信支付回调】计算合作人分成成功==================");
        }
        return i;
    }

    /**
     * 计算分账金额
     * @param rlDividendDetails
     * @return
     */
    @Override
    public int calculationAmount(List<RlDividendDetailVO> rlDividendDetails,BigDecimal totalDividendAmount) {
        log.info("【计算分账金额】计算分账金额开始============");
        for(RlDividendDetailVO dividendDetail:rlDividendDetails){
            log.info("【计算分账金额】租赁费和逾期费进行分成============"+totalDividendAmount);
            RlUserExt rlUserExt = userExtService.selectRlUserExtByUserId(dividendDetail.getPartnerId());
            if(ObjectUtil.isNull(rlUserExt)){
                throw new ServiceException("用户【"+dividendDetail.getPartnerName()+"】不存在");
            }
            // 保留两位小数点
            BigDecimal dividendAmount = rlUserExt.getDividendProportion().multiply(totalDividendAmount)
                    .setScale(2, RoundingMode.HALF_UP);
            dividendDetail.setDividendAmount(dividendAmount);
            dividendDetail.setStatus(ServiceConstants.DIVIDEND_STATUS_SETTLED);
            updateRlDividendDetail(dividendDetail);
        }
        return 1;
    }

    /**
     * 更新退款金额
     *
     * @param id 分成明细id
     * @param userRefundAmount 用户退款金额
     * @return 结果
     */
    @Override
    public int updateRefundAmount(Long id, BigDecimal userRefundAmount) {
        RlDividendDetail rlDividendDetail = new RlDividendDetail();
        rlDividendDetail.setRefundAmount(userRefundAmount);
        rlDividendDetail.setId(id);
        return rlDividendDetailMapper.updateRlDividendDetail(rlDividendDetail);
    }


}
