package com.ruoyi.system.service.impl;

import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.mapper.RlFeeRuleMapper;
import com.ruoyi.system.service.IRlFeeRuleService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 收费模板Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class RlFeeRuleServiceImpl implements IRlFeeRuleService
{
    @Resource
    private RlFeeRuleMapper rlFeeRuleMapper;

    /**
     * 查询收费模板
     *
     * @param ruleId 收费模板主键
     * @return 收费模板
     */
    @Override
    public RlFeeRule selectRlFeeRuleByRuleId(Long ruleId)
    {
        return rlFeeRuleMapper.selectRlFeeRuleByRuleId(ruleId);
    }

    /**
     * 查询收费模板列表
     *
     * @param rlFeeRule 收费模板
     * @return 收费模板
     */
    @Override
    public List<RlFeeRule> selectRlFeeRuleList(RlFeeRule rlFeeRule)
    {
        return rlFeeRuleMapper.selectRlFeeRuleList(rlFeeRule);
    }

    /**
     * 查询收费模板列表
     *
     * @param modelId 车型id
     * @return 收费模板
     */
    @Override
    public List<RlFeeRule> selectRlFeeRuleListByModelId(Long modelId)
    {
        return rlFeeRuleMapper.selectRlFeeRuleListByModelId(modelId);
    }

    /**
     * 查询收费模板列表ids
     *
     * @param modelId 车型id
     * @return 收费模板集合
     */
    @Override
    public List<Long> selectRlFeeRuleLongListByModelId(Long modelId) {
        return rlFeeRuleMapper.selectRlFeeRuleLongListByModelId(modelId);
    }


    /**
     * 新增收费模板
     *
     * @param rlFeeRule 收费模板
     * @return 结果
     */
    @Override
    public int insertRlFeeRule(RlFeeRule rlFeeRule)
    {
        return rlFeeRuleMapper.insertRlFeeRule(rlFeeRule);
    }

    /**
     * 修改收费模板
     *
     * @param rlFeeRule 收费模板
     * @return 结果
     */
    @Override
    public int updateRlFeeRule(RlFeeRule rlFeeRule)
    {
        return rlFeeRuleMapper.updateRlFeeRule(rlFeeRule);
    }

    /**
     * 批量删除收费模板
     *
     * @param ruleIds 需要删除的收费模板主键
     * @return 结果
     */
    @Override
    public int deleteRlFeeRuleByRuleIds(Long[] ruleIds)
    {
        return rlFeeRuleMapper.deleteRlFeeRuleByRuleIds(ruleIds);
    }

    /**
     * 删除收费模板信息
     *
     * @param ruleId 收费模板主键
     * @return 结果
     */
    @Override
    public int deleteRlFeeRuleByRuleId(Long ruleId)
    {
        return rlFeeRuleMapper.deleteRlFeeRuleByRuleId(ruleId);
    }
}
