package com.ruoyi.system.service.impl;

import com.ruoyi.system.domain.modelRule.RlModelRule;
import com.ruoyi.system.mapper.RlModelRuleMapper;
import com.ruoyi.system.service.IRlModelRuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 车型和收费方式关联Service业务层处理
 *
 * @author qzz
 * @date 2024-10-18
 */
@Service
public class RlModelRuleServiceImpl implements IRlModelRuleService
{
    @Autowired
    private RlModelRuleMapper rlModelRuleMapper;

    /**
     * 查询车型和收费方式关联
     *
     * @param modelId 车型和收费方式关联主键
     * @return 车型和收费方式关联
     */
    @Override
    public RlModelRule selectRlModelRuleByModelId(Long modelId)
    {
        return rlModelRuleMapper.selectRlModelRuleByModelId(modelId);
    }

    /**
     * 查询车型和收费方式关联列表
     *
     * @param rlModelRule 车型和收费方式关联
     * @return 车型和收费方式关联
     */
    @Override
    public List<RlModelRule> selectRlModelRuleList(RlModelRule rlModelRule)
    {
        return rlModelRuleMapper.selectRlModelRuleList(rlModelRule);
    }

    /**
     * 新增车型和收费方式关联
     *
     * @param rlModelRule 车型和收费方式关联
     * @return 结果
     */
    @Override
    public int insertRlModelRule(RlModelRule rlModelRule)
    {
        return rlModelRuleMapper.insertRlModelRule(rlModelRule);
    }

    /**
     * 修改车型和收费方式关联
     *
     * @param rlModelRule 车型和收费方式关联
     * @return 结果
     */
    @Override
    public int updateRlModelRule(RlModelRule rlModelRule)
    {
        return rlModelRuleMapper.updateRlModelRule(rlModelRule);
    }

    /**
     * 批量删除车型和收费方式关联
     *
     * @param modelIds 需要删除的车型和收费方式关联主键
     * @return 结果
     */
    @Override
    public int deleteRlModelRuleByModelIds(Long[] modelIds)
    {
        return rlModelRuleMapper.deleteRlModelRuleByModelIds(modelIds);
    }

    /**
     * 删除车型和收费方式关联信息
     *
     * @param modelId 车型和收费方式关联主键
     * @return 结果
     */
    @Override
    public int deleteRlModelRuleByModelId(Long modelId)
    {
        return rlModelRuleMapper.deleteRlModelRuleByModelId(modelId);
    }
}
