package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.RlFunction;
import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.model.RlModel;
import com.ruoyi.system.domain.model.RlModelQuery;
import com.ruoyi.system.domain.model.RlModelVO;
import com.ruoyi.system.domain.modelAccessory.RlModelAccessory;
import com.ruoyi.system.domain.modelRule.RlModelRule;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.mapper.RlModelMapper;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 车辆型号Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Slf4j
@Service
public class RlModelServiceImpl implements IRlModelService
{
    @Resource
    private RlModelMapper rlModelMapper;

    @Autowired
    private IRlDeviceService eDeviceService;

    @Autowired
    private IRlAccessoryService accessoryService;

    @Autowired
    private IRlFunctionService rlFunctionService;

    @Autowired
    private RlStoreService storeService;

    @Autowired
    private IRlModelAccessoryService modelAccessoryService;

    @Autowired
    private IRlFeeRuleService rlFeeRuleService;

    @Autowired
    private IRlModelRuleService rlModelRuleService;

    @Autowired
    private TransactionTemplate transactionTemplate;



    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    @Override
    public RlModelVO selectEModelByModelId(Long modelId)
    {
        RlModelVO etModel = rlModelMapper.selectEModelByModelId(modelId);
        if(ObjectUtil.isNotNull(etModel)){
            // 配件列表
            List<RlAccessoryVO> accessories = accessoryService.selectRlAccessoryListByModelId(modelId);
            if(ObjectUtil.isNotNull(accessories)){
                etModel.setAccessorys(accessories);
                // 将List<RlAccessoryVO> accessories 转为List<Long> list
                List<Long> accessoryIds = accessories.stream()
                        .map(RlAccessoryVO::getAccessoryId) // 假设 RlAccessoryVO 有一个 getAccessoryId() 方法
                        .collect(Collectors.toList());
                etModel.setAccessorysIds(accessoryIds);
            }
            // 功能列表
            List<RlFunction> functionList = getFunctionListByModelId(modelId);
            etModel.setFunctionList(functionList);

            // 收费模版
            List<Long> rlFeeRules = rlFeeRuleService.selectRlFeeRuleLongListByModelId(modelId);
            etModel.setFeeRulesIds(rlFeeRules);
        }
        return etModel;
    }

    /**
     * 查询车辆型号列表
     *
     * @param rlModel 车辆型号
     * @return 车辆型号
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<RlModel> selectEModelList(RlModel rlModel)
    {
        List<RlModel> etModels = rlModelMapper.selectEModelList(rlModel);
        for(RlModel model: etModels){
            // 投放数量统计
            RlDevice device = new RlDevice();
            device.setModelId(model.getModelId());
            Integer integer = eDeviceService.selectCountByModelId(model.getModelId());
//            model.setDeviceNum(integer);
        }
        return etModels;
    }

    /**
     * 查询车辆型号列表
     *
     * @param agentId 代理商id
     * @return 车辆型号
     */
    @Override
    public List<RlModel> selectEModelListByAgentId(Long agentId)
    {
        List<RlModel> etModels = rlModelMapper.selectEModelListByAgentId(agentId);
        return etModels;
    }

    /**
     * 根据商户id查询车辆型号列表
     *
     * @param merchantId 商户id
     * @return 车辆型号集合
     */
    @Override
    public List<RlModelVO> getModelListByMerchantId(Long merchantId)
    {
        List<RlModelVO> rlModelVOS = new ArrayList<>();
        // 根据商户获取到店铺列表
        List<StoreVo> storeVos = storeService.selectSmStoreListBYMerchantId(merchantId);
        for(StoreVo storeVo:storeVos){
            List<RlModelVO> rlModelVOS1 = selectEModelListByStoreId(storeVo.getStoreId());
            rlModelVOS.addAll(rlModelVOS1);
        }
        return rlModelVOS;
    }


    /**
     * 根据店铺id查询车辆型号列表
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    @Override
    public List<RlModelVO> selectEModelListByStoreId(Long storeId)
    {
        List<RlModelVO> models = rlModelMapper.selectEModelListByStoreId(storeId);
        for (RlModelVO model:models) {
            List<RlFunction> functionList = getFunctionListByModelId(model.getModelId());
            model.setFunctionList(functionList);

            List<RlAccessoryVO> accessories = accessoryService.selectRlAccessoryListByModelId(model.getModelId());
            if(ObjectUtil.isNotNull(accessories)){
                model.setAccessorys(accessories);
            }
        }
        return models;
    }

    /**
     * 根据店铺id查询车辆型号列表ids
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    @Override
    public List<Long> selectEModelLongListByStoreId(Long storeId) {
        return rlModelMapper.selectEModelLongListByStoreId(storeId);
    }


    /**
     * 新增车辆型号
     *
     * @param modelQuery 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int insertEModel(RlModelQuery modelQuery)
    {
        Boolean execute = transactionTemplate.execute(e -> {
            modelQuery.setCreateTime(DateUtils.getNowDate());
            int i = rlModelMapper.insertEModel(modelQuery);
            if(i == 0){
                throw new ServiceException("新增失败");
            }
            /** 关联配件*/
            associatedAccessory(modelQuery);
            /** 关联收费方式*/
            associatedFeeRule(modelQuery);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("修改车辆型号失败");
        return 1;
    }

    /**
     * 修改车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int updateEModel(RlModelQuery etModel)
    {
        etModel.setUpdateTime(DateUtils.getNowDate());
        Boolean execute = transactionTemplate.execute(e -> {
            int i = rlModelMapper.updateEModel(etModel);
            if(i == 0){
                throw new ServiceException("修改车辆型号失败");
            }
            /** 关联配件*/
            associatedAccessory(etModel);
            /** 关联收费方式*/
            associatedFeeRule(etModel);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("修改车辆型号失败");
        return 1;
    }

    @Nullable
    private Integer associatedAccessory(RlModelQuery etModel) {
        Long[] accessoryIds = etModel.getAccessoryIds();
        if(ObjectUtil.isNotNull(accessoryIds) && accessoryIds.length>0){
            int i1 = modelAccessoryService.deleteRlModelAccessoryByModelId(etModel.getModelId());
            log.info("【修改车辆型号,删除车型配件关联成功】");
            for (Long accessoryId:accessoryIds) {
                RlModelAccessory modelAccessory = new RlModelAccessory();
                modelAccessory.setAccessoryId(accessoryId);
                modelAccessory.setModelId(etModel.getModelId());
                int i = modelAccessoryService.insertRlModelAccessory(modelAccessory);
                if (i > 0){
                    log.info("【修改车辆型号,添加配件关联成功】");
                }
            }
            return 1;
        }
        return null;
    }

    @Nullable
    private Integer associatedFeeRule(RlModelQuery etModel) {
        Long[] feeRulesIds = etModel.getFeeRulesIds();
        if(ObjectUtil.isNotNull(feeRulesIds) && feeRulesIds.length>0){
            int i1 = rlModelRuleService.deleteRlModelRuleByModelId(etModel.getModelId());
            log.info("【修改车辆型号,删除收费方式成功】");
            for (Long feeRuleId:feeRulesIds) {
                RlModelRule modelRule = new RlModelRule();
                modelRule.setRuleId(feeRuleId);
                modelRule.setModelId(etModel.getModelId());
                int i = rlModelRuleService.insertRlModelRule(modelRule);
                if (i > 0){
                    log.info("【修改车辆型号,添加收费方式成功】");
                }
            }
            return 1;
        }
        return null;
    }

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEModelByModelIds(Long[] modelIds)
    {
        return rlModelMapper.deleteEModelByModelIds(modelIds);
    }

    /**
     * 删除车辆型号信息
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEModelByModelId(Long modelId)
    {
        return rlModelMapper.deleteEModelByModelId(modelId);
    }

    @Override
    public int selectAllCount() {
        return rlModelMapper.selectAllCount();
    }

    /**
     * 根据车型id获取功能
     */
    @Override
    public List<RlFunction> getFunctionListByModelId(Long modelId) {
        return rlFunctionService.selectRlFunctionByModelId(modelId);
    }

}
