package com.ruoyi.system.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.RlOnlineLog;
import com.ruoyi.system.mapper.RlOnlineLogMapper;
import com.ruoyi.system.service.IRlOnlineLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 在线/离线日志Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-07-13
 */
@Service
public class RlOnlineLogServiceImpl implements IRlOnlineLogService
{
    @Autowired
    private RlOnlineLogMapper rlOnlineLogMapper;

    /**
     * 查询在线/离线日志
     *
     * @param id 在线/离线日志主键
     * @return 在线/离线日志
     */
    @Override
    public RlOnlineLog selectEtOnlineLogById(Long id)
    {
        return rlOnlineLogMapper.selectEtOnlineLogById(id);
    }

    /**
     * 查询在线/离线日志列表
     *
     * @param rlOnlineLog 在线/离线日志
     * @return 在线/离线日志
     */
    @Override
    public List<RlOnlineLog> selectEtOnlineLogList(RlOnlineLog rlOnlineLog)
    {
        return rlOnlineLogMapper.selectEtOnlineLogList(rlOnlineLog);
    }

    /**
     * 新增在线/离线日志
     *
     * @param rlOnlineLog 在线/离线日志
     * @return 结果
     */
    @Override
    public int insertEtOnlineLog(RlOnlineLog rlOnlineLog)
    {
        rlOnlineLog.setCreateTime(DateUtils.getNowDate());
        return rlOnlineLogMapper.insertEtOnlineLog(rlOnlineLog);
    }

    /**
     * 修改在线/离线日志
     *
     * @param rlOnlineLog 在线/离线日志
     * @return 结果
     */
    @Override
    public int updateEtOnlineLog(RlOnlineLog rlOnlineLog)
    {
        return rlOnlineLogMapper.updateEtOnlineLog(rlOnlineLog);
    }

    /**
     * 批量删除在线/离线日志
     *
     * @param ids 需要删除的在线/离线日志主键
     * @return 结果
     */
    @Override
    public int deleteEtOnlineLogByIds(Long[] ids)
    {
        return rlOnlineLogMapper.deleteEtOnlineLogByIds(ids);
    }

    /**
     * 删除在线/离线日志信息
     *
     * @param id 在线/离线日志主键
     * @return 结果
     */
    @Override
    public int deleteEtOnlineLogById(Long id)
    {
        return rlOnlineLogMapper.deleteEtOnlineLogById(id);
    }
}
