package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.area.RlOperatingArea;
import com.ruoyi.system.domain.area.RlOperatingAreaVO;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.RlOperatingAreaMapper;
import com.ruoyi.system.service.IRlOperatingAreaService;

import javax.annotation.Resource;

/**
 * 运营区Service业务层处理
 *
 * @author qzz
 * @date 2024-09-18
 */
@Service
public class RlOperatingAreaServiceImpl implements IRlOperatingAreaService
{
    @Resource
    private RlOperatingAreaMapper rlOperatingAreaMapper;

    /**
     * 查询运营区
     *
     * @param areaId 运营区主键
     * @return 运营区
     */
    @Override
    public RlOperatingArea selectRlOperatingAreaByAreaId(Long areaId)
    {
        return rlOperatingAreaMapper.selectRlOperatingAreaByAreaId(areaId);
    }

    /**
     * 查询运营区
     *
     * @param agentId 运营区主键
     * @return 运营区
     */
    @Override
    public RlOperatingAreaVO selectRlOperatingAreaByAgentId(Long agentId)
    {
        return rlOperatingAreaMapper.selectRlOperatingAreaByAgentId(agentId);
    }

    /**
     * 查询运营区列表
     *
     * @param rlOperatingArea 运营区
     * @return 运营区
     */
    @Override
    public List<RlOperatingArea> selectRlOperatingAreaList(RlOperatingArea rlOperatingArea)
    {
        return rlOperatingAreaMapper.selectRlOperatingAreaList(rlOperatingArea);
    }

    /**
     * 新增运营区
     *
     * @param rlOperatingArea 运营区
     * @return 结果
     */
    @Override
    public int insertRlOperatingArea(RlOperatingArea rlOperatingArea)
    {
        rlOperatingArea.setCreateTime(DateUtils.getNowDate());
        return rlOperatingAreaMapper.insertRlOperatingArea(rlOperatingArea);
    }

    /**
     * 修改运营区
     *
     * @param rlOperatingArea 运营区
     * @return 结果
     */
    @Override
    public int updateRlOperatingArea(RlOperatingArea rlOperatingArea)
    {
        return rlOperatingAreaMapper.updateRlOperatingArea(rlOperatingArea);
    }

    /**
     * 批量删除运营区
     *
     * @param areaIds 需要删除的运营区主键
     * @return 结果
     */
    @Override
    public int deleteRlOperatingAreaByAreaIds(Long[] areaIds)
    {
        return rlOperatingAreaMapper.deleteRlOperatingAreaByAreaIds(areaIds);
    }

    /**
     * 删除运营区信息
     *
     * @param areaId 运营区主键
     * @return 结果
     */
    @Override
    public int deleteRlOperatingAreaByAreaId(Long areaId)
    {
        return rlOperatingAreaMapper.deleteRlOperatingAreaByAreaId(areaId);
    }
}
