package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.RlFunction;
import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.domain.agent.RlAgent;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrder;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetail;
import com.ruoyi.system.domain.dividendDetail.RlDividendDetailVO;
import com.ruoyi.system.domain.model.RlModelVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.orderOper.RlOrderOper;
import com.ruoyi.system.domain.refund.RlRefund;
import com.ruoyi.system.domain.refund.RlRefundVO;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.system.domain.vo.PriceVO;
import com.ruoyi.system.mapper.RlOrderMapper;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 订单Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class RlOrderServiceImpl implements IRlOrderService
{

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Resource
    private RlOrderMapper orderMapper;

    @Resource
    private IWxPayService wxPayService;

    @Autowired
    private IRlFeeRuleService feeRuleService;

    @Resource
    private IRlCityService cityService;

    @Autowired
    private IRlAccessoryService accessoryService;

    @Autowired
    private IRlDeviceService deviceService;

    @Resource
    private IRlAgentService agentService;

    @Autowired
    private IRlModelService modelService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private RlStoreService storeService;

    @Resource
    private IRlUserService userService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRlUserExtService userExtService;

    @Autowired
    private IRlRefundService refundService;

    @Autowired
    private IRlOrderOperService rlOrderOperService;

    @Resource
    private IRlDividendDetailService rlDividendDetailService;

    @Autowired
    private IRlOrderOperService orderOperService;

    @Autowired
    private IRlChangeBalanceService changeBalanceService;

    @Resource
    private IRlDividendDetailService dividendDetailService;

    @Autowired
    private IRlDeliveryOrderService deliveryOrderService;

    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOrderId(Long orderId)
    {
        return orderMapper.selectRlOrderByOrderId(orderId);
    }

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOrderNo(String orderNo)
    {
        return orderMapper.selectRlOrderByOrderNo(orderNo);
    }

    /**
     * 第三方订单号
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOutTradeNo(String outTradeNo)
    {
        return orderMapper.selectRlOrderByOutTradeNo(outTradeNo);
    }


    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO getOrderInfoByOrderNo(String orderNo)
    {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order.getModelId() == null){
           throw new ServiceException("订单中没有车型id");
        }
        RlModelVO rlModelVO = modelService.selectEModelByModelId(order.getModelId());
        if(ObjectUtil.isNotNull(rlModelVO)){
            List<RlFunction> functionList = modelService.getFunctionListByModelId(order.getModelId());
            List<RlAccessoryVO> rlAccessoryVOS = accessoryService.selectRlAccessoryListByModelId(order.getModelId());
            order.setFunctionList(functionList);
            order.setAccessorys(rlAccessoryVOS);
        }

        RlDevice rlDevice = deviceService.selectDeviceBySn(order.getSn());
        if(ObjectUtil.isNotNull(rlDevice)) {
            order.setPicture(rlDevice.getPicture());
        }
        /** 超时规则*/
        Long ruleId = order.getRuleId();
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(ruleId);
        order.setRule(feeRule);

        /** 操作记录*/
        RlOrderOper orderOper = new RlOrderOper();
        orderOper.setOrderNo(orderNo);
        List<RlOrderOper> rlOrderOpers = rlOrderOperService.selectRlOrderOperList(new RlOrderOper());
        order.setOrderOpers(rlOrderOpers);
        /** 收益明细*/
        RlDividendDetail rlDividendDetail = new RlDividendDetail();
        rlDividendDetail.setOrderNo(orderNo);
        List<RlDividendDetailVO> rlDividendDetails = rlDividendDetailService.selectRlDividendDetailList(rlDividendDetail);
        order.setRlDividendDetails(rlDividendDetails);

        RlDeliveryOrder deliveryOrder = deliveryOrderService.selectRlDeliveryOrderByOrderNo(orderNo);
        if(ObjectUtil.isNotNull(deliveryOrder)){
            order.setDeliveryStatus(deliveryOrder.getStatus());
            order.setDeliveryOrder(deliveryOrder);
        }

        /** 显示可退款金额  根据订单号查询可退款金额  refundableAmount*/
        getRefundable(orderNo, order);
        return order;
    }

    private void getRefundable(String orderNo, RlOrderVO order) {
        RlRefundVO refundVO = refundService.getRefundedAmount(orderNo);
        BigDecimal refundableAmount = order.getPayFee().subtract(refundVO.getTotalRefundAmount());
        BigDecimal refundableDispatchFee = order.getDispatchFee().subtract(refundVO.getTotalDispatchFee());
        BigDecimal refundableDeliveryFee = order.getDeliveryFee().subtract(refundVO.getTotalDeliveryFee());
        BigDecimal refundableDeposit = order.getDeposit().subtract(refundVO.getTotalDeposit());
        BigDecimal refundableLeaseFee = order.getLeaseFee().subtract(refundVO.getTotalLeaseFee());
        order.setRefundableTotalAmount(refundableAmount);
        order.setRefundableDispatchFee(refundableDispatchFee);
        order.setRefundableDeliveryFee(refundableDeliveryFee);
        order.setRefundableDeposit(refundableDeposit);
        order.setRefundableLeaseFee(refundableLeaseFee);
    }

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<RlOrderVO> selectRlOrderList(RlOrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 查询所有订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<RlOrderVO> selectAllRlOrderList(RlOrderQuery rlOrder)
    {
        return orderMapper.selectAllRlOrderList(rlOrder);
    }

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int insertRlOrder(RlOrderQuery rlOrder)
    {
        rlOrder.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertRlOrder(rlOrder);
    }

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrder(RlOrder rlOrder)
    {
        return orderMapper.updateRlOrder(rlOrder);
    }

    /**
     * 根据订单号修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrderByOrderNo(RlOrder rlOrder)
    {
        return orderMapper.updateRlOrderByOrderNo(rlOrder);
    }


    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderIds(Long[] orderIds)
    {
        return orderMapper.deleteRlOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderId(Long orderId)
    {
        return orderMapper.deleteRlOrderByOrderId(orderId);
    }

    /**
     * 下单
     * 1. 创建订单
     * 2. 预支付
     * 3.
     *
     * @param order 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO createOrder(RlOrderQuery order,Long userId) {
        String orderNo = IdUtils.getOrderNo("zl");
        PrepayResponseVO responseVO = getPrepayResponseVO(order,orderNo,ServiceConstants.ORDER_TYPE_LEASE,userId);
        return responseVO;
    }

    /**
     *  确认支付
     * @param orderNo 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO confirmPay(String orderNo) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){
            throw new ServiceException("订单已支付");
        }
        if(!order.getStatus().equals(ServiceConstants.ORDER_STATUS_TO_BE_PAID)){
            throw new ServiceException("订单非待支付状态");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        responseVO.setOrderNo(orderNo);
        return responseVO;
    }

    @NotNull
    private PrepayResponseVO getPrepayResponseVO(RlOrderQuery order,String orderNo, String type,Long userId) {
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        setFeeRule(order, feeRule);
        /** 解析地址  analyzeAddress */
        analyzeAddress(order);
        order.setCreateTime(DateUtils.getNowDate());
        order.setOrderNo(orderNo);
        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        order.setType(type);
        order.setUserId(userId);
        RlUser rlUser = userService.selectUserById(userId);
        if(ObjectUtil.isNotNull(rlUser)){
            order.setUserName(rlUser.getUserName());
            order.setPhone(rlUser.getPhonenumber());
            order.setRealName(rlUser.getRealName());
        }
        String configByKey = configService.selectConfigByKey("rl.order.auto.cancel");
        order.setAutoCancelTime(DateUtils.getTimeAfterXMinutes(order.getCreateTime(),Integer.parseInt(configByKey)));

        StoreVo storeVo = storeService.selectSmStoreById(order.getStoreId());
        if(ObjectUtil.isNotNull(storeVo)){
            order.setMerchantId(storeVo.getMerchantId());
            order.setReturnAddress(storeVo.getAddress());
            order.setReturnStoreId(storeVo.getStoreId());
        }

        /** 计算价格 */
        onceMoreCalculatePrice(order);
        order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_PAID);
        RlModelVO rlModelVO = modelService.selectEModelByModelId(order.getModelId());
        if(ObjectUtil.isNotNull(rlModelVO)){
            order.setModel(rlModelVO.getModel());
        }
        /** 记录分成比例，状态为未出账 */
        int i1 = dividendDetailService.calculationDividend(order,ServiceConstants.USER_TYPE_MERCHANT);
        if (i1 < 1) {
            throw new RuntimeException("记录商户分成失败");
        }
        int i2 = dividendDetailService.calculationDividend(order,ServiceConstants.USER_TYPE_AGENT);
        if (i2 < 1) {
            throw new RuntimeException("记录代理商分成失败");
        }
        int i = orderMapper.insertRlOrder(order);
        if (i < 1) {
           throw new RuntimeException("下单失败");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        responseVO.setOrderNo(orderNo);
        return responseVO;
    }

    private void onceMoreCalculatePrice(RlOrderQuery order) {
        PriceVO priceVO = calculatePrice(order);
        order.setDeposit(priceVO.getDeposit());
        order.setLeaseFee(priceVO.getRent());
        order.setDeliveryFee(priceVO.getDeliveryFee());
        order.setTotalFee(priceVO.getTotalFee());
        order.setPayFee(priceVO.getTotalFee());
    }

    /** 解析地址 */
    private void analyzeAddress(RlOrderQuery order) {
        String address = cityService.getAddressByLocation(order.getPickupLon(),order.getPickupLat());
        order.setPickupLoc(address);
    }

    private void setFeeRule(RlOrderQuery order, RlFeeRule feeRule) {
        order.setNum(order.getNum());
        order.setPrice(feeRule.getPrice());
        order.setExplain(feeRule.getExplain());
        order.setInstructions(feeRule.getInstructions());
        order.setOutUnit(feeRule.getOutUnit());
        order.setOutPrice(feeRule.getOutPrice());
        order.setRentalUnit(feeRule.getRentalUnit());
        /** 如果是租赁订单，根据取车时间计算到期时间
         * 如果是续费订单，根据到期时间计算续费订单的到期时间 */
        Date startTime = null;
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_LEASE)){
            startTime = order.getPickupTime();
        }else{
            startTime = order.getExpiryTime();
        }
        Date expiryTime = DateUtils.getTimeAfterXUnit(startTime, order.getNum(), feeRule.getRentalUnit());//先根据预估的取车时间，计算出到期时间，后面根据具体的取车时间再算一次
        order.setExpiryTime(expiryTime);
    }

    /**
     * 扫码绑定车辆
     */
    @Override
    public int bandDevice(String orderNo, String sn) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(StrUtil.isNotBlank(order.getSn())){
            throw new RuntimeException("该订单已绑定车辆："+order.getSn());
        }
        RlOrderVO order1 = getInProgressOrder(sn);
        if(ObjectUtil.isNotNull(order1)){
            throw new RuntimeException("该车辆【"+sn+"】有进行中的订单：");
        }
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【扫码绑定车辆】==================");
            RlOrder rlOrder = new RlOrder();
            rlOrder.setOrderNo(orderNo);
            rlOrder.setSn(sn);
            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
            rlOrder.setPickupTime(DateUtils.getNowDate());
            Date expiryTime = DateUtils.getTimeAfterXUnit(DateUtils.getNowDate(), order.getNum(), order.getRentalUnit());
            rlOrder.setExpiryTime(expiryTime);
            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
            if(i > 0){
                /** 更新设备状态 */
                RlDevice rlDevice = new RlDevice();
                rlDevice.setSn(sn);
                rlDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
                deviceService.updateDeviceBySn(rlDevice);
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("扫码绑定车辆失败");
        return 1;
    }

    /**
     * 续租
     * 到期时间
     * 1.  根据原来的订单号，写入新的订单
     * 2.  续租订单标记类型
     * 3.
     */
    @Override
    public PrepayResponseVO relet(RlOrderQuery query) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(query.getOrderNo());
        if(ObjectUtil.isNull(order)){
            throw new RuntimeException("订单["+query.getOrderNo()+"]不存在");
        }
        RlOrderQuery orderQuery = new RlOrderQuery();
        BeanUtils.copyBeanProp(orderQuery,order);
        /** 原订单已经到期  续租订单的到期时间是根据原订单的到期时间算出来的 */
//        Date nowDate = DateUtils.getNowDate();
//        if (order.getExpiryTime().before(nowDate)) {
//            /** 到期后先计算逾期费用，保存在订单中，在还车时一起结算 */
//            RlOrder updateOrder = new RlOrder();
//            updateOrder.setOrderId(order.getOrderId());
//            BigDecimal overdueFee= BigDecimal.ZERO;
//            if(ObjectUtil.isNull(order.getExpiryTime())){
//                throw new ServiceException("【续租】订单没有到期时间");
//            }
//            if (order.getExpiryTime().before(DateUtils.getNowDate())) {
//                overdueFee = deviceService.computeOverdueFee(order, updateOrder);
//            }
//            updateOrder.setOverdueFee(overdueFee);
//            int i = updateRlOrder(updateOrder);
//            if(i==0){
//                throw new ServiceException("更新订单状态失败");
//            }
//        }
        orderQuery.setOriginalOrderNo(query.getOrderNo());
        orderQuery.setRuleId(query.getRuleId());
        orderQuery.setPickupTime(order.getExpiryTime());
        orderQuery.setNum(query.getNum());
        String newOrderNo = IdUtils.getOrderNo("xz");
        PrepayResponseVO responseVO = getPrepayResponseVO(orderQuery,newOrderNo,ServiceConstants.ORDER_TYPE_RELET,SecurityUtils.getUserId());
        return responseVO;
    }

    /**
     * 订单取消
     *
     */
    @Override
    public Boolean orderCancel(String orderNo) {
        if(StrUtil.isBlank(orderNo)){
            throw new RuntimeException("订单号不能为空");
        }
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(!(ServiceConstants.ORDER_STATUS_TO_BE_TAKEN.equals(order.getStatus()) ||
                ServiceConstants.ORDER_STATUS_TO_BE_SENT.equals(order.getStatus()) || ServiceConstants.ORDER_STATUS_TO_BE_PAID.equals(order.getStatus()))){
            throw new RuntimeException("订单已配送或已使用，不能取消");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
            throw new RuntimeException("订单已支付，不能取消");
        }
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setStatus(ServiceConstants.ORDER_STATUS_CANCE);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        if(i>0){
            /** 记录订单履历 */
            if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_USER_CANCEL,
                    order.getStatus(),rlOrder.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"取消订单")){
                throw new ServiceException("【订单履历-支付】更新订单信息失败");
            }
            return true;
        }
        return false;
    }

    /**
     * 车损扣款
     */
    @Override
    public Boolean deductMoney(BigDecimal money, String orderNo) {
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setDeductionAmount(money);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        /** 记录订单履历 */
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_DEDUCTION,
                order.getStatus(),order.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"车损扣款：扣款金额："+money+"，实际押金退款："+order.getDeposit()+"-"+money)){
            throw new ServiceException("【改价】更新订单信息失败");
        }
        return i>0;
    }

    /**
     * 押金退款
     */
    @Override
    public Boolean depositRefund(String orderNo) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(order)){
            throw new ServiceException("订单不存在");
        }
        BigDecimal deposit = order.getDeposit();
        BigDecimal overdueFee = order.getOverdueFee();
        BigDecimal deductionAmount = order.getDeductionAmount();
        BigDecimal remainingDeposit = deposit.subtract(overdueFee).subtract(deductionAmount);
        if(remainingDeposit.compareTo(BigDecimal.ZERO) <= 0){
            return true;
        }
        if(!ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
          throw new ServiceException("订单未支付，不能退款", HttpStatus.ERROR);
        }
        if(!ServiceConstants.ORDER_STATUS_ORDER_END.equals(order.getStatus())){
            throw new ServiceException("订单状态异常，不能退款", HttpStatus.ERROR);
        }
//        /** 更新订单信息*/
//        RlOrder updateOrder = new RlOrder();
//        updateOrder.setOrderNo(orderNo);
//        updateOrder.setRefundAmount(remainingDeposit);
//        updateOrder.setStatus(ServiceConstants.ORDER_STATUS_REFUND);
//        int i = orderMapper.updateRlOrderByOrderNo(updateOrder);
        /** 记录订单履历*/
        if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_RETURN_END,
                order.getStatus(),order.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"已退款：-"+remainingDeposit+"元")){
            throw new ServiceException("【记录订单履历失败】");
        }
        // 执行退还剩余押金操作
        wxPayService.refund(order, "审核后退押金",remainingDeposit,order.getOutTradeNo());
        return null;
    }

    /**
     * todo 未完成
     * 退款
     * 1. 校验
     * 2. 用户余额不足判断
     * 3. 计算出退款的比例
     * 4. 遍历用户，新增账变
     * 5. 新增退款明细
     * 6. 发起退款
     */
    @Transactional
    @Override
    public int refund(RlOrderQuery etOrder) {
        //根据订单号查询订单信息
        /** 1. 校验 */
        RlOrderVO rlOrderVO = orderMapper.selectRlOrderByOrderNo(etOrder.getOrderNo());
        refundVerify(rlOrderVO);
        /** 2.计算出退款的比例 保留两位小数点  比如退款金额为5元，  订单金额为32元  退款比例 = 5/32 = 0.16 （四舍五入）由系统来平账*/
        //退款金额  调度费(商家取车)  配送费（送车上门）  租赁费
        BigDecimal refundAmount = new BigDecimal("0");
        BigDecimal dispatchFee = etOrder.getDispatchFee();
        if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(dispatchFee);
        }
        BigDecimal deliveryFee = etOrder.getDeliveryFee();
        if(ObjectUtil.isNotNull(deliveryFee) && !deliveryFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(deliveryFee);
        }
        BigDecimal leaseFee = etOrder.getLeaseFee();
        if(ObjectUtil.isNotNull(leaseFee) && !leaseFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(leaseFee);
        }
        if(refundAmount.compareTo(BigDecimal.ZERO) == 0){
            throw new ServiceException("总退款金额不能为0");
        }
        BigDecimal refundPercentage = refundAmount.divide(rlOrderVO.getPayFee()).setScale(2, RoundingMode.HALF_UP);
        BigDecimal finalRefundAmount = refundAmount;
        Boolean execute = transactionTemplate.execute(e -> {
            /** 3. 根据分成列表，遍历用户，新增账变 */
            refundDividendHandle(rlOrderVO, refundPercentage);
            /** 4.记录退款表  创建退款对象*/
            String outRefundNo = saveRefundObj(rlOrderVO, finalRefundAmount, dispatchFee, deliveryFee, leaseFee);
            if(!orderOperService.recordOrderHistory(rlOrderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_REFUND,
                    rlOrderVO.getStatus(),rlOrderVO.getStatus(),rlOrderVO.getPayFee(),rlOrderVO.getPayFee(),rlOrderVO.getUserId(),rlOrderVO.getPhone(),"已退款: "+finalRefundAmount+"元")){
                throw new ServiceException("【改价】更新订单信息失败");
            }
            /** 5. 发起退款 */
            wxPayService.refund(rlOrderVO, "商家退款金额", finalRefundAmount,outRefundNo);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【订单退款】失败");
        return 1;
    }

    @NotNull
    private String saveRefundObj(RlOrderVO rlOrderVO, BigDecimal refundAmount, BigDecimal dispatchFee, BigDecimal deliveryFee, BigDecimal leaseFee) {
        String outRefundNo = IdUtils.getOrderNo("ref");
        RlRefund refund1= createRefund("商家退款金额", rlOrderVO.getUserId(), rlOrderVO.getOrderNo(), refundAmount, dispatchFee, deliveryFee, leaseFee, outRefundNo,ServiceConstants.REFUND_TYPE_SYSTEM);
        int refundResult = refundService.insertEtRefund(refund1);
        if(refundResult>0){
            logger.info("保存退款对象成功");
        }
        logger.info("总金额：【{}】，退款金额：【{}】", rlOrderVO.getTotalFee(), refundAmount);
        return outRefundNo;
    }

    private void refundDividendHandle(RlOrderVO rlOrderVO, BigDecimal refundPercentage) {
        RlDividendDetail rlDividendDetail = new RlDividendDetail();
        rlDividendDetail.setOrderNo(rlOrderVO.getOrderNo());
        List<RlDividendDetailVO> dividendDetails = rlDividendDetailService.selectRlDividendDetailList(rlDividendDetail);
        for(RlDividendDetail detail:dividendDetails){
            /** 4.1 余额不足判断 */
            RlUserExt rlUserExt = userExtService.selectRlUserExtByUserId(detail.getPartnerId());
            RlUser user = userService.selectUserById(rlUserExt.getUserId());
            // 计算出这个用户将要退款的金额
            BigDecimal userRefundAmount = detail.getDividendAmount().multiply(refundPercentage);//用户退款金额  = 0.16 * 5元 = 0.8元  如果这个值小于余额则提示 余额不足
            BigDecimal subtract = rlUserExt.getBalance().subtract(userRefundAmount);
            if(subtract.compareTo(BigDecimal.ZERO) <= 0){
                throw new ServiceException("用户【"+user.getPhonenumber()+"】余额不足，不能退款");
            }
            /** 4.2 新增账变 */
            int i = changeBalanceService.generateChanggeBalance(rlOrderVO.getOrderNo(), rlOrderVO.getOutTradeNo(), ServiceConstants.FLOW_TYPE_DISBURSE,
                    ServiceConstants.ORDER_REFUND, userRefundAmount, rlUserExt.getUserId(), user.getUserName(), user.getPhonenumber());
            if(i<=0)throw new ServiceException("用户【"+user.getPhonenumber()+"】新增账变失败");
            /** todo 4.3 更新分成表中的退款金额 */
            int i1 = rlDividendDetailService.updateRefundAmount(detail.getId(), userRefundAmount);
            if(i1<=0)throw new ServiceException("【退款】更新分成表中的退款金额失败");
        }
    }

    /** 退款校验*/
    private void refundVerify(RlOrderVO rlOrderVO) {
        if(ObjectUtil.isNull(rlOrderVO)){
            throw new ServiceException("订单不存在");
        }
        if(!ServiceConstants.ORDER_STATUS_ORDER_END.equals(rlOrderVO.getStatus())){
            throw new ServiceException("退款失败，订单未结束");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(rlOrderVO.getPaid())){
            throw new ServiceException("订单未支付，不能退款");
        }
    }

    @Override
    public RlRefund createRefund(String reason, Long userId,String orderNo,BigDecimal refundAmount, BigDecimal dispatchFee, BigDecimal deliveryFee, BigDecimal leaseFee, String outRefundNo, String type) {
        RlRefund etRefund = new RlRefund();
        etRefund.setAmount(refundAmount);
        etRefund.setReason(reason);
        etRefund.setDispatchFee(dispatchFee);
        etRefund.setDeliveryFee(deliveryFee);
        etRefund.setLeaseFee(leaseFee);
        etRefund.setOrderNo(orderNo);
        etRefund.setUserId(userId);
        etRefund.setRefundNo(outRefundNo);
        etRefund.setType(type);
        if(type.equals(ServiceConstants.REFUND_TYPE_DEPOSIT)){
            etRefund.setItemDesc("押金退款");
        }else{
            StringBuilder itemDesc = new StringBuilder();
            if(ObjectUtil.isNotNull(deliveryFee) && !deliveryFee.equals(BigDecimal.ZERO)){
                itemDesc.append("配送费:").append(deliveryFee).append("元,");
            }
            if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
                itemDesc.append("调度费:").append(dispatchFee).append("元,");
            }
            if(ObjectUtil.isNotNull(leaseFee) && !leaseFee.equals(BigDecimal.ZERO)){
                itemDesc.append("租赁费:").append(leaseFee).append("元,");
            }
            etRefund.setItemDesc(itemDesc.toString());
        }
        return etRefund;
    }

    /**
     * 计算价格
     */
    @Override
    public PriceVO calculatePrice(RlOrderQuery order) {
        PriceVO priceVO = new PriceVO();
        BigDecimal totalFee = BigDecimal.ZERO;
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_LEASE)){
            // 押金
            BigDecimal deposit = deposit(order.getModelId(), priceVO);
            totalFee = totalFee.add(deposit);
            // 配送费
            BigDecimal deliveryFee = deliveryFee(order, priceVO);
            totalFee = totalFee.add(deliveryFee);
        }
        // 租金
        BigDecimal rentFee = rentFee(order, priceVO);
        totalFee = totalFee.add(rentFee);
        priceVO.setTotalFee(totalFee);
        return priceVO;
    }

    /**
     * 根据订单号查询可还车的店铺
     */
    @Override
    public List<StoreVo> getReturnStoreList(String orderNo,String lon,String lat) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        List<StoreVo> storeVos = storeService.selectSmStoreListBYMerchantId(order.getMerchantId());
        for (StoreVo storeVo:storeVos) {
            BigDecimal storeLon = storeVo.getLng();
            BigDecimal storeLat = storeVo.getLat();
            if (storeLon != null && storeLat != null && StrUtil.isNotBlank(lon) && StrUtil.isNotBlank(lat)) {
                double[] point1 = {storeLon.doubleValue(), storeLat.doubleValue()};
                double[] point2 = {Double.valueOf(lon), Double.valueOf(lat)};
                double distance = GeoUtils.haversineDistance(point1, point2);
                BigDecimal distanceFormatted = new BigDecimal(distance).setScale(1, RoundingMode.HALF_UP);
                storeVo.setDistance(distanceFormatted.doubleValue());
            }
        }
        List<StoreVo> storeList = new ArrayList<>(storeVos);
        Collections.sort(storeList, new Comparator<StoreVo>() {
            @Override
            public int compare(StoreVo o1, StoreVo o2) {
                // 注意：由于我们想要按距离由近到远排序，因此当 o1 的距离小于 o2 的距离时，应返回 -1
                // 当距离相等时返回 0，当 o1 的距离大于 o2 的距离时返回 1
                return Double.compare(o1.getDistance(), o2.getDistance());
            }
        });
        return storeList;
    }

    /**
     * 根据sn查询正在进行中的订单
     */
    @Override
    public RlOrderVO getInProgressOrder(String sn) {
        List<RlOrderVO> orders = orderMapper.getInProgressOrder(sn);
        if(ObjectUtil.isNotNull(orders.size()) && orders.size() > 0){
            return orders.get(0);
        }
        return null;
    }

    @Override
    public void validate(RlOrderQuery orderQuery) {
        if(StrUtil.isBlank(orderQuery.getOrderNo())){
            throw new ServiceException("订单号不能为空");
        }
        if(StrUtil.isBlank(orderQuery.getType())){
            throw new ServiceException("类型不能为空");
        }
        if(orderQuery.getRuleId() == null || orderQuery.getRuleId() <= 0){
            throw new ServiceException("计费规则id不能为空");
        }
        if(orderQuery.getNum() == null || orderQuery.getNum() <= 0){
            throw new ServiceException("数量不能为空或小于0");
        }
    }

    /**
     * 首页统计
     */
    @Override
    public IndexAdminVo indexStatistics(Long userId) {
        IndexAdminVo indexAdminVo = new IndexAdminVo();
        ArrayList<IndexAdminVo.IncomeVo> incomeVos = new ArrayList<>();
        for (int i = 1; i < 7; i++) {
            IndexAdminVo.IncomeVo orderFee = new IndexAdminVo.IncomeVo();
            orderFee.setDay("2024-10-0"+i);
            orderFee.setOrderFee(BigDecimal.valueOf(i*100));
            orderFee.setOrderNum(i*10);
            incomeVos.add(orderFee);
        }
        indexAdminVo.setIncomeVoList(incomeVos);
        indexAdminVo.setTodayOrderNum(new BigDecimal(10));
        indexAdminVo.setTodayIncome(new BigDecimal(100));
        indexAdminVo.setComparedYesterdayIncome(new BigDecimal(5));
        indexAdminVo.setComparedYesterdayOrderNum(new BigDecimal(5));
        return indexAdminVo;
    }

    private BigDecimal deposit(Long modelId, PriceVO priceVO) {
        RlModelVO rlModelVO = modelService.selectEModelByModelId(modelId);
        if(rlModelVO == null){
            throw new ServiceException("车型不存在");
        }
        BigDecimal deposit = rlModelVO.getDeposit();
        if(rlModelVO == null){
            throw new ServiceException("车型不存在");
        }
        priceVO.setDeposit(deposit);
        return deposit;
    }

    private BigDecimal deliveryFee(RlOrderQuery orderQuery, PriceVO priceVO) {
        BigDecimal deliveryFee = BigDecimal.ZERO;
        RlAgent agent = agentService.selectRlAgentByAgentId(orderQuery.getAgentId());
        if(agent == null){
            throw new ServiceException("代理商不存在");
        }
        if(orderQuery.getDeliveryMethod().equals(ServiceConstants.DELIVERY_METHOD_CHARGE_DELIVERY_CAR)){
            deliveryFee = agent.getDeliveryFee();
            priceVO.setDeliveryFee(deliveryFee);
        }
        return deliveryFee;
    }

    private BigDecimal rentFee(RlOrderQuery order, PriceVO priceVO) {
        BigDecimal rent = BigDecimal.ZERO;
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        if(feeRule == null){
            throw new ServiceException("租赁规则不存在");
        }
        Integer num = order.getNum();
        if(num != null){
            rent = feeRule.getPrice().multiply(new BigDecimal(num));
            priceVO.setRent(rent);
        }
        return rent;
    }


}
