package com.ruoyi.system.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.triplog.RlTripLog;
import com.ruoyi.system.mapper.RlTripLogMapper;
import com.ruoyi.system.service.IRlDeviceService;
import com.ruoyi.system.service.IRlTripLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 行程记录Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Slf4j
@Service
public class RlTripLogServiceImpl implements IRlTripLogService
{
    @Resource
    private RlTripLogMapper rlTripLogMapper;

    @Resource
    private IRlDeviceService deviceService;



    /**
     * 查询行程记录
     *
     * @param tripId 行程记录主键
     * @return 行程记录
     */
    @Override
    public RlTripLog selectEtTripLogByTripId(Long tripId)
    {
        return rlTripLogMapper.selectEtTripLogByTripId(tripId);
    }

    /**
     * 查询行程记录列表
     *
     * @param etTripLog 行程记录
     * @return 行程记录
     */
    @Override
    public List<RlTripLog> selectEtTripLogList(RlTripLog etTripLog)
    {
        return rlTripLogMapper.selectEtTripLogList(etTripLog);
    }

    /**
     * 新增行程记录
     *
     * @param etTripLog 行程记录
     * @return 结果
     */
    @Override
    public int insertEtTripLog(RlTripLog etTripLog)
    {
        etTripLog.setCreateTime(DateUtils.getNowDate());
        return rlTripLogMapper.insertEtTripLog(etTripLog);
    }

    /**
     * 修改行程记录
     *
     * @param etTripLog 行程记录
     * @return 结果
     */
    @Override
    public int updateEtTripLog(RlTripLog etTripLog)
    {
        return rlTripLogMapper.updateEtTripLog(etTripLog);
    }

    /**
     * 批量删除行程记录
     *
     * @param tripIds 需要删除的行程记录主键
     * @return 结果
     */
    @Override
    public int deleteEtTripLogByTripIds(Long[] tripIds)
    {
        return rlTripLogMapper.deleteEtTripLogByTripIds(tripIds);
    }

    /**
     * 删除行程记录信息
     *
     * @param tripId 行程记录主键
     * @return 结果
     */
    @Override
    public int deleteEtTripLogByTripId(Long tripId)
    {
        return rlTripLogMapper.deleteEtTripLogByTripId(tripId);
    }


    public RlTripLog createTripLog(String orderNo,String sn,String type) {
        RlTripLog etTripLog = new RlTripLog();
        etTripLog.setType(type);
        etTripLog.setOrderNo(orderNo);
        RlDeviceVO asDevice = null;
        if(StrUtil.isNotBlank(sn)){
            asDevice = deviceService.getLatestLocationBySn(sn);
            etTripLog.setLatitude(asDevice.getLatitude());
            etTripLog.setLongitude(asDevice.getLongitude());
            etTripLog.setSn(sn);
        }
        log.info("创建行程记录:{}", JSON.toJSON(etTripLog));
        return etTripLog;
    }

    /**
     * 创建行程记录并保存
     * @param orderNo
     * @return
     */
    public int tripLog(String orderNo,String sn,String type){
        RlTripLog tripLog = createTripLog(orderNo,sn,type);
        return insertEtTripLog(tripLog);
    }
}
