package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.ChangeBalanceInterface;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.core.domain.entity.RlUserVO;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.common.utils.verify.VerifyIdentityUtil;
import com.ruoyi.system.domain.RlMsgLog;
import com.ruoyi.system.domain.SysUserPost;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.domain.agent.RlAgentVO;
import com.ruoyi.system.domain.changeBalance.RlChangeBalance;
import com.ruoyi.system.domain.changeBalance.RlChangeBalanceQuery;
import com.ruoyi.system.domain.channel.ChannelVO;
import com.ruoyi.system.domain.channelWithdraw.ChannelWithdrawVO;
import com.ruoyi.system.domain.query.AuthenticationQuery;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.domain.vo.RlUserQuery;
import com.ruoyi.system.domain.withdraw.RlWithdraw;
import com.ruoyi.system.domain.withdraw.RlWithdrawQuery;
import com.ruoyi.system.mapper.RlUserMapper;
import com.ruoyi.system.mapper.SysUserPostMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import javax.validation.Validator;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 用户 业务层处理
 *
 * @author ruoyi
 */
@Slf4j
@Service
public class RlUserServiceImpl implements IRlUserService{

    @Resource
    private RlUserMapper rlUserMapper;

    @Resource
    private IRlUserExtService userExtService;

    @Resource
    private IRlAgentService agentService;

    @Autowired
    private RlChannelService channelService;

    @Autowired
    protected Validator validator;

    @Autowired
    private IRlWithdrawService rlWithdrawService;

    @Autowired
    private IRlChangeBalanceService rlChangeBalanceService;

    @Resource
    private SysUserRoleMapper userRoleMapper;

    @Resource
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IRlMsgLogService msgLogService;

    @Autowired
    private RlChannelWithdrawService etChannelWithdrawService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRlChangeBalanceService changeBalanceService;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode3}")
    private String templateCode3;

    @Value("${aliyun.phone}")
    private String phone;

//    @Autowired
//    private IEtOrderService etOrderService;
//
//    @Autowired
//    private IEtOperatingAreaService etOperatingAreaService;
//
//    @Autowired
//    private RedisCache redisCache;
//
//    @Resource
//    private EtOrderMapper etOrderMapper;

//    @Resource
//    private RlUserMapper userMapper;

    @Value("${et.verifyUrl}")
    private String verifyUrl;

    @Value("${et.appcode}")
    private String appcode;

    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public List<RlUserVO> selectUserList(RlUserQuery user)
    {
        /** 获取设备绑定数量*/
        List<RlUserVO> users = rlUserMapper.selectUserList(user);
        return users;
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<RlUser> selectAllocatedList(RlUser user)
    {
        return rlUserMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<RlUser> selectUnallocatedList(RlUser user)
    {
        return rlUserMapper.selectUnallocatedList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public RlUser selectUserByUserName(String userName)
    {
        return rlUserMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public RlUserVO selectUserById(Long userId)
    {
        RlUserVO users = rlUserMapper.selectUserById(userId);
        if(ObjectUtil.isNull(users)){
            throw new ServiceException("没有该用户："+userId);
        }
        return users;
    }

    /**
     * 通过用户ID查询账户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public RlUserVO myAccountInfoByUserId(Long userId)
    {
        RlUserVO users = rlUserMapper.myAccountInfoByUserId(userId);
        /** 累计提现*/
        BigDecimal totalWithdrawAmount = rlWithdrawService.cumulativeWithdrawalAmount(userId);
        if(ObjectUtil.isNull(totalWithdrawAmount)){
            totalWithdrawAmount = BigDecimal.ZERO.setScale(2, RoundingMode.HALF_UP);
        }
        users.setTotalWithdrawAmount(totalWithdrawAmount);
        return users;
    }


//    /**
//     * 校验用户名称是否唯一
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    public boolean checkUserNameUnique(EUser user)
//    {
//        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
//        EUser info = EUserMapper.checkUserNameUnique(user.getUserName());
//        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
//        {
//            return UserConstants.NOT_UNIQUE;
//        }
//        return UserConstants.UNIQUE;
//    }
//
//    /**
//     * 校验手机号码是否唯一
//     *
//     * @param user 用户信息
//     * @return
//     */
//    @Override
//    public boolean checkPhoneUnique(EUser user)
//    {
//        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
//        EUser info = EUserMapper.checkPhoneUnique(user.getPhonenumber());
//        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
//        {
//            return UserConstants.NOT_UNIQUE;
//        }
//        return UserConstants.UNIQUE;
//    }
//
//    /**
//     * 校验email是否唯一
//     *
//     * @param user 用户信息
//     * @return
//     */
//    @Override
//    public boolean checkEmailUnique(EUser user)
//    {
//        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
//        EUser info = EUserMapper.checkEmailUnique(user.getEmail());
//        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
//        {
//            return UserConstants.NOT_UNIQUE;
//        }
//        return UserConstants.UNIQUE;
//    }
//
//
    /**
     * 新增保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(RlUser user)
    {
        return rlUserMapper.insertUser(user);
    }
//
//    /**
//     * 注册用户信息
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    public boolean registerUser(EUser user)
//    {
//        return EUserMapper.insertUser(user) > 0;
//    }
//
    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(RlUser user)
    {
        return rlUserMapper.updateUser(user);
    }


    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(RlUser user)
    {
        return rlUserMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(RlUser user)
    {
        return rlUserMapper.updateUser(user);
    }
//
//    /**
//     * 修改用户头像
//     *
//     * @param userName 用户名
//     * @param avatar 头像地址
//     * @return 结果
//     */
//    @Override
//    public boolean updateUserAvatar(String userName, String avatar)
//    {
//        return EUserMapper.updateUserAvatar(userName, avatar) > 0;
//    }
//
//    /**
//     * 重置用户密码
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    public int resetPwd(EUser user)
//    {
//        return EUserMapper.updateUser(user);
//    }
//
//    /**
//     * 重置用户密码
//     *
//     * @param userName 用户名
//     * @param password 密码
//     * @return 结果
//     */
//    @Override
//    public int resetUserPwd(String userName, String password)
//    {
//        return EUserMapper.resetUserPwd(userName, password);
//    }
//
//
//    /**
//     * 通过用户ID删除用户
//     *
//     * @param userId 用户ID
//     * @return 结果
//     */
//    @Override
//    @Transactional
//    public int deleteUserById(Long userId)
//    {
//        return EUserMapper.deleteUserById(userId);
//    }
//
//    /**
//     * 批量删除用户信息
//     *
//     * @param userIds 需要删除的用户ID
//     * @return 结果
//     */
//    @Override
//    @Transactional
//    public int deleteUserByIds(Long[] userIds)
//    {
//        return EUserMapper.deleteUserByIds(userIds);
//    }
//
//    /**
//     * 导入用户数据
//     *
//     * @param userList 用户数据列表
//     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
//     * @param operName 操作用户
//     * @return 结果
//     */
//    @Override
//    public String importUser(List<EUser> userList, Boolean isUpdateSupport, String operName)
//    {
//        if (StringUtils.isNull(userList) || userList.size() == 0)
//        {
//            throw new ServiceException("导入用户数据不能为空！");
//        }
//        int successNum = 0;
//        int failureNum = 0;
//        StringBuilder successMsg = new StringBuilder();
//        StringBuilder failureMsg = new StringBuilder();
//        String password = configService.selectConfigByKey("sys.user.initPassword");
//        for (EUser user : userList)
//        {
//            try
//            {
//                // 验证是否存在这个用户
//                EUser u = EUserMapper.selectUserByUserName(user.getUserName());
//                if (StringUtils.isNull(u))
//                {
//                    BeanValidators.validateWithException(validator, user);
//                    user.setPassword(SecurityUtils.encryptPassword(password));
//                    user.setCreateBy(operName);
//                    EUserMapper.insertUser(user);
//                    successNum++;
//                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
//                }
//                else if (isUpdateSupport)
//                {
//                    BeanValidators.validateWithException(validator, user);
//                    user.setUserId(u.getUserId());
//                    user.setUpdateBy(operName);
//                    EUserMapper.updateUser(user);
//                    successNum++;
//                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
//                }
//                else
//                {
//                    failureNum++;
//                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
//                }
//            }
//            catch (Exception e)
//            {
//                failureNum++;
//                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
//                failureMsg.append(msg + e.getMessage());
//                log.error(msg, e);
//            }
//        }
//        if (failureNum > 0)
//        {
//            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
//            throw new ServiceException(failureMsg.toString());
//        }
//        else
//        {
//            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
//        }
//        return successMsg.toString();
//    }
//
//    /**
//     * 通过手机号查询用户
//     *
//     * @param phone 手机号
//     * @return 用户对象信息
//     */
//    @Override
//    public EUser selectUserByPhone(String phone) {
//        return EUserMapper.selectUserByPhone(phone);
//    }

    /**
     * 通过用户手机号和appid查询用户
     *
     * @param phone 手机号
     * @return 用户对象信息
     */
    @Override
    public RlUser selectUserByPhoneAndAppId(String phone, String appid) {
        return rlUserMapper.selectUserByPhoneAndAppId(phone,appid);
    }

    /**
     * 通过openid查询用户
     *
     * @param openid 手机号
     * @return 用户对象信息
     */
    @Override
    public RlUser selectUserByWxopenid(String openid) {
        return rlUserMapper.selectUserByWxopenid(openid);
    }

//    /**
//     * 检测用户是否有充值过押金
//     * @param userId
//     */
//    @Override
//    public Boolean checkIsDeposit(Long userId) {
//        EUser eUser = EUserMapper.selectUserById(userId);
//        BigDecimal balance = asUser.getBalance();
////        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(asUser.getAreaId());
////        BigDecimal deposit = new BigDecimal(area.getDeposit());
//        if(ObjectUtil.isNull(balance)){
//            log.info("用户【{}】余额为空",userId);
//            return false;
//        }
//        if(balance.compareTo(BigDecimal.ZERO)==0){//余额为0
//            log.info("用户【{}】余额为0",userId);
//            return false;
//        }
//        // 判断是否有押金充值记录
//        EtOrder etOrder = new EtOrder();
//        etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
//        etOrder.setUserId(userId);
//        etOrder.setPaid("1");
//        if(etOrderMapper.selectEtOrderList(etOrder).size()<1){//没有充值押金记录
//            log.info("用户【{}】没有充值押金记录",userId);
//            return false;
//        }
//        return true;
//    }

//    /**
//     * 新增用户
//     */
//    @Override
//    public String getNewUser(String powerStart, String powerEnd,String areaId) {
//        if(StringUtils.isNotEmpty(powerStart) && StringUtils.isNotEmpty(powerEnd)){
//            return EUserMapper.getNewUser(powerStart,powerEnd,areaId);
//        }
//        return null;
//    }
//
//    /**
//     * 总用户
//     */
//    @Override
//    public String getTotalUser(String areaId) {
//        return EUserMapper.getTotalUser(areaId);
//    }
//
//    /**
//     * 租赁用户
//     */
//    @Override
//    public String getLeaseUser(String powerStart, String powerEnd,String areaId) {
//        if(StringUtils.isNotEmpty(powerStart) && StringUtils.isNotEmpty(powerEnd)){
//            return EUserMapper.getLeaseUser(powerStart,powerEnd,areaId);
//        }
//        return null;
//    }
//
//    /**
//     * 绑定系统用户
//     */
//    @Transactional
//    @Override
//    public int bandSystemUser(EUser user) {
//        // 删除用户缓存
//        Collection<String> keys = SpringUtils.getBean(RedisCache.class).keys(CacheConstants.APP_LOGIN_TOKEN_KEY + "*");
//        redisCache.deleteObject(keys);
//        return EUserMapper.updateUser(user);
//    }
//
    /**
     * 实名认证
     */
    @Override
    public Object authentication(AuthenticationQuery authenticationQuery) {
        String idCard = authenticationQuery.getIdCard();
        String realName = authenticationQuery.getRealName();
        Object o = VerifyIdentityUtil.verifyIdentity(verifyUrl, appcode, idCard, realName);
        return o;
    }
//
//    /**
//     * 检测是否实名认证
//     */
//    @Override
//    public boolean checkIsAuthentication(Long userId) {
//        if(EUserMapper.selectUserById(userId).getIsAuthentication().equals("1")){
//            return true;
//        }
//        return false;
//    }
//
//    /**
//     * 根据手机号快速搜索用户列表
//     */
//    @Override
//    public List<LabelVo> fastSearch(String phonenumber) {
//        /** 获取设备绑定数量*/
//        List<LabelVo> users = EUserMapper.fastSearch(phonenumber);
//        return users;
//    }

    /**
     * 更新用户密码
     */
    @Override
    public void updateUserPwd(Long userId, String newPassword) {
        rlUserMapper.updateUserPwd(userId, newPassword);
    }

    @Override
    public void checkUserDataScope(Long userId) {

    }

    @Override
    public String importUser(List<RlUser> userList, Boolean isUpdateSupport, String operName) {
        return null;
    }

    @Override
    public boolean checkUserNameUnique(RlUser user) {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        RlUser info = rlUserMapper.checkUserNameUnique(user.getUserName());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public boolean registerUser(RlUser user) {
        return false;
    }

    @Override
    public boolean checkPhoneUnique(RlUser user) {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        RlUser info = rlUserMapper.checkPhoneUnique(user.getPhonenumber());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public boolean checkEmailUnique(RlUser user) {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        RlUser info = rlUserMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public String selectUserRoleGroup(String userName) {
        return null;
    }

    @Override
    public String selectUserPostGroup(String userName) {
        return null;
    }

    @Override
    public boolean updateUserAvatar(String userName, String avatar) {
        return false;
    }

    @Override
    public int resetUserPwd(String userName, String password) {
        return 0;
    }

    @Override
    public void checkUserAllowed(RlUser user) {

    }

    @Override
    public int deleteUserByIds(Long[] userIds) {
        return 0;
    }

    @Override
    public int resetPwd(RlUser user) {
        return 0;
    }

    /**
     * 用户授权角色
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional
    public void insertUserAuth(Long userId, Long[] roleIds)
    {
        userRoleMapper.deleteUserRoleByUserId(userId);
        insertUserRole(userId, roleIds);
    }

    /**
     * 获取用户余额
     *
     * @param userId 用户id
     * @return 结果
     */
    @Override
    public BigDecimal selectUserBalanceById(Long userId) {
        return rlUserMapper.selectUserBalanceById(userId);
    }

    /**
     * 账变记录
     */
    @Override
    public List<RlChangeBalance> myChangeBalanceList(RlChangeBalanceQuery query,Long userId) {
        /** 账变明细*/
        List<RlChangeBalance> rlChangeBalances = rlChangeBalanceService.selectRlChangeBalanceListByUserId(query,userId);
        return rlChangeBalances;
    }

    /**
     * 管理员提现
     * 1. 获取当前用户
     * 2. 记录提现记录
     * 3. 扣余额,记录账变
     * 4. 发起提现（如果是自动api提现）
     */
    @Override
    public int adminWithdraw(RlWithdrawQuery withdraw) {
        String orderNo = IdUtils.getOrderNo("tx");
        /** 1. 获取当前用户*/
        RlUser user = rlUserMapper.selectUserById(SecurityUtils.getUserId());
        RlUserExt rlUserExt = userExtService.selectRlUserExtByUserId(user.getUserId());

        Boolean execute = transactionTemplate.execute(e -> {
            /** 2. 记录提现记录*/
            withdrawalRecord(withdraw, orderNo, user, rlUserExt);
            /** 3. 扣余额,记录账变  */
            int i = changeBalanceService.generateChanggeBalance(orderNo, null, ServiceConstants.FLOW_TYPE_DISBURSE,
                    ServiceConstants.WITHDRAWAL, withdraw.getAmount(), user.getUserId(), user.getUserName(), user.getPhonenumber());
            if(i<=0)throw new ServiceException("【提现】记录账变失败");
            // 发短信通知转账人
            asynchronousMsg(user.getUserName());
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【提现】失败");
        return 1;
    }

    /**
     * 更新用户余额
     */
    @Override
    public int changeBalance(BigDecimal changeAmount, Long userId) {
        return rlUserMapper.changeBalance(changeAmount, userId);
    }

    private void withdrawalRecord(RlWithdrawQuery withdraw, String orderNo, RlUser user, RlUserExt rlUserExt) {
        RlWithdraw rlWithdraw = new RlWithdraw();
        rlWithdraw.setWithdrawNo(orderNo);
        rlWithdraw.setCreateTime(DateUtils.getNowDate());
        rlWithdraw.setAmount(withdraw.getAmount());
        rlWithdraw.setStatus(ServiceConstants.FLOW_STATUS_APPLY);
        rlWithdraw.setOwnerId(user.getUserId());
        rlWithdraw.setOwnerName(user.getUserName());
        rlWithdraw.setOwnerPhone(user.getPhonenumber());
        ChannelWithdrawVO channelWithdrawVO = etChannelWithdrawService.selectChannelWithdrawByChannelId(withdraw.getWithdrawChannelId());
        if(ObjectUtil.isNull(channelWithdrawVO)){
            throw new ServiceException("提现渠道不存在");
        }
        BigDecimal cost;
        if(channelWithdrawVO.getHandlingChargeType().equals(ServiceConstants.HANDLING_CHARGE_TYPE_PERCENT)){
            cost = channelWithdrawVO.getCostRate().divide(new BigDecimal(100)).multiply(withdraw.getAmount()).setScale(2, RoundingMode.HALF_UP);
        }else{
            cost = channelWithdrawVO.getWithdrawHandlingCharge();
        }
        rlWithdraw.setCost(cost);
        rlWithdraw.setWithdrawChannelId(withdraw.getWithdrawChannelId());
        BigDecimal handlingCharge;
        if(rlUserExt.getHandlingChargeType().equals(ServiceConstants.HANDLING_CHARGE_TYPE_PERCENT)){
            BigDecimal handlingCharge1 = rlUserExt.getWithdrawHandlingCharge();// 5.4
            BigDecimal bigDecimal = handlingCharge1.divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);// 0.0054
            handlingCharge = bigDecimal.multiply(withdraw.getAmount()).setScale(2, RoundingMode.HALF_UP);// 0.0054 * 1000 = 0.54
        }else{
            handlingCharge = rlUserExt.getWithdrawHandlingCharge();
        }
        rlWithdraw.setHandlingCharge(handlingCharge);
        rlWithdraw.setHandlingChargeType(rlUserExt.getHandlingChargeType());
        rlWithdraw.setWithdrawHandlingCharge(rlUserExt.getWithdrawHandlingCharge());
        int i = rlWithdrawService.insertRlWithdraw(rlWithdraw);
        if(i == 0){
            throw new ServiceException("新增记录提现记录失败");
        }
    }

    private void asynchronousMsg(String userName) {
        scheduledExecutorService.schedule(() -> {
            /** 提现申请发送一个短信给审核人*/
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", userName);

            SendSmsVo sendSmsVo = new SendSmsVo();
            sendSmsVo.setMobile(phone);
            sendSmsVo.setTemplateCode(templateCode3);
            sendSmsVo.setParam(jsonObject.toJSONString());
            sendSmsVo.setSignName(signName);
            SendSmsResponse response = null;
            log.info("【提现申请】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
            try {
                response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
            } catch (ClientException e) {
                e.printStackTrace();
            }
            log.info("【提现申请】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));

            // 短信日志
            RlMsgLog etMsgLog = new RlMsgLog();
            etMsgLog.setPhone(phone);
            etMsgLog.setContent("用户【"+userName+"】，发起押金提现申请，请尽快审核！");
            etMsgLog.setType("2");
            etMsgLog.setSignName(signName);
            etMsgLog.setTemplateCode(templateCode3);
            int i = msgLogService.insertEtMsgLog(etMsgLog);
            if(i>0){
                log.info("【提现申请】短信日志记录成功");
            }else{
                log.info("【提现申请】短信日志记录失败");
            }
        }, 0 , TimeUnit.HOURS);
    }

    /**
     * 新增用户岗位信息
     *
     * @param user 用户对象
     */
    public void insertUserPost(RlUser user)
    {
        Long[] posts = user.getPostIds();
        if (StringUtils.isNotEmpty(posts))
        {
            // 新增用户与岗位管理
            List<SysUserPost> list = new ArrayList<SysUserPost>(posts.length);
            for (Long postId : posts)
            {
                SysUserPost up = new SysUserPost();
                up.setUserId(user.getUserId());
                up.setPostId(postId);
                list.add(up);
            }
            userPostMapper.batchUserPost(list);
        }
    }

    /**
     * 新增用户角色信息
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(Long userId, Long[] roleIds)
    {
        if (StringUtils.isNotEmpty(roleIds))
        {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>(roleIds.length);
            for (Long roleId : roleIds)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            userRoleMapper.batchUserRole(list);
        }
    }

}
