package com.ruoyi.system.service.store.impl;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.bean.collection.CollectionUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.system.domain.model.RlModelVO;
import com.ruoyi.system.domain.modelStore.RlModelStore;
import com.ruoyi.system.domain.store.Store;
import com.ruoyi.system.domain.store.StoreCountVO;
import com.ruoyi.system.domain.store.StoreQuery;
import com.ruoyi.system.domain.store.StoreVo;
import com.ruoyi.system.domain.store.enums.StoreGroupBy;
import com.ruoyi.system.domain.userExt.RlUserExt;
import com.ruoyi.system.mapper.StoreMapper;
import com.ruoyi.system.service.*;
import com.ruoyi.system.service.store.RlStoreService;
import com.ruoyi.system.service.store.StoreAssembler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 店铺Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
@Slf4j
public class StoreServiceImpl implements RlStoreService
{
    @Resource
    private StoreMapper storeMapper;

    @Autowired
    private IRlDeviceService deviceService;

    @Autowired
    private StoreAssembler storeAssembler;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Resource
    private IRlUserExtService userExtService;

//    @Autowired
//    private StoreApplyService storeApplyService;

//    @Autowired
//    private StoreApplyConverter storeApplyConverter;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private IRlModelService modelService;

    @Autowired
    private IRlModelStoreService modelStoreService;

    @Resource
    private IRlUserService userService;

    /**
     * 查询店铺
     *
     * @param storeId 店铺主键
     * @return 店铺
     */
    @Override
    public StoreVo selectSmStoreById(Long storeId)
    {
        StoreVo store = storeMapper.selectSmStoreById(storeId);
        if (store != null) {
            // 获取可租车辆数量
            Integer rentalCount = deviceService.selectRentalDeviceCountByStoreId(storeId,null);
            store.setRentalCar(rentalCount);

            // 获取车型列表
            List<RlModelVO> modelList = modelService.selectEModelListByStoreId(storeId);
            // 遍历
            for (RlModelVO model : modelList) {
                // 获取车型对应的车辆数量
                Integer modelRentalCount = deviceService.selectRentalDeviceCountByStoreId(storeId,model.getModelId());
                model.setRentalCar(modelRentalCount);
            }
            store.setModels(modelList);
        }
        return store;
    }

    /**
     * 查询店铺列表
     *
     * @param store 店铺
     * @return 店铺
     */
    @Override
    public List<StoreVo> selectSmStoreList(StoreQuery store)
    {
        List<StoreVo> storeVos = storeMapper.selectSmStoreList(store);
        for (StoreVo storeVo:storeVos) {
            BigDecimal lng = storeVo.getLng();
            BigDecimal lat = storeVo.getLat();
            if (lng != null && lat != null && StrUtil.isNotBlank(store.getPhoneLon()) && StrUtil.isNotBlank(store.getPhoneLat())) {
                double[] point1 = {lng.doubleValue(), lat.doubleValue()};
                double[] point2 = {Double.valueOf(store.getPhoneLon()), Double.valueOf(store.getPhoneLat())};
                double distance = GeoUtils.haversineDistance(point1, point2);
                BigDecimal distanceFormatted = new BigDecimal(distance).setScale(1, RoundingMode.HALF_UP);
                storeVo.setDistance(distanceFormatted.doubleValue());
            }
            Integer allDeviceCount = deviceService.selectAllDeviceCountByStoreId(storeVo.getStoreId());
            storeVo.setDeviceCount(allDeviceCount);// 所有设备数
            // todo 统计
            storeVo.setDailyRevenue(new BigDecimal("861"));// 日营收
            storeVo.setMonthlyRevenue(new BigDecimal("8222"));//月营收
            storeVo.setLastMonthlyRevenue(new BigDecimal("7545"));//上月营收
            RlUserExt rlUserExt = userExtService.selectRlUserExtByUserId(SecurityUtils.getUserId());
            storeVo.setDividendProportion(rlUserExt.getDividendProportion());// 分红比例
        }
        return storeVos;
    }


    /**
     * 根据代理商id查询店铺列表
     *
     * @param agentId 店铺
     * @return 店铺
     */
    @Override
    public List<StoreVo> selectSmStoreListByAgentId(Long agentId)
    {
        StoreQuery storeQuery = new StoreQuery();
        storeQuery.setAgentId(agentId);
        List<StoreVo> storeVos = storeMapper.selectSmStoreList(storeQuery);
        return storeVos;
    }

    /**
     * 根据商户查询店铺列表列表
     *
     * @param merchantId 商户id
     * @return 商户集合
     */
    @Override
    public List<StoreVo> selectSmStoreListBYMerchantId(Long merchantId)
    {
        List<StoreVo> storeVos = storeMapper.selectSmStoreListBYMerchantId(merchantId);
        return storeVos;
    }


    /**
     * 查询所有店铺列表
     *
     * @return 店铺
     */
    @Override
    public List<StoreVo> getAllStores()
    {
        StoreQuery query = new StoreQuery();
        return storeMapper.selectSmStoreList(query);
    }


    /**
     * 新增店铺
     *
     * @param store 店铺
     * @return 结果
     */
    @Override
    public int insertSmStore(Store store)
    {
        store.setCreateTime(DateUtils.getNowDate());
        store.setCreateBy(SecurityUtils.getUsername());

        Integer result = transactionTemplate.execute(status -> {
            bandModel(store);
            // 新增店铺
            int i = storeMapper.insertSmStore(store);
            ServiceUtil.assertion(i != 1, "新增店铺失败");

            // 如果没有默认店铺，则将其设为默认店铺
            if (this.selectDefaultStore(store.getMerchantId()) == null) {
                this.setDefault(store.getMerchantId(), store.getStoreId());
            }

            return i;
        });
        return result == null ? 0 : result;
    }

    private void bandModel(Store store) {
        /*关联车型*/
        List<Long> models = store.getModelIds();
        if (models != null && models.size() > 0) {
            for(Long modelId:models){
                RlModelStore rlModelStore = new RlModelStore();
                rlModelStore.setStoreId(store.getStoreId());
                rlModelStore.setModeId(modelId);
                int i = modelStoreService.insertRlModelStore(rlModelStore);
                if(i == 0){
                    throw new ServiceException("关联车型失败");
                }
            }
        }
    }

    /**
     * 修改店铺
     *
     * @param store 店铺
     * @return 结果
     */
    @Override
    public int updateSmStore(Store store)
    {
        Boolean execute = transactionTemplate.execute(e -> {
            store.setUpdateTime(DateUtils.getNowDate());
            store.setUpdateBy(SecurityUtils.getUsername());
            Long userId = store.getMerchantId();
            /** 修改用户类型 */
            RlUser rlUser = new RlUser();
            rlUser.setUserId(userId);
            rlUser.setUserType(ServiceConstants.USER_TYPE_MERCHANT);
            int i1 = userService.updateUserStatus(rlUser);
            if(i1 == 0){
                throw new ServiceException("【修改商户】失败");
            }
            /** 保存代理商*/
            RlUserExt userExt = new RlUserExt();
            userExt.setUserId(userId);
            userExt.setAgentId(store.getAgentId());
            int i2 = userExtService.updateRlUserExt(userExt);
            ServiceUtil.assertion(i2 != 1, "【修改商户】失败");
            /*关联车型*/
            int i = modelStoreService.deleteRlModelStoreByStoreId(store.getStoreId());
            bandModel(store);
            int i3 = storeMapper.updateSmStore(store);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("修改店铺失败");
        return 1;
    }

    @Override
    @Transactional
    public boolean changeSort(List<Store> list) {
        if (list == null) {
            return true;
        }
        List<Store> updateList = new ArrayList<>();
        for (Store item : list) {
            Store store = new Store();
            store.setStoreId(item.getStoreId());
            store.setGroupSort(item.getGroupSort());
            updateList.add(store);
        }
        storeMapper.batchUpdate(updateList);
        return true;
    }

    @Override
    public List<StoreVo> listCount(StoreQuery query) {
        List<StoreVo> list = new ArrayList<>();

        // 查询各各店铺的设备数据
        List<StoreVo> storeList = storeMapper.selectSmStoreList(query);
        storeAssembler.assembleDeviceCount(storeList);

//        // 查询全部设备数据
//        DeviceQuery deviceDto = new DeviceQuery();
//        deviceDto.setUserId(query.getUserId());
//        int count = smDeviceService.selectCount(deviceDto);
        StoreVo all = new StoreVo();
//        all.setDeviceCount(count);
        all.setName("全部");

        list.add(all);
        list.addAll(storeList);

        return list;
    }

    @Override
    public StoreVo selectDefaultStore(Long userId) {
        if (userId == null) {
            return null;
        }
        StoreQuery dto = new StoreQuery();
        dto.setIsDefault(true);
        dto.setMerchantId(userId);
        List<StoreVo> list = selectSmStoreList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }


    /**
     * ids逻辑删除
     *
     * @param ids id列表
     */
    @Override
    public int logicDel(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            log.warn("ids is empty");
            return 0;
        }
        Integer result = transactionTemplate.execute(status -> {
            // 若删除的包含默认店铺则将默认店铺设置为第一个店铺
            StoreQuery query = new StoreQuery();
            query.setStoreIds(ids);
            query.setIsDefault(true);
            List<StoreVo> defaultList = selectSmStoreList(query);
            if (!CollectionUtils.isEmptyElement(defaultList)) {
                // 包含默认店铺，则查询第一个店铺
                query = new StoreQuery();
                query.setMerchantId(defaultList.get(0).getMerchantId());
                StoreVo other = storeMapper.selectOne(query);
                if (other != null) {
                    // 其他店铺不为空，则设置为第一个店铺
                    this.setDefault(other.getMerchantId(), other.getStoreId());
                }
            }

            // 执行逻辑删除
            return storeMapper.logicDel(ids);
        });
        return result == null ? 0 : result;
    }

    /**
     * 查询附近的店铺
     *
     * @param query
     * @return
     */
    @Override
    public List<StoreVo> listNearBy(StoreQuery query) {
        if (query == null) {
            return Collections.emptyList();
        }
        if (CollectionUtils.isEmpty(query.getCenter())) {
            return Collections.emptyList();
        }
        return this.selectSmStoreList(query);
    }

    /**
     * ids查询查询列表
     *
     * @param storeIds
     * @return
     */
    @Override
    public List<StoreVo> selectStoreByIds(List<Long> storeIds) {
        if (CollectionUtils.isEmpty(storeIds)) {
            return Collections.emptyList();
        }
        return storeMapper.selectSmStoreByIds(storeIds);
    }

    /**
     * 查询数量
     *
     * @param query 查询条件
     */
    @Override
    public int selectCount(StoreQuery query) {
        return storeMapper.selectCount(query);
    }

    /**
     * 查询用户ID列表
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, Integer> selectCountMapByUserIds(List<Long> userIds) {
        StoreQuery query = new StoreQuery();
        query.setUserIds(userIds);
        query.setGroupBy(StoreGroupBy.user_id.name());
        return this.selectCountMap(query, StoreCountVO::getUserId);
    }

    /**
     * 设为默认店铺
     *
     * @param userId
     * @param storeId
     * @return
     */
    @Override
    public boolean setDefault(Long userId, Long storeId) {
        storeMapper.setDefault(userId, storeId);
        return true;
    }

    /**
     * 查询店铺映射表
     *
     * @param query
     * @param keyMapper
     */
    @Override
    public <K> Map<K, StoreVo> selectMap(StoreQuery query, Function<? super StoreVo, ? extends K> keyMapper) {
        List<StoreVo> list = this.selectSmStoreList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }

    /**
     * ids查询列表
     *
     * @param ids
     * @param userId
     */
    @Override
    public List<StoreVo> selectSmStoreListByIds(List<Long> ids, Long userId) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        StoreQuery query = new StoreQuery();
        query.setStoreIds(ids);
        query.setMerchantId(userId);
        return selectSmStoreList(query);
    }

    @Override
    public List<StoreVo> selectByUserId(Long userId) {
        if (userId == null) {
            return Collections.emptyList();
        }
        StoreQuery query = new StoreQuery();
        query.setMerchantId(userId);
        return storeMapper.selectSmStoreList(query);
    }

//    @Override
//    public int insertAndApply(StoreBO store) {
//        store.setStatus(StoreStatus.APPROVING.getStatus());
//        store.setEnabled(false);
//
//        Integer result = transactionTemplate.execute(status -> {
//            int insert = this.insertSmStore(store);
//            ServiceUtil.assertion(insert != 1, "新增店铺失败");
//
////            int insertApply = storeApplyService.insertStoreApply(storeApplyConverter.toPo(store, SecurityUtils.getUserId(), StoreApplyType.ADD));
//            ServiceUtil.assertion(insertApply != 1, "提交审核失败");
//            return insertApply;
//        });
//
//        return result == null ? 0 : result;
//    }

//    @Override
//    public int updateApply(StoreBO data) {
//        Integer result = transactionTemplate.execute(status -> {
//            // 修改店铺状态为审核中
//            Store updateData = new Store();
//            updateData.setStatus(StoreStatus.APPROVING.getStatus());
//            StoreQuery updateQuery = new StoreQuery();
//            updateQuery.setStoreId(data.getStoreId());
//            updateQuery.setStatus(StoreStatus.NORMAL.getStatus());
//            int update = this.updateByQuery(updateData, updateQuery);
//            ServiceUtil.assertion(update != 1, "店铺状态已发生变化，请刷新后重试");
//
//            // 新增申请
//            int insert = storeApplyService.insertStoreApply(storeApplyConverter.toPo(data, SecurityUtils.getUserId(), StoreApplyType.UPDATE));
//            ServiceUtil.assertion(insert != 1, "创建申请失败");
//
//            return insert;
//        });
//
//        return result == null ? 0 : result;
//    }

    @Override
    public int updateByQuery(Store data, StoreQuery query) {
        if (query == null) {
            return 0;
        }
        return storeMapper.updateByQuery(data, query);
    }

    /**
     * 通用查询数量
     *
     * @param query
     * @param keyMapper
     * @return
     */
    private <K> Map<K, Integer> selectCountMap(StoreQuery query, Function<? super StoreCountVO, ? extends K > keyMapper) {
        List<StoreCountVO> list = storeMapper.selectCommonCount(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, StoreCountVO::getCount));
    }


    /**
     * 根据定位获取附近店铺列表
     * 车型为空的过滤
     * 距离太远的过滤
     */
    @Override
    public List<StoreVo> getStoreListByLocation(StoreQuery query) {
        BigDecimal radiusFromQuery = query.getRadius();
        double radiusInMeters = (radiusFromQuery != null) ? radiusFromQuery.doubleValue() : Double.parseDouble(sysConfigService.selectConfigByKey("nearby.store")) * 1000;
        log.info("搜索附近【{}】米范围分润店铺", radiusInMeters);
        // 根据定位获取附近方圆X公里的店铺列表
        double userLon = Double.parseDouble(query.getPhoneLon());
        double userLat = Double.parseDouble(query.getPhoneLat());
        // 获取所有店铺的列表
        List<StoreVo> allStores = storeMapper.selectSmStoreList(query);
        // 过滤出在方圆X公里内的店铺
        List<StoreVo> nearbyStores = allStores.stream()
                .map(store -> {
                    List<RlModelVO> list = modelService.selectEModelListByStoreId(store.getStoreId());
                    log.info("店铺【{}】的车型列表：{}", store.getStoreId(), list);
                    // 如果 list 为空或为 null，直接返回 null
                    if (list == null || list.isEmpty()) {
                        return null;
                    }
                    store.setModels(list);
                    /** 日均  */
                    // 计算最低价格
                    BigDecimal minPrice = list.stream()
                            .map(RlModelVO::getPrice) // 获取每个模型的价格
                            .filter(Objects::nonNull) // 过滤掉可能为 null 的价格
                            .min(BigDecimal::compareTo) // 找到最小值
                            .orElse(BigDecimal.ZERO); // 如果列表为空，返回 BigDecimal.ZERO
                    store.setBottomPrice(minPrice);
                    /** 多少辆可租 */
                    Integer integer = deviceService.selectRentalDeviceCountByStoreId(store.getStoreId(),null);
                    store.setRentalCar(integer);
                    double storeLon = store.getLng().doubleValue();
                    double storeLat = store.getLat().doubleValue();
                    double distance = GeoUtils.haversineDistance(
                            new double[]{userLon, userLat},
                            new double[]{storeLon, storeLat}
                    ); // 转换为公里
                    double formattedDistance = Math.round(distance * 10) / 10.0;
                    store.setDistance(formattedDistance); // 设置距离
                    return store; // 返回更新后的 store
                })
                .filter(Objects::nonNull) // 过滤掉为 null 的 store 对象
                .filter(store -> store.getRentalCar() > 0) // 过滤掉租赁车辆数量为 0 的店铺
                .filter(store -> store.getDistance() <= radiusInMeters) // 过滤距离
                .sorted(Comparator.comparingDouble(StoreVo::getDistance)) // 根据距离排序
                .collect(Collectors.toList());
        return nearbyStores;
    }

    /**
     * 计算距离
     */
    @Override
    public BigDecimal computeDistance(String lon1,String lat1,Long storeId) {
        StoreVo storeVo = storeMapper.selectSmStoreById(storeId);
        if(storeVo == null){
            throw new ServiceException("店铺不存在");
        }
        BigDecimal lng2 = storeVo.getLng();
        BigDecimal lat2 = storeVo.getLat();
        if (lng2 != null && lat2 != null) {
            double[] point1 = {lng2.doubleValue(), lat2.doubleValue()};
            double[] point2 = {Double.valueOf(lon1), Double.valueOf(lat1)};
            double distance = GeoUtils.haversineDistance(point1, point2);
            BigDecimal distanceFormatted = new BigDecimal(distance).setScale(1, RoundingMode.HALF_UP);
            return distanceFormatted;
        }
        return null;
    }

}
