package com.ruoyi.system.task;

import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.service.IRlOrderOperService;
import com.ruoyi.system.service.IRlOrderService;
import com.ruoyi.system.service.IWxPayService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 定时任务调度测试
 *
 * @author ruoyi
 */
@Slf4j
@Component("rlTask")
public class RlTask {


    @Autowired
    private IWxPayService wxPayService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlOrderOperService orderOperService;



    /**
     * 1.启动时判断是否有未取消预约的订单
     * 2.判断已完成的订单未退还押金的
     * 3.启动时判断是否分账
     */
    @Transactional
    @PostConstruct
    public void init() {
    }


    /**
     *  到过期时间后改变订单状态
     *  处理过期订单
     */
    public void handleExpiredOrder(){
        log.info("-------------------【定时任务】处理过期订单---开始----------------");
        RlOrderQuery rlOrderQuery = new RlOrderQuery();
        rlOrderQuery.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        List<RlOrderVO> rlOrderVOS = orderService.selectAllRlOrderList(rlOrderQuery);
        for (RlOrderVO orderVO:rlOrderVOS) {
            // 检查autoCancelTime是否已经过了当前时间
            if (!orderVO.getIsOverdue() && orderVO.getAutoCancelTime().before(new Date())) {
                RlOrder rlOrder = new RlOrder();
                rlOrder.setOrderId(orderVO.getOrderId());
                rlOrder.setStatus(ServiceConstants.ORDER_STATUS_AUTO_CANCEL);
                int i = orderService.updateRlOrder(rlOrder);
                if(i>0){
                    if(!orderOperService.recordOrderHistory(orderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_SYSTEM_CANCEL,
                            orderVO.getStatus(),rlOrder.getStatus(),orderVO.getPayFee(),orderVO.getPayFee(),orderVO.getUserId(),orderVO.getPhone(),"超时系统自动取消")){
                        throw new ServiceException("【记录订单履历--超时系统自动取消失败】");
                    }
                }
            }
        }
        log.info("-------------------【定时任务】处理过期订单---结束----------------");
    }

    /**
     *  超时过期时间的订单增加一个超时标记
     *  处理过期订单
     */
    public void expiredMarkers(){
        log.info("-------------------【定时任务】超时标记---开始----------------");
        RlOrderQuery rlOrderQuery = new RlOrderQuery();
        rlOrderQuery.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
        List<RlOrderVO> rlOrderVOS = orderService.selectAllRlOrderList(rlOrderQuery);
        for (RlOrderVO orderVO:rlOrderVOS) {
            // 判断订单是否已过期，如果过期了，则标记为已过期
            if (orderVO.getExpiryTime().before(new Date())) {
                RlOrder rlOrder = new RlOrder();
                rlOrder.setOrderId(orderVO.getOrderId());
                rlOrder.setIsOverdue(true);
                orderService.updateRlOrder(rlOrder);
            }
        }
        log.info("-------------------【定时任务】超时标记---结束----------------");
    }

}
